#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Defines unit tests for :mod:`colour.notation.munsell` module.
"""

from __future__ import division, unicode_literals

import numpy as np
import unittest

from colour.notation.munsell import (
    parse_munsell_colour,
    is_grey_munsell_colour,
    normalize_munsell_specification)
from colour.notation.munsell import (
    munsell_colour_to_munsell_specification,
    munsell_specification_to_munsell_colour)
from colour.notation.munsell import (
    xyY_from_renotation,
    is_specification_in_renotation)
from colour.notation.munsell import bounding_hues_from_renotation
from colour.notation.munsell import hue_to_hue_angle, hue_angle_to_hue
from colour.notation.munsell import hue_to_ASTM_hue
from colour.notation.munsell import (
    interpolation_method_from_renotation_ovoid,
    xy_from_renotation_ovoid)
from colour.notation.munsell import LCHab_to_munsell_specification
from colour.notation.munsell import maximum_chroma_from_renotation
from colour.notation.munsell import munsell_specification_to_xy
from colour.notation.munsell import (
    munsell_specification_to_xyY,
    xyY_to_munsell_specification)
from colour.notation import (
    munsell_value_Priest1920,
    munsell_value_Munsell1933,
    munsell_value_Moon1943,
    munsell_value_Saunderson1944,
    munsell_value_Ladd1955,
    munsell_value_McCamy1987,
    munsell_value_ASTMD153508)
from colour.utilities import ignore_numpy_errors

__author__ = 'Colour Developers'
__copyright__ = 'Copyright (C) 2013 - 2015 - Colour Developers'
__license__ = 'New BSD License - http://opensource.org/licenses/BSD-3-Clause'
__maintainer__ = 'Colour Developers'
__email__ = 'colour-science@googlegroups.com'
__status__ = 'Production'

__all__ = ['MUNSELL_SPECIFICATIONS',
           'MUNSELL_GREYS_SPECIFICATIONS',
           'MUNSELL_EVEN_SPECIFICATIONS',
           'MUNSELL_BOUNDING_HUES',
           'MUNSELL_HUE_TO_ANGLE',
           'MUNSELL_HUE_TO_ASTM_HUE',
           'MUNSELL_INTERPOLATION_METHODS',
           'MUNSELL_XY_FROM_RENOTATION_OVOID',
           'MUNSELL_SPECIFICATIONS_TO_XY',
           'MUNSELL_COLOURS_TO_XYY',
           'MUNSELL_GREYS_TO_XYY',
           'XYY_TO_MUNSELL_SPECIFICATIONS',
           'XYY_TO_MUNSELL_GREYS_SPECIFICATIONS',
           'NON_CONVERGING_XYY',
           'TestMunsellValuePriest1920',
           'TestMunsellValueMunsell1933',
           'TestMunsellValueMoon1943',
           'TestMunsellValueSaunderson1944',
           'TestMunsellValueLadd1955',
           'TestMunsellValueMcCamy1992',
           'TestMunsellValueASTMD153508',
           'TestMunsellSpecification_to_xyY',
           'TestMunsellColour_to_xyY',
           'TestxyY_to_munsell_specification',
           'TestxyY_to_munsell_colour',
           'TestParseMunsellColour',
           'TestIsGreyMunsellColour',
           'TestNormalizeMunsellSpecification',
           'TestMunsellColourToMunsellSpecification',
           'TestMunsellSpecificationToMunsellColour',
           'Test_xyY_fromRenotation',
           'TestIsSpecificationInRenotation',
           'TestBoundingHuesFromRenotation',
           'TestHueToHueAngle',
           'TestHueAngleToHue',
           'TestHueTo_ASTM_hue',
           'TestInterpolationMethodFromRenotationOvoid',
           'Test_xy_fromRenotationOvoid',
           'TestLCHabToMunsellSpecification',
           'TestMaximumChromaFromRenotation',
           'TestMunsellSpecification_to_xy']

# TODO: Investigate if tests can be simplified by using a common valid set of
# specifications.
MUNSELL_SPECIFICATIONS = (
    (2.5000000000000000, 7.9653798470827155, 11.9285463083509686, 4),
    (2.5000000000000000, 6.1977948220908790, 6.9236108262088836, 4),
    (2.5000000000000000, 5.3119569782567533, 2.0000000000000000, 4),
    (5.6130070624423842, 8.4027565380707916, 18.5659089404439115, 4),
    (5.8456400710049072, 8.0626386645201364, 5.7823256145522954, 4),
    (5.7807941210595990, 3.1748040810258358, 3.3492086825591487, 4),
    (5.4836842996391173, 3.8994120994080133, 5.7614590625067148, 4),
    (5.8095803088134961, 5.8169751438995121, 6.6626137539588992, 4),
    (5.2092529556629028, 2.9770364483569107, 5.1414726438100136, 4),
    (7.7061058539115734, 2.7899422016542408, 11.3966488972748969, 4),
    (7.5675942867463615, 9.5693782641549276, 16.7149188607744144, 4),
    (8.1176405645643435, 2.7489429651492028, 3.1653563832640272, 4),
    (7.8731203012311255, 2.6438472620092806, 13.2411079692977136, 4),
    (8.0498332221428903, 2.4630649870973422, 7.5019246790810632, 4),
    (8.3553075693910621, 2.7032422741986490, 11.9254413443363916, 4),
    (8.3427957605776086, 1.0627446691234035, 6.2988181459092560, 4),
    (7.5947244020062845, 1.5750745121803325, 4.6266131353312874, 4),
    (8.1951778660857908, 8.7325043135138642, 23.5711220101815080, 4),
    (7.7547636349124689, 8.4372061378255854, 21.0094490106106804, 4),
    (9.0102319629788621, 6.1312711883866395, 6.8033705689301751, 4),
    (9.0415668516516217, 6.4540531985593965, 17.0100372035664478, 4),
    (9.9156521698279132, 8.5643879767914601, 11.1310821598843201, 4),
    (10.0000000000000000, 8.6514703493413077, 27.3220461867991027, 4),
    (9.9613361115981434, 8.0396827392235242, 13.2000986334405592, 4),
    (9.8874065510631812, 8.3213426539871840, 2.0660963235598375, 4),
    (10.0000000000000000, 3.4007871217870842, 2.5700932200974145, 4),
    (10.0000000000000000, 3.0639156094536428, 13.5140666071695144, 4),
    (10.0000000000000000, 5.4614654917981493, 12.7538997749639886, 4),
    (10.0000000000000000, 5.9008140948605901, 15.2445982768494179, 4),
    (10.0000000000000000, 5.4222087054147545, 27.9290010198770950, 4),
    (9.7570396457430526, 5.6536474118724431, 3.4112871270786895, 4),
    (10.0000000000000000, 5.7903571340714244, 24.8636013065843109, 4),
    (9.8620758176293215, 4.4878642136718669, 7.6719680950003797, 4),
    (3.2140937198013564, 9.3451635951997183, 3.4367939376082868, 3),
    (3.4840057595993792, 9.5721189585529416, 14.9050794241396130, 3),
    (3.1967035260607033, 9.0595733766045878, 24.7800313890532884, 3),
    (2.5000000000000000, 9.4791299568422183, 27.7365817049776346, 3),
    (2.7908763449337677, 8.1660999219462784, 20.8683045640276035, 3),
    (3.2214995668974771, 5.5077419206642650, 5.4677262571376586, 3),
    (2.6225120704322471, 5.9893806523738169, 19.3644722529733038, 3),
    (3.2873061024849806, 5.4398925249339651, 19.8557241925879140, 3),
    (5.7276124050033674, 3.0132953274578180, 10.7466425521665023, 3),
    (5.3479557011490932, 3.0035377095038158, 18.9004718151949049, 3),
    (5.7385751713204325, 3.9875599935298509, 4.2231608377596563, 3),
    (5.7208241035815108, 1.8040375230431649, 4.8780681593635187, 3),
    (5.3167800244843564, 1.0305080135789524, 8.0439576065413636, 3),
    (5.7623230008312385, 1.6541934959363132, 9.5074117162556888, 3),
    (5.9855795053875953, 2.2109765673980277, 14.8034345271893475, 3),
    (5.4616196034207549, 2.8055682359374789, 6.6471547360970025, 3),
    (7.8382779261952082, 2.8050500161595604, 6.2385280252185922, 3),
    (8.2830613968174998, 2.7163438216736111, 10.3508251747691542, 3),
    (7.6031550323552723, 6.1394212951580345, 29.1395411651987040, 3),
    (8.3241150395279764, 6.9718015553038741, 23.5157789731952569, 3),
    (8.4442427312468595, 6.6574923053332222, 2.4130843113046656, 3),
    (8.3090617745210764, 6.3711907194545638, 17.5072521345144878, 3),
    (8.1403711706809201, 2.6868573867536836, 14.6499332953540424, 3),
    (8.4849035532136945, 2.2057045177976002, 11.8795622626339483, 3),
    (8.4541090296230159, 2.3630506284708144, 4.6063171733042516, 3),
    (8.3052624291689856, 5.4605355171827092, 3.9045072719017924, 3),
    (8.1897300045792871, 5.0699333987924406, 28.1269927592368632, 3),
    (7.5402877810747500, 5.7799956125476619, 6.6353191939359162, 3),
    (7.9629991342362985, 5.2335977013885158, 20.2933548056268656, 3),
    (8.4329595590383715, 5.7971283545076657, 26.4699708737570667, 3),
    (10.0000000000000000, 9.0051614847828851, 6.0469956581432704, 3),
    (9.7713539460569141, 9.3837598368299009, 20.8297527154788895, 3),
    (9.3763807965222234, 9.4604482045089409, 13.3485223941066824, 3),
    (9.9127041795322288, 4.0578049585768747, 25.7782317703519226, 3),
    (10.0000000000000000, 4.8536959640450510, 13.7122476433708371, 3),
    (10.0000000000000000, 4.2212112925094569, 28.5879233609310326, 3),
    (9.2875351467329246, 4.4042068687042750, 6.9973895652846254, 3),
    (10.0000000000000000, 5.7178974228675292, 30.9324350684787923, 3),
    (10.0000000000000000, 5.1210462428544776, 7.9468547464613932, 3),
    (10.0000000000000000, 5.6311865015719071, 26.1724102978957731, 3),
    (2.5000000000000000, 6.8222787673793750, 12.6434105570570861, 2),
    (2.5000000000000000, 3.3435596434006034, 19.1675377625573944, 2),
    (3.2845817745734109, 3.7457477655465423, 10.3167618622771258, 2),
    (3.0814075494281132, 3.3027890209934192, 4.0316837245147514, 2),
    (2.5000000000000000, 9.5952672227596540, 9.1364350412201212, 2),
    (2.5899169115530087, 9.5505578550805392, 8.2631333972333536, 2),
    (2.5342634625499727, 9.4942990746072660, 14.8636631042532183, 2),
    (5.2759205646620941, 9.0228201875137408, 12.8791359497697275, 2),
    (5.5228564491289642, 9.3877113963474379, 17.4125865956868147, 2),
    (5.8859149397779467, 9.1911190893689660, 17.3880868140724374, 2),
    (5.4717401116974616, 9.8688621878686380, 11.6468485388216667, 2),
    (5.9565603219671557, 4.1861233351978830, 4.3116902048143899, 2),
    (5.6279111948942635, 4.5472024297877738, 16.5668191444311503, 2),
    (5.8534547245334565, 4.5925997997392267, 18.8350698050853502, 2),
    (5.1447203696302557, 5.3185754864266883, 18.9791729668054074, 2),
    (5.2907074463880175, 6.0009909462768771, 13.5985209980560526, 2),
    (5.4158444031977657, 6.3980311109227372, 15.1786174644616256, 2),
    (8.2041448522882447, 5.9021079780772370, 4.0201776913722949, 2),
    (9.3660699534030183, 3.3728653869498273, 15.4227661827945788, 2),
    (10.0000000000000000, 3.9490817635970838, 9.1923876167058154, 2),
    (10.0000000000000000, 3.1874555799564490, 15.9542478936070324, 2),
    (9.2605862715376066, 3.4545177339210404, 10.5951757917016192, 2),
    (9.5716758646706186, 3.1497371248916179, 17.3988475313979336, 2),
    (3.2387393821759787, 4.8276509158647949, 3.7435106940988625, 1),
    (2.5000000000000000, 4.3022043540842603, 7.3993436144209168, 1),
    (2.5000000000000000, 4.3294709437986389, 8.8608404173678377, 1),
    (2.5000000000000000, 7.6200943276782551, 10.8872656168291240, 1),
    (2.5000000000000000, 7.1449996531857725, 10.1023353741859108, 1),
    (2.6104349455855846, 7.7009394890939928, 4.2361715150659922, 1),
    (2.5000000000000000, 8.5244556473474056, 5.3613636980274295, 1),
    (3.1731014606584806, 8.1336581464164190, 15.1995362353083028, 1),
    (2.5000000000000000, 7.1293721622530732, 5.4202608625739925, 1),
    (2.5000000000000000, 7.7085098502487703, 9.6193649384034430, 1),
    (3.2525815090531771, 7.0815325435574206, 6.2224060204343745, 1),
    (2.5000000000000000, 7.6755794494015603, 12.2618083975850567, 1),
    (2.5000000000000000, 3.4825807865537914, 7.7685055469176172, 1),
    (2.5000000000000000, 3.0207831579625881, 6.9988409117240948, 1),
    (3.0205621196907169, 3.1223174909201346, 5.2030875391050824, 1),
    (5.2190911687613255, 3.0070655951585925, 13.5738875509672745, 1),
    (5.5962506280473505, 2.1572825521633900, 5.1651068503657331, 1),
    (5.0785748388973584, 2.9637552645895053, 6.8599427244043705, 1),
    (5.1756171558445825, 2.7729517039066369, 4.5608003821410303, 1),
    (5.4973530207828443, 5.4105514189426884, 2.0000000000000000, 1),
    (5.8417735135440010, 5.6866676240854268, 13.2893656678185508, 1),
    (5.5805491854636680, 6.9641877356627768, 16.1803201492634017, 1),
    (5.2877727269225270, 6.8653966948539340, 14.0989464615804039, 1),
    (8.3582212856142686, 4.5915942564151919, 17.2715635972971029, 1),
    (7.8772447963597703, 4.7444381406648972, 5.5989343468594752, 1),
    (8.3233369535874786, 4.5668003762850411, 7.0881523668119195, 1),
    (7.8454860962996813, 4.5862707370177151, 16.2337951792823887, 1),
    (8.3825695023449427, 4.5622116440691229, 13.9751241108762905, 1),
    (7.8555937497823543, 3.2383503563015479, 5.3604358250617752, 1),
    (7.6555011537339137, 3.9039238810826622, 9.7695930479633919, 1),
    (7.6530191580084930, 6.3483962709336987, 11.7045897666252809, 1),
    (10.0000000000000000, 2.7176353295329094, 5.4158461678022469, 1),
    (9.1966481560049633, 8.1507829349934902, 5.0692233667592408, 1),
    (10.0000000000000000, 6.0406948226250909, 7.7628023164068498, 1),
    (10.0000000000000000, 6.7190177925216776, 18.3743753864025088, 1),
    (2.8739501345809000, 3.5100389001084373, 4.4945211069126740, 10),
    (2.9797638317158932, 8.5642374861117005, 6.4267107939641992, 10),
    (2.5000000000000000, 8.9248766467859824, 2.4912528414503781, 10),
    (2.5000000000000000, 8.1213521871194558, 8.8233798640361893, 10),
    (2.5000000000000000, 4.6431603939375377, 18.8393399778644905, 10),
    (2.5000000000000000, 4.9254430598361214, 5.4177118115989469, 10),
    (2.5000000000000000, 8.5093858827924329, 8.0453567253469096, 10),
    (2.5000000000000000, 2.7096473563856671, 16.1958101598068147, 10),
    (5.6678871626197305, 1.8444622064585485, 18.2260108117431834, 10),
    (5.7596738401992056, 1.9609725996843761, 30.4287315274152483, 10),
    (5.7836346614632728, 1.5360819708237339, 21.4801942145111369, 10),
    (5.1181732488629281, 1.5400563354602976, 41.8684733585788322, 10),
    (5.7573497243896670, 1.6383453350505301, 13.6096042678049560, 10),
    (5.2793040612960453, 4.9008406413604320, 22.8761275280486629, 10),
    (5.7157098010598082, 4.5703571087881230, 30.3602134880221577, 10),
    (5.9479473045208477, 4.2734225361802469, 4.8966439066197935, 10),
    (5.0989932248172396, 4.9475052272793167, 26.2687504247525787, 10),
    (5.5322294976298503, 4.6299108939644320, 7.7564492627214818, 10),
    (5.9235845417681920, 4.5932393967953056, 19.5676050308493856, 10),
    (5.9501563870301712, 2.4246349934363298, 4.9536669461614116, 10),
    (5.6141581365353224, 2.4812727587161407, 20.6449539043668935, 10),
    (5.4359081407306382, 2.7884847594702746, 21.5850643322003926, 10),
    (5.5399085613433288, 2.9864344023506266, 44.9036990399531604, 10),
    (5.3792514320991325, 2.1370360382654239, 25.8890745588287317, 10),
    (5.6329098306822463, 5.9349482115124506, 21.3840425068616966, 10),
    (5.2033265149329200, 5.8253671955490480, 15.5144674274224315, 10),
    (5.9277936921340721, 5.4480790503486123, 3.7766395197414253, 10),
    (5.8173223961875111, 5.2921858627166669, 11.3180415809075203, 10),
    (7.9499606335916067, 2.8737657312264488, 25.6213689020893334, 10),
    (8.3825924368107589, 2.4615704172167452, 40.5412719529260102, 10),
    (7.9637973633225698, 2.2001346713122278, 36.7073187099669482, 10),
    (8.3739244566104745, 2.3066883154384743, 8.6238460649901665, 10),
    (8.1519906864733880, 2.2622251239305577, 42.2291271964581441, 10),
    (8.2550276453260594, 9.6091828151923178, 7.0809860460282792, 10),
    (8.4883840852320755, 8.0985231119575776, 9.7796280723158073, 10),
    (8.4383570688761633, 2.6893452283620705, 26.8734524920740441, 10),
    (8.3094349065304414, 2.4623229011742396, 48.4996639934449902, 10),
    (7.7115794149655015, 2.7247286450173140, 5.7298598433541956, 10),
    (7.6273740879401934, 2.2251923932068416, 26.7249730707769224, 10),
    (7.6939233372260842, 2.6579274123978887, 48.4078975056904852, 10),
    (10.0000000000000000, 6.1974183910238620, 10.9719538159106609, 10),
    (9.1130972747403813, 6.2709966382451574, 2.7564645951736484, 10),
    (10.0000000000000000, 9.2352325807952376, 6.0033883251860249, 10),
    (10.0000000000000000, 5.0503674469973294, 19.1707567215596981, 10),
    (9.3801100887551563, 5.5356649305105154, 18.8175077437544154, 10),
    (9.0017959465770332, 7.7860618089167026, 4.4538545632120776, 10),
    (10.0000000000000000, 7.6920309563165672, 3.6531597236888560, 10),
    (9.0461828964214455, 3.0439259875156295, 22.3009468068498471, 10),
    (9.4594207963837835, 3.0372188559586917, 10.5525569494149547, 10),
    (10.0000000000000000, 3.3229506269252425, 31.2476220198124004, 10),
    (10.0000000000000000, 3.1004893435032645, 29.2734347311524985, 10),
    (2.5000000000000000, 7.9902138365557152, 8.3750743751782615, 9),
    (2.5000000000000000, 7.2983010691578754, 9.5028468626493314, 9),
    (2.8619005171223564, 7.2754260023179667, 7.4661261346289010, 9),
    (3.0874221941355513, 8.4850005613008470, 2.4938578293607869, 9),
    (2.5000000000000000, 3.6906678593666271, 19.7747167807561688, 9),
    (3.2205535070037539, 3.2815072105597061, 37.0593806627261628, 9),
    (2.5000000000000000, 3.8989428412499203, 39.1664185009443742, 9),
    (2.7654037016841957, 3.1169069187360945, 29.7265355691379369, 9),
    (2.5000000000000000, 3.7034489401910289, 12.0876546872501276, 9),
    (2.5000000000000000, 3.4331943859432581, 3.3828527595771778, 9),
    (2.8366121370807811, 3.9924265837199604, 2.0000000000000000, 9),
    (2.8888545547050946, 3.2474346036095905, 14.6183070378328566, 9),
    (5.1643993319905190, 6.2346627424063925, 9.1114653837439121, 9),
    (5.5003569030033876, 6.7368412399724260, 13.1541281319682977, 9),
    (5.5358100577424327, 6.9703425360344591, 8.8927166641344755, 9),
    (5.5900409663439943, 3.5668609688847175, 22.7566127868985504, 9),
    (5.2826202617433458, 3.2367340323019573, 18.7328236887543831, 9),
    (5.1728956401601813, 3.0043051231231956, 6.2292543458148515, 9),
    (5.2597218547319811, 3.3004333429874864, 35.8908721106814141, 9),
    (5.5536463415959245, 3.4948508349893386, 10.0766837095490551, 9),
    (5.7300039721591451, 2.4880341411732072, 15.9856983902699774, 9),
    (5.7823815169906521, 2.4812045413951833, 28.7746185183793024, 9),
    (5.0693797816654609, 6.7415333253524787, 2.2194841714206595, 9),
    (5.1346796709796605, 6.1031391336824816, 27.7263986439234174, 9),
    (5.3832606878646239, 5.5609978413428900, 18.3022959341279225, 9),
    (5.8697920884647008, 5.2333113793479047, 32.5534321679666334, 9),
    (5.4624511435406120, 5.7464718088999831, 30.9488646344402127, 9),
    (5.3574452696396984, 5.6885266719444099, 5.2614344690064048, 9),
    (5.6263734530030343, 5.7710036938275246, 25.1708466664452359, 9),
    (8.2842008951649930, 2.4660498194749279, 17.2388998041601766, 9),
    (8.3181027841240187, 2.2658035726687236, 22.5961473835359179, 9),
    (7.8519368662427134, 7.4522933534587796, 20.9623744079114580, 9),
    (8.1460813360327027, 7.7144059064726367, 13.5339629184693369, 9),
    (8.0972086431627499, 7.2473398419466069, 17.3389915505245398, 9),
    (7.8302562919917973, 6.8724169942694147, 10.7068221638259242, 9),
    (7.8006589706884801, 6.3306783238247419, 6.2113756808778051, 9),
    (8.0448636471186354, 6.8082263176114708, 15.5571552615442279, 9),
    (8.4617748029090709, 4.7459652528207172, 36.0372969397773204, 9),
    (7.6123828822072843, 4.3723674708923266, 14.1686907807062248, 9),
    (8.1696339276959975, 4.4883347380035703, 27.2358461038644108, 9),
    (9.6020311360157748, 5.5279706384135521, 20.5806356758181010, 9),
    (9.6636860301788179, 5.5169784631012053, 29.0476584729829561, 9),
    (9.7529285473647107, 5.4611625531978438, 34.1149316052812921, 9),
    (10.0000000000000000, 5.6504249041674308, 4.2162157304370860, 9),
    (10.0000000000000000, 5.7365436776659697, 34.7285267558391624, 9),
    (10.0000000000000000, 5.4360854849263855, 14.7796272948823670, 9),
    (10.0000000000000000, 5.7954415579527900, 2.0000000000000000, 9),
    (9.4970509139487298, 5.9141054791488150, 10.8088547800987307, 9),
    (9.8266351634655322, 1.9759992882300867, 7.0671144318498502, 9),
    (9.3825023503012588, 4.7097387178377552, 19.9994768774463623, 9),
    (9.1155305918192742, 4.9860253865670323, 5.8834364886948176, 9),
    (10.0000000000000000, 4.8130330158828309, 24.7458702329524449, 9),
    (9.3783595885807927, 4.5743768022516917, 26.2957872574229228, 9),
    (10.0000000000000000, 2.1709322459501545, 21.5725763566023510, 9),
    (10.0000000000000000, 2.5713046143569223, 26.0398724912355775, 9),
    (2.5000000000000000, 2.6357605512712707, 4.7121381662539816, 8),
    (2.8874578666829285, 2.0337681540970594, 13.9948960521457479, 8),
    (3.4354195604394651, 2.2299190864211247, 6.7189891135327322, 8),
    (2.9925336062737173, 1.9289335576450750, 7.1980143398663090, 8),
    (2.5000000000000000, 1.3726890604845965, 14.1567267100244649, 8),
    (2.6104579288975813, 1.2137704813997643, 3.3458156268951917, 8),
    (5.1670653045538115, 7.7615028403678448, 2.1409481568506346, 8),
    (5.0544341143469511, 7.0114569040639632, 6.4421573326031334, 8),
    (5.8037356824506121, 8.5129934544039099, 10.4438417735233937, 8),
    (5.0448775397799679, 6.3420360036696213, 18.4244287014075532, 8),
    (5.4848324026214836, 6.7395105985555626, 5.4747774912956473, 8),
    (5.1623004272002886, 6.5767221693498898, 24.9990562485251253, 8),
    (5.8772563607434130, 6.7897767911821179, 15.4504441432596611, 8),
    (8.1974490801098732, 2.2092984979309276, 2.0000000000000000, 8),
    (7.9972372657542374, 2.0603130944663230, 11.6558293358065175, 8),
    (7.9731925609071839, 8.6712830748870893, 4.2728868868791814, 8),
    (7.8364986461862207, 8.1687015261860942, 13.5966587179990253, 8),
    (7.7821869659085170, 9.2021935284645853, 13.9021055240679452, 8),
    (9.5317952667717609, 5.0377553779670317, 2.0000000000000000, 8),
    (10.0000000000000000, 5.4166121033139696, 11.0556249127789368, 8),
    (9.3122708373931626, 7.4662031204124188, 11.1852220991899731, 8),
    (10.0000000000000000, 7.0979058872703629, 13.8954559024466775, 8),
    (9.9256699400322717, 4.6921921662838253, 7.2040789887667955, 8),
    (9.4167408824024026, 4.6973687961211494, 8.7201163481804915, 8),
    (10.0000000000000000, 4.3385095147563364, 16.4696989109913723, 8),
    (10.0000000000000000, 6.4022012644562833, 6.5992372339473091, 8),
    (10.0000000000000000, 5.1822080733381393, 4.5502697844677806, 8),
    (9.9703325305196788, 5.9032095408122123, 10.8370227220876441, 8),
    (2.9627075871745849, 9.2513521634857003, 9.9991169316305388, 7),
    (3.1672052728994915, 9.1411346171540266, 7.3836247298929152, 7),
    (2.5000000000000000, 5.0498580894669791, 17.8815938530076153, 7),
    (2.7415018638966284, 5.6809766282284908, 18.0029087378013806, 7),
    (2.5000000000000000, 5.8481154189353175, 10.2326689962714923, 7),
    (2.8779022261852312, 5.5674143852975151, 3.5582034231201787, 7),
    (2.5000000000000000, 5.8534450733346004, 27.7799959269169712, 7),
    (5.4128217712844577, 2.5214549204115335, 7.2580400206056073, 7),
    (5.8375474760508403, 2.5302731816257218, 11.9982613806154710, 7),
    (5.9693975439749885, 4.3487706338488001, 14.3979064202833023, 7),
    (5.0040790005633813, 4.6572733453200046, 22.7366776144687748, 7),
    (5.1684384259452916, 4.2464127172076900, 4.8448605479076932, 7),
    (5.8632843152020939, 4.3591537966290641, 23.4897100232465128, 7),
    (5.7563333894119593, 8.9520112257136351, 7.3011356184221414, 7),
    (5.1083374030147883, 8.3115420243251794, 11.3597715314910968, 7),
    (8.3148984373785346, 9.1859535132810457, 4.2382336360058428, 7),
    (8.2014603996082265, 4.2309654154461391, 11.5898408445204275, 7),
    (7.5956049192734421, 4.8844511386513396, 6.7982657472219277, 7),
    (8.3781863618289165, 9.4848195822578312, 8.0223578906755613, 7),
    (8.0282362844647785, 9.7577016174440523, 11.5741982710620857, 7),
    (8.2292707621139733, 8.6917863535795146, 6.3500223969273417, 7),
    (10.0000000000000000, 3.3059509658558612, 3.1152259635487924, 7),
    (9.7562679983086813, 3.1863606517354883, 14.8033847219145844, 7),
    (10.0000000000000000, 3.5046891678155427, 13.9016096097173900, 7),
    (10.0000000000000000, 8.7841366291592120, 6.2184909658821841, 7),
    (10.0000000000000000, 8.3743452832613805, 13.8874930442766242, 7),
    (10.0000000000000000, 4.6140458786416998, 14.6890715994669296, 7),
    (10.0000000000000000, 8.0330373009170302, 13.5181723549434167, 7),
    (2.7455640547144746, 1.6521001852026693, 5.5691106735491642, 6),
    (3.1452880891491906, 5.1555158340566534, 8.5958327172909996, 6),
    (2.5000000000000000, 4.3890476613687266, 4.9506791516086910, 6),
    (2.5000000000000000, 4.3948638371895408, 4.3832312494231553, 6),
    (2.5000000000000000, 1.5580252510526358, 3.3072822748362349, 6),
    (5.0455832680055721, 8.6353345439035287, 9.5919452486024408, 6),
    (5.5942845260414558, 8.6320252698002999, 10.1972012381662864, 6),
    (5.9888024672139428, 8.1325318169145824, 12.3059519561692294, 6),
    (5.4258509473962517, 5.1854456006395786, 8.0461568627031124, 6),
    (5.3693642401195847, 5.0880777431684781, 7.3405738273399619, 6),
    (5.7020458215905094, 5.2717939849983750, 10.3256520517245409, 6),
    (5.4110963269588286, 5.5458983729698828, 5.2920348430950259, 6),
    (8.2429685366357628, 9.0824007428950111, 4.9002058653288101, 6),
    (8.0504264222588624, 9.7805379585063719, 18.9783397207514177, 6),
    (8.2387545704858169, 8.6024899113383668, 5.9413301103786500, 6),
    (8.3956842438974792, 4.5067364277363531, 9.4615159687151351, 6),
    (10.0000000000000000, 5.1387571364699527, 12.7049634856464984, 6),
    (10.0000000000000000, 5.1599126106312809, 15.6753707607593995, 6),
    (10.0000000000000000, 5.5494729651212173, 3.5065733883684942, 6),
    (10.0000000000000000, 5.7950904213307490, 14.0639228795685085, 6),
    (10.0000000000000000, 6.9831232345997147, 3.1284434139449528, 6),
    (10.0000000000000000, 6.6802047543668470, 11.6324059143146474, 6),
    (9.0502631824660114, 6.7218006479189771, 17.0836769427597908, 6),
    (10.0000000000000000, 6.0634616201345715, 4.7369669473269207, 6),
    (9.4094025438018623, 6.9442036306924901, 6.2876602116865898, 6),
    (9.6333946040069609, 7.5058275540068680, 4.6230440017025254, 6),
    (9.0207701922757479, 7.3138794160617016, 13.4222450145776442, 6),
    (9.2631760968615406, 7.3579949308718327, 15.2332951824776668, 6),
    (3.3327820263877230, 7.2256790897526173, 16.1134199776775375, 5),
    (2.5000000000000000, 5.4286631163584183, 6.5436496028361315, 5),
    (2.5000000000000000, 2.8290725241063579, 2.0000000000000000, 5),
    (2.8285591842433737, 8.7303908236239156, 21.4732588172908727, 5),
    (2.5000000000000000, 8.1701201003613502, 12.0201086586348378, 5),
    (2.5000000000000000, 8.7435404561839807, 14.4279044141537192, 5),
    (2.5000000000000000, 4.6389139628117171, 8.3802438034108171, 5),
    (3.3630794166715381, 4.6706516456254858, 2.7755096642090313, 5),
    (5.3390799626536243, 8.0640948231086753, 16.6115749394242549, 5),
    (5.3473567647815976, 8.4364176210146393, 15.4121651982320493, 5),
    (5.3689506096346218, 7.3716538071858944, 7.0381659199243058, 5),
    (5.9295528545359080, 6.8959269208164553, 7.5728134470480599, 5),
    (5.7279465595089096, 6.5816608478595349, 10.6681726339340361, 5),
    (5.6417821396686794, 6.4580191046930642, 9.5490168857451856, 5),
    (5.3443596420587474, 2.8710977581940789, 5.4304895609724859, 5),
    (7.7499092978023167, 4.3288327210550914, 4.2689337511750507, 5),
    (8.1454092289099975, 4.8650217144084049, 7.5456335290643839, 5),
    (7.9072536701593048, 5.6883950965465484, 10.7709862292896226, 5),
    (7.5925084922613122, 5.0989976044552208, 4.9335683444997134, 5),
    (7.6748726904108207, 5.4410490198888786, 3.5502452884794837, 5),
    (7.9919799870620540, 6.6162954836141061, 3.2837012487472252, 5),
    (9.3455991852868827, 7.2247365867351672, 17.4885217578818200, 5),
    (9.6595952185113880, 7.8995777767239241, 3.3572177484844636, 5))

MUNSELL_GREYS_SPECIFICATIONS = np.linspace(0, 10, 25)

MUNSELL_EVEN_SPECIFICATIONS = (
    (2.5, 5.0, 12.0, 4),
    (2.5, 5.0, 32.0, 4),
    (2.5, 5.0, 22.0, 4),
    (2.5, 5.0, 32.0, 4),
    (2.5, 6.0, 18.0, 4),
    (2.5, 6.0, 32.0, 4),
    (2.5, 6.0, 6.0, 4),
    (2.5, 5.0, 42.0, 4),
    (2.5, 5.0, 26.0, 4),
    (2.5, 5.0, 48.0, 4),
    (2.5, 2.0, 14.0, 4),
    (2.5, 2.0, 14.0, 4),
    (2.5, 0.0, 14.0, 4),
    (2.5, 0.0, 2.0, 4),
    (5.0, 1.0, 46.0, 4),
    (5.0, 1.0, 38.0, 4),
    (5.0, 1.0, 12.0, 4),
    (5.0, 1.0, 10.0, 4),
    (5.0, 4.0, 16.0, 4),
    (5.0, 2.0, 44.0, 4),
    (5.0, 7.0, 2.0, 4),
    (5.0, 7.0, 8.0, 4),
    (5.0, 7.0, 32.0, 4),
    (7.5, 2.0, 28.0, 4),
    (7.5, 2.0, 12.0, 4),
    (7.5, 2.0, 34.0, 4),
    (7.5, 4.0, 24.0, 4),
    (7.5, 4.0, 10.0, 4),
    (7.5, 4.0, 18.0, 4),
    (7.5, 9.0, 44.0, 4),
    (7.5, 5.0, 12.0, 4),
    (7.5, 5.0, 40.0, 4),
    (7.5, 5.0, 30.0, 4),
    (7.5, 5.0, 12.0, 4),
    (10.0, 3.0, 38.0, 4),
    (10.0, 3.0, 16.0, 4),
    (10.0, 3.0, 32.0, 4),
    (10.0, 3.0, 44.0, 4),
    (10.0, 3.0, 42.0, 4),
    (10.0, 3.0, 34.0, 4),
    (10.0, 3.0, 18.0, 4),
    (10.0, 7.0, 10.0, 4),
    (10.0, 7.0, 40.0, 4),
    (10.0, 7.0, 12.0, 4),
    (10.0, 6.0, 42.0, 4),
    (10.0, 6.0, 6.0, 4),
    (10.0, 4.0, 40.0, 4),
    (2.5, 7.0, 28.0, 3),
    (2.5, 7.0, 26.0, 3),
    (2.5, 9.0, 44.0, 3),
    (2.5, 9.0, 26.0, 3),
    (2.5, 0.0, 32.0, 3),
    (2.5, 0.0, 26.0, 3),
    (2.5, 8.0, 30.0, 3),
    (2.5, 8.0, 30.0, 3),
    (2.5, 8.0, 6.0, 3),
    (2.5, 6.0, 32.0, 3),
    (2.5, 6.0, 12.0, 3),
    (5.0, 7.0, 28.0, 3),
    (5.0, 7.0, 26.0, 3),
    (5.0, 7.0, 46.0, 3),
    (5.0, 7.0, 10.0, 3),
    (5.0, 6.0, 10.0, 3),
    (5.0, 6.0, 44.0, 3),
    (5.0, 1.0, 2.0, 3),
    (5.0, 9.0, 34.0, 3),
    (5.0, 9.0, 30.0, 3),
    (7.5, 3.0, 12.0, 3),
    (7.5, 7.0, 26.0, 3),
    (7.5, 7.0, 18.0, 3),
    (7.5, 7.0, 42.0, 3),
    (7.5, 7.0, 20.0, 3),
    (7.5, 7.0, 16.0, 3),
    (7.5, 3.0, 36.0, 3),
    (7.5, 3.0, 38.0, 3),
    (7.5, 3.0, 14.0, 3),
    (7.5, 2.0, 30.0, 3),
    (7.5, 2.0, 12.0, 3),
    (7.5, 2.0, 8.0, 3),
    (7.5, 2.0, 6.0, 3),
    (7.5, 6.0, 34.0, 3),
    (7.5, 6.0, 12.0, 3),
    (10.0, 4.0, 14.0, 3),
    (10.0, 4.0, 40.0, 3),
    (10.0, 5.0, 2.0, 3),
    (10.0, 5.0, 26.0, 3),
    (10.0, 6.0, 40.0, 3),
    (10.0, 6.0, 46.0, 3),
    (10.0, 6.0, 18.0, 3),
    (10.0, 6.0, 38.0, 3),
    (10.0, 3.0, 16.0, 3),
    (10.0, 3.0, 32.0, 3),
    (10.0, 3.0, 26.0, 3),
    (10.0, 3.0, 22.0, 3),
    (10.0, 8.0, 2.0, 3),
    (10.0, 8.0, 10.0, 3),
    (10.0, 8.0, 12.0, 3),
    (10.0, 8.0, 18.0, 3),
    (10.0, 8.0, 44.0, 3),
    (2.5, 8.0, 2.0, 2),
    (2.5, 8.0, 42.0, 2),
    (2.5, 7.0, 34.0, 2),
    (2.5, 4.0, 36.0, 2),
    (2.5, 4.0, 34.0, 2),
    (2.5, 4.0, 22.0, 2),
    (2.5, 0.0, 42.0, 2),
    (2.5, 0.0, 32.0, 2),
    (2.5, 1.0, 28.0, 2),
    (2.5, 1.0, 2.0, 2),
    (2.5, 1.0, 24.0, 2),
    (2.5, 1.0, 12.0, 2),
    (5.0, 5.0, 22.0, 2),
    (5.0, 5.0, 46.0, 2),
    (5.0, 5.0, 24.0, 2),
    (5.0, 1.0, 48.0, 2),
    (5.0, 1.0, 12.0, 2),
    (5.0, 1.0, 16.0, 2),
    (5.0, 1.0, 2.0, 2),
    (5.0, 1.0, 18.0, 2),
    (5.0, 8.0, 28.0, 2),
    (5.0, 8.0, 32.0, 2),
    (5.0, 8.0, 24.0, 2),
    (5.0, 8.0, 38.0, 2),
    (5.0, 2.0, 24.0, 2),
    (5.0, 2.0, 4.0, 2),
    (5.0, 2.0, 32.0, 2),
    (5.0, 2.0, 38.0, 2),
    (5.0, 9.0, 36.0, 2),
    (5.0, 9.0, 34.0, 2),
    (5.0, 9.0, 4.0, 2),
    (7.5, 7.0, 28.0, 2),
    (7.5, 7.0, 10.0, 2),
    (7.5, 7.0, 48.0, 2),
    (7.5, 9.0, 48.0, 2),
    (7.5, 9.0, 48.0, 2),
    (7.5, 9.0, 30.0, 2),
    (7.5, 5.0, 42.0, 2),
    (7.5, 5.0, 46.0, 2),
    (7.5, 6.0, 26.0, 2),
    (7.5, 6.0, 28.0, 2),
    (7.5, 6.0, 22.0, 2),
    (7.5, 6.0, 10.0, 2),
    (7.5, 6.0, 32.0, 2),
    (7.5, 6.0, 32.0, 2),
    (10.0, 7.0, 10.0, 2),
    (10.0, 7.0, 30.0, 2),
    (10.0, 7.0, 30.0, 2),
    (10.0, 7.0, 14.0, 2),
    (10.0, 7.0, 10.0, 2),
    (10.0, 7.0, 12.0, 2),
    (10.0, 8.0, 12.0, 2),
    (10.0, 8.0, 28.0, 2),
    (10.0, 8.0, 42.0, 2),
    (10.0, 8.0, 4.0, 2),
    (10.0, 8.0, 10.0, 2),
    (10.0, 8.0, 22.0, 2),
    (10.0, 9.0, 6.0, 2),
    (10.0, 9.0, 38.0, 2),
    (2.5, 2.0, 18.0, 1),
    (2.5, 2.0, 24.0, 1),
    (2.5, 9.0, 18.0, 1),
    (2.5, 9.0, 28.0, 1),
    (2.5, 9.0, 20.0, 1),
    (2.5, 4.0, 14.0, 1),
    (2.5, 4.0, 36.0, 1),
    (2.5, 4.0, 26.0, 1),
    (2.5, 3.0, 22.0, 1),
    (2.5, 3.0, 42.0, 1),
    (2.5, 3.0, 32.0, 1),
    (2.5, 3.0, 16.0, 1),
    (2.5, 3.0, 38.0, 1),
    (2.5, 9.0, 2.0, 1),
    (2.5, 9.0, 8.0, 1),
    (2.5, 9.0, 26.0, 1),
    (2.5, 9.0, 42.0, 1),
    (5.0, 2.0, 2.0, 1),
    (5.0, 2.0, 14.0, 1),
    (5.0, 2.0, 8.0, 1),
    (5.0, 1.0, 20.0, 1),
    (5.0, 1.0, 32.0, 1),
    (5.0, 3.0, 48.0, 1),
    (5.0, 0.0, 42.0, 1),
    (7.5, 3.0, 2.0, 1),
    (7.5, 3.0, 36.0, 1),
    (7.5, 5.0, 32.0, 1),
    (7.5, 5.0, 20.0, 1),
    (7.5, 5.0, 34.0, 1),
    (7.5, 0.0, 6.0, 1),
    (7.5, 0.0, 12.0, 1),
    (7.5, 8.0, 48.0, 1),
    (7.5, 8.0, 32.0, 1),
    (7.5, 8.0, 4.0, 1),
    (10.0, 5.0, 22.0, 1),
    (10.0, 5.0, 18.0, 1),
    (10.0, 5.0, 46.0, 1),
    (10.0, 5.0, 12.0, 1),
    (10.0, 5.0, 30.0, 1),
    (10.0, 7.0, 36.0, 1),
    (10.0, 7.0, 30.0, 1),
    (10.0, 7.0, 20.0, 1),
    (10.0, 7.0, 38.0, 1),
    (10.0, 7.0, 20.0, 1),
    (10.0, 1.0, 18.0, 1),
    (10.0, 1.0, 10.0, 1),
    (10.0, 1.0, 18.0, 1),
    (10.0, 1.0, 20.0, 1),
    (10.0, 0.0, 12.0, 1),
    (10.0, 0.0, 46.0, 1),
    (10.0, 0.0, 38.0, 1),
    (2.5, 7.0, 40.0, 10),
    (2.5, 7.0, 22.0, 10),
    (2.5, 4.0, 12.0, 10),
    (2.5, 4.0, 32.0, 10),
    (2.5, 4.0, 36.0, 10),
    (2.5, 0.0, 20.0, 10),
    (2.5, 0.0, 30.0, 10),
    (2.5, 3.0, 40.0, 10),
    (2.5, 3.0, 10.0, 10),
    (2.5, 8.0, 42.0, 10),
    (2.5, 8.0, 4.0, 10),
    (2.5, 8.0, 44.0, 10),
    (2.5, 8.0, 32.0, 10),
    (2.5, 8.0, 24.0, 10),
    (5.0, 9.0, 42.0, 10),
    (5.0, 9.0, 18.0, 10),
    (5.0, 9.0, 2.0, 10),
    (5.0, 7.0, 46.0, 10),
    (5.0, 7.0, 42.0, 10),
    (5.0, 7.0, 34.0, 10),
    (5.0, 0.0, 46.0, 10),
    (5.0, 0.0, 8.0, 10),
    (5.0, 5.0, 28.0, 10),
    (5.0, 1.0, 4.0, 10),
    (5.0, 1.0, 10.0, 10),
    (5.0, 1.0, 26.0, 10),
    (7.5, 3.0, 26.0, 10),
    (7.5, 3.0, 42.0, 10),
    (7.5, 3.0, 36.0, 10),
    (7.5, 0.0, 16.0, 10),
    (7.5, 0.0, 40.0, 10),
    (7.5, 2.0, 4.0, 10),
    (7.5, 2.0, 14.0, 10),
    (7.5, 2.0, 46.0, 10),
    (7.5, 8.0, 38.0, 10),
    (7.5, 8.0, 6.0, 10),
    (7.5, 8.0, 24.0, 10),
    (7.5, 8.0, 20.0, 10),
    (7.5, 0.0, 48.0, 10),
    (7.5, 0.0, 20.0, 10),
    (7.5, 0.0, 46.0, 10),
    (7.5, 0.0, 38.0, 10),
    (10.0, 2.0, 32.0, 10),
    (10.0, 2.0, 10.0, 10),
    (10.0, 2.0, 30.0, 10),
    (10.0, 8.0, 14.0, 10),
    (10.0, 8.0, 24.0, 10),
    (10.0, 8.0, 44.0, 10),
    (10.0, 9.0, 28.0, 10),
    (10.0, 9.0, 36.0, 10),
    (10.0, 9.0, 12.0, 10),
    (10.0, 6.0, 20.0, 10),
    (10.0, 6.0, 46.0, 10),
    (10.0, 6.0, 20.0, 10),
    (10.0, 6.0, 28.0, 10),
    (10.0, 6.0, 16.0, 10),
    (10.0, 6.0, 44.0, 10),
    (10.0, 6.0, 28.0, 10),
    (2.5, 6.0, 6.0, 9),
    (2.5, 5.0, 24.0, 9),
    (2.5, 5.0, 6.0, 9),
    (2.5, 2.0, 42.0, 9),
    (2.5, 2.0, 24.0, 9),
    (2.5, 2.0, 36.0, 9),
    (2.5, 2.0, 42.0, 9),
    (2.5, 2.0, 16.0, 9),
    (2.5, 2.0, 22.0, 9),
    (2.5, 2.0, 26.0, 9),
    (2.5, 2.0, 36.0, 9),
    (2.5, 8.0, 30.0, 9),
    (2.5, 8.0, 6.0, 9),
    (5.0, 9.0, 6.0, 9),
    (5.0, 9.0, 22.0, 9),
    (5.0, 9.0, 42.0, 9),
    (5.0, 1.0, 10.0, 9),
    (5.0, 2.0, 32.0, 9),
    (5.0, 2.0, 28.0, 9),
    (5.0, 0.0, 34.0, 9),
    (5.0, 0.0, 22.0, 9),
    (5.0, 4.0, 2.0, 9),
    (5.0, 4.0, 2.0, 9),
    (5.0, 4.0, 4.0, 9),
    (7.5, 5.0, 6.0, 9),
    (7.5, 5.0, 28.0, 9),
    (7.5, 3.0, 2.0, 9),
    (7.5, 3.0, 34.0, 9),
    (7.5, 3.0, 8.0, 9),
    (7.5, 7.0, 46.0, 9),
    (7.5, 9.0, 34.0, 9),
    (7.5, 9.0, 44.0, 9),
    (7.5, 4.0, 10.0, 9),
    (7.5, 4.0, 10.0, 9),
    (10.0, 4.0, 16.0, 9),
    (10.0, 4.0, 4.0, 9),
    (10.0, 1.0, 44.0, 9),
    (10.0, 1.0, 16.0, 9),
    (10.0, 1.0, 30.0, 9),
    (10.0, 1.0, 44.0, 9),
    (10.0, 3.0, 4.0, 9),
    (10.0, 3.0, 46.0, 9),
    (10.0, 0.0, 14.0, 9),
    (2.5, 1.0, 4.0, 8),
    (2.5, 1.0, 18.0, 8),
    (2.5, 1.0, 8.0, 8),
    (2.5, 3.0, 32.0, 8),
    (2.5, 3.0, 28.0, 8),
    (2.5, 3.0, 46.0, 8),
    (2.5, 3.0, 12.0, 8),
    (2.5, 3.0, 18.0, 8),
    (2.5, 0.0, 34.0, 8),
    (2.5, 0.0, 22.0, 8),
    (2.5, 2.0, 22.0, 8),
    (2.5, 2.0, 14.0, 8),
    (2.5, 2.0, 42.0, 8),
    (2.5, 0.0, 16.0, 8),
    (5.0, 4.0, 24.0, 8),
    (5.0, 4.0, 26.0, 8),
    (5.0, 0.0, 26.0, 8),
    (5.0, 2.0, 44.0, 8),
    (5.0, 2.0, 38.0, 8),
    (5.0, 2.0, 48.0, 8),
    (5.0, 2.0, 26.0, 8),
    (5.0, 2.0, 6.0, 8),
    (5.0, 4.0, 12.0, 8),
    (5.0, 9.0, 36.0, 8),
    (5.0, 9.0, 48.0, 8),
    (5.0, 9.0, 16.0, 8),
    (5.0, 9.0, 6.0, 8),
    (7.5, 0.0, 34.0, 8),
    (7.5, 6.0, 10.0, 8),
    (7.5, 6.0, 12.0, 8),
    (7.5, 6.0, 10.0, 8),
    (7.5, 9.0, 26.0, 8),
    (7.5, 9.0, 6.0, 8),
    (7.5, 9.0, 40.0, 8),
    (7.5, 9.0, 18.0, 8),
    (7.5, 4.0, 20.0, 8),
    (7.5, 4.0, 42.0, 8),
    (7.5, 4.0, 16.0, 8),
    (10.0, 5.0, 24.0, 8),
    (10.0, 5.0, 44.0, 8),
    (10.0, 5.0, 20.0, 8),
    (10.0, 5.0, 22.0, 8),
    (10.0, 3.0, 16.0, 8),
    (10.0, 3.0, 14.0, 8),
    (10.0, 3.0, 6.0, 8),
    (10.0, 2.0, 24.0, 8),
    (10.0, 2.0, 36.0, 8),
    (10.0, 2.0, 46.0, 8),
    (10.0, 0.0, 42.0, 8),
    (10.0, 0.0, 26.0, 8),
    (10.0, 0.0, 42.0, 8),
    (10.0, 0.0, 10.0, 8),
    (10.0, 9.0, 12.0, 8),
    (10.0, 9.0, 8.0, 8),
    (2.5, 4.0, 42.0, 7),
    (2.5, 4.0, 14.0, 7),
    (2.5, 4.0, 46.0, 7),
    (2.5, 4.0, 18.0, 7),
    (2.5, 2.0, 2.0, 7),
    (2.5, 2.0, 38.0, 7),
    (2.5, 2.0, 14.0, 7),
    (2.5, 8.0, 26.0, 7),
    (2.5, 7.0, 12.0, 7),
    (2.5, 7.0, 46.0, 7),
    (2.5, 1.0, 44.0, 7),
    (5.0, 8.0, 26.0, 7),
    (5.0, 0.0, 46.0, 7),
    (5.0, 9.0, 44.0, 7),
    (5.0, 9.0, 16.0, 7),
    (5.0, 9.0, 40.0, 7),
    (5.0, 5.0, 14.0, 7),
    (5.0, 7.0, 6.0, 7),
    (5.0, 7.0, 30.0, 7),
    (7.5, 1.0, 16.0, 7),
    (7.5, 1.0, 18.0, 7),
    (7.5, 1.0, 30.0, 7),
    (7.5, 1.0, 4.0, 7),
    (7.5, 1.0, 10.0, 7),
    (7.5, 1.0, 40.0, 7),
    (7.5, 1.0, 18.0, 7),
    (7.5, 3.0, 14.0, 7),
    (7.5, 3.0, 48.0, 7),
    (7.5, 3.0, 48.0, 7),
    (7.5, 3.0, 6.0, 7),
    (7.5, 0.0, 36.0, 7),
    (7.5, 0.0, 42.0, 7),
    (7.5, 0.0, 22.0, 7),
    (10.0, 9.0, 8.0, 7),
    (10.0, 9.0, 8.0, 7),
    (10.0, 7.0, 36.0, 7),
    (10.0, 7.0, 46.0, 7),
    (10.0, 7.0, 20.0, 7),
    (10.0, 7.0, 14.0, 7),
    (10.0, 4.0, 32.0, 7),
    (10.0, 4.0, 16.0, 7),
    (10.0, 1.0, 24.0, 7),
    (10.0, 1.0, 40.0, 7),
    (10.0, 1.0, 18.0, 7),
    (10.0, 1.0, 24.0, 7),
    (10.0, 7.0, 30.0, 7),
    (10.0, 7.0, 30.0, 7),
    (2.5, 5.0, 24.0, 6),
    (2.5, 5.0, 42.0, 6),
    (2.5, 8.0, 34.0, 6),
    (2.5, 8.0, 34.0, 6),
    (2.5, 8.0, 48.0, 6),
    (2.5, 8.0, 26.0, 6),
    (2.5, 3.0, 28.0, 6),
    (5.0, 4.0, 44.0, 6),
    (5.0, 4.0, 46.0, 6),
    (5.0, 6.0, 28.0, 6),
    (5.0, 6.0, 46.0, 6),
    (5.0, 2.0, 10.0, 6),
    (5.0, 2.0, 4.0, 6),
    (5.0, 2.0, 34.0, 6),
    (5.0, 4.0, 46.0, 6),
    (5.0, 4.0, 20.0, 6),
    (7.5, 7.0, 2.0, 6),
    (7.5, 6.0, 46.0, 6),
    (7.5, 6.0, 48.0, 6),
    (7.5, 6.0, 36.0, 6),
    (7.5, 6.0, 42.0, 6),
    (10.0, 7.0, 8.0, 6),
    (10.0, 8.0, 18.0, 6),
    (10.0, 8.0, 42.0, 6),
    (10.0, 8.0, 32.0, 6),
    (10.0, 2.0, 22.0, 6),
    (10.0, 2.0, 28.0, 6),
    (10.0, 2.0, 6.0, 6),
    (10.0, 2.0, 36.0, 6),
    (10.0, 2.0, 48.0, 6),
    (10.0, 2.0, 28.0, 6),
    (10.0, 2.0, 36.0, 6),
    (10.0, 2.0, 6.0, 6),
    (10.0, 9.0, 24.0, 6),
    (2.5, 6.0, 42.0, 5),
    (2.5, 6.0, 8.0, 5),
    (2.5, 7.0, 20.0, 5),
    (2.5, 7.0, 4.0, 5),
    (2.5, 7.0, 16.0, 5),
    (2.5, 5.0, 34.0, 5),
    (2.5, 5.0, 22.0, 5),
    (2.5, 2.0, 26.0, 5),
    (2.5, 2.0, 38.0, 5),
    (2.5, 2.0, 30.0, 5),
    (5.0, 9.0, 42.0, 5),
    (5.0, 9.0, 20.0, 5),
    (5.0, 9.0, 32.0, 5),
    (5.0, 6.0, 46.0, 5),
    (5.0, 6.0, 14.0, 5),
    (5.0, 0.0, 38.0, 5),
    (5.0, 0.0, 16.0, 5),
    (5.0, 4.0, 16.0, 5),
    (5.0, 4.0, 42.0, 5),
    (5.0, 4.0, 8.0, 5),
    (5.0, 9.0, 2.0, 5),
    (5.0, 9.0, 26.0, 5),
    (7.5, 4.0, 20.0, 5),
    (7.5, 4.0, 6.0, 5),
    (7.5, 9.0, 6.0, 5),
    (7.5, 9.0, 48.0, 5),
    (7.5, 9.0, 46.0, 5),
    (7.5, 9.0, 36.0, 5),
    (7.5, 6.0, 40.0, 5),
    (7.5, 3.0, 28.0, 5),
    (7.5, 3.0, 24.0, 5),
    (7.5, 7.0, 14.0, 5),
    (7.5, 7.0, 26.0, 5),
    (7.5, 7.0, 48.0, 5),
    (7.5, 7.0, 16.0, 5),
    (10.0, 4.0, 42.0, 5),
    (10.0, 9.0, 42.0, 5),
    (10.0, 9.0, 6.0, 5),
    (10.0, 9.0, 12.0, 5),
    (10.0, 0.0, 16.0, 5),
    (10.0, 0.0, 14.0, 5),
    (10.0, 8.0, 28.0, 5),
    (10.0, 8.0, 12.0, 5),
    (10.0, 8.0, 34.0, 5),
    (10.0, 6.0, 22.0, 5),
    (10.0, 6.0, 44.0, 5),
    (10.0, 6.0, 32.0, 5))

MUNSELL_BOUNDING_HUES = (
    ((2.5, 4.0), (2.5, 4.0)),
    ((2.5, 4.0), (2.5, 4.0)),
    ((2.5, 4.0), (2.5, 4.0)),
    ((5.0, 4.0), (7.5, 4.0)),
    ((5.0, 4.0), (7.5, 4.0)),
    ((5.0, 4.0), (7.5, 4.0)),
    ((5.0, 4.0), (7.5, 4.0)),
    ((5.0, 4.0), (7.5, 4.0)),
    ((5.0, 4.0), (7.5, 4.0)),
    ((7.5, 4.0), (10.0, 4.0)),
    ((7.5, 4.0), (10.0, 4.0)),
    ((7.5, 4.0), (10.0, 4.0)),
    ((7.5, 4.0), (10.0, 4.0)),
    ((7.5, 4.0), (10.0, 4.0)),
    ((7.5, 4.0), (10.0, 4.0)),
    ((7.5, 4.0), (10.0, 4.0)),
    ((7.5, 4.0), (10.0, 4.0)),
    ((7.5, 4.0), (10.0, 4.0)),
    ((7.5, 4.0), (10.0, 4.0)),
    ((7.5, 4.0), (10.0, 4.0)),
    ((7.5, 4.0), (10.0, 4.0)),
    ((7.5, 4.0), (10.0, 4.0)),
    ((10.0, 4.0), (10.0, 4.0)),
    ((7.5, 4.0), (10.0, 4.0)),
    ((7.5, 4.0), (10.0, 4.0)),
    ((10.0, 4.0), (10.0, 4.0)),
    ((10.0, 4.0), (10.0, 4.0)),
    ((10.0, 4.0), (10.0, 4.0)),
    ((10.0, 4.0), (10.0, 4.0)),
    ((10.0, 4.0), (10.0, 4.0)),
    ((7.5, 4.0), (10.0, 4.0)),
    ((10.0, 4.0), (10.0, 4.0)),
    ((7.5, 4.0), (10.0, 4.0)),
    ((2.5, 3.0), (5.0, 3.0)),
    ((2.5, 3.0), (5.0, 3.0)),
    ((2.5, 3.0), (5.0, 3.0)),
    ((2.5, 3.0), (2.5, 3.0)),
    ((2.5, 3.0), (5.0, 3.0)),
    ((2.5, 3.0), (5.0, 3.0)),
    ((2.5, 3.0), (5.0, 3.0)),
    ((2.5, 3.0), (5.0, 3.0)),
    ((5.0, 3.0), (7.5, 3.0)),
    ((5.0, 3.0), (7.5, 3.0)),
    ((5.0, 3.0), (7.5, 3.0)),
    ((5.0, 3.0), (7.5, 3.0)),
    ((5.0, 3.0), (7.5, 3.0)),
    ((5.0, 3.0), (7.5, 3.0)),
    ((5.0, 3.0), (7.5, 3.0)),
    ((5.0, 3.0), (7.5, 3.0)),
    ((7.5, 3.0), (10.0, 3.0)),
    ((7.5, 3.0), (10.0, 3.0)),
    ((7.5, 3.0), (10.0, 3.0)),
    ((7.5, 3.0), (10.0, 3.0)),
    ((7.5, 3.0), (10.0, 3.0)),
    ((7.5, 3.0), (10.0, 3.0)),
    ((7.5, 3.0), (10.0, 3.0)),
    ((7.5, 3.0), (10.0, 3.0)),
    ((7.5, 3.0), (10.0, 3.0)),
    ((7.5, 3.0), (10.0, 3.0)),
    ((7.5, 3.0), (10.0, 3.0)),
    ((7.5, 3.0), (10.0, 3.0)),
    ((7.5, 3.0), (10.0, 3.0)),
    ((7.5, 3.0), (10.0, 3.0)),
    ((10.0, 3.0), (10.0, 3.0)),
    ((7.5, 3.0), (10.0, 3.0)),
    ((7.5, 3.0), (10.0, 3.0)),
    ((7.5, 3.0), (10.0, 3.0)),
    ((10.0, 3.0), (10.0, 3.0)),
    ((10.0, 3.0), (10.0, 3.0)),
    ((7.5, 3.0), (10.0, 3.0)),
    ((10.0, 3.0), (10.0, 3.0)),
    ((10.0, 3.0), (10.0, 3.0)),
    ((10.0, 3.0), (10.0, 3.0)),
    ((2.5, 2.0), (2.5, 2.0)),
    ((2.5, 2.0), (2.5, 2.0)),
    ((2.5, 2.0), (5.0, 2.0)),
    ((2.5, 2.0), (5.0, 2.0)),
    ((2.5, 2.0), (2.5, 2.0)),
    ((2.5, 2.0), (5.0, 2.0)),
    ((2.5, 2.0), (5.0, 2.0)),
    ((5.0, 2.0), (7.5, 2.0)),
    ((5.0, 2.0), (7.5, 2.0)),
    ((5.0, 2.0), (7.5, 2.0)),
    ((5.0, 2.0), (7.5, 2.0)),
    ((5.0, 2.0), (7.5, 2.0)),
    ((5.0, 2.0), (7.5, 2.0)),
    ((5.0, 2.0), (7.5, 2.0)),
    ((5.0, 2.0), (7.5, 2.0)),
    ((5.0, 2.0), (7.5, 2.0)),
    ((5.0, 2.0), (7.5, 2.0)),
    ((7.5, 2.0), (10.0, 2.0)),
    ((7.5, 2.0), (10.0, 2.0)),
    ((10.0, 2.0), (10.0, 2.0)),
    ((10.0, 2.0), (10.0, 2.0)),
    ((7.5, 2.0), (10.0, 2.0)),
    ((7.5, 2.0), (10.0, 2.0)),
    ((2.5, 1.0), (5.0, 1.0)),
    ((2.5, 1.0), (2.5, 1.0)),
    ((2.5, 1.0), (2.5, 1.0)),
    ((2.5, 1.0), (2.5, 1.0)),
    ((2.5, 1.0), (2.5, 1.0)),
    ((2.5, 1.0), (5.0, 1.0)),
    ((2.5, 1.0), (2.5, 1.0)),
    ((2.5, 1.0), (5.0, 1.0)),
    ((2.5, 1.0), (2.5, 1.0)),
    ((2.5, 1.0), (2.5, 1.0)),
    ((2.5, 1.0), (5.0, 1.0)),
    ((2.5, 1.0), (2.5, 1.0)),
    ((2.5, 1.0), (2.5, 1.0)),
    ((2.5, 1.0), (2.5, 1.0)),
    ((2.5, 1.0), (5.0, 1.0)),
    ((5.0, 1.0), (7.5, 1.0)),
    ((5.0, 1.0), (7.5, 1.0)),
    ((5.0, 1.0), (7.5, 1.0)),
    ((5.0, 1.0), (7.5, 1.0)),
    ((5.0, 1.0), (7.5, 1.0)),
    ((5.0, 1.0), (7.5, 1.0)),
    ((5.0, 1.0), (7.5, 1.0)),
    ((5.0, 1.0), (7.5, 1.0)),
    ((7.5, 1.0), (10.0, 1.0)),
    ((7.5, 1.0), (10.0, 1.0)),
    ((7.5, 1.0), (10.0, 1.0)),
    ((7.5, 1.0), (10.0, 1.0)),
    ((7.5, 1.0), (10.0, 1.0)),
    ((7.5, 1.0), (10.0, 1.0)),
    ((7.5, 1.0), (10.0, 1.0)),
    ((7.5, 1.0), (10.0, 1.0)),
    ((10.0, 1.0), (10.0, 1.0)),
    ((7.5, 1.0), (10.0, 1.0)),
    ((10.0, 1.0), (10.0, 1.0)),
    ((10.0, 1.0), (10.0, 1.0)),
    ((2.5, 10.0), (5.0, 10.0)),
    ((2.5, 10.0), (5.0, 10.0)),
    ((2.5, 10.0), (2.5, 10.0)),
    ((2.5, 10.0), (2.5, 10.0)),
    ((2.5, 10.0), (2.5, 10.0)),
    ((2.5, 10.0), (2.5, 10.0)),
    ((2.5, 10.0), (2.5, 10.0)),
    ((2.5, 10.0), (2.5, 10.0)),
    ((5.0, 10.0), (7.5, 10.0)),
    ((5.0, 10.0), (7.5, 10.0)),
    ((5.0, 10.0), (7.5, 10.0)),
    ((5.0, 10.0), (7.5, 10.0)),
    ((5.0, 10.0), (7.5, 10.0)),
    ((5.0, 10.0), (7.5, 10.0)),
    ((5.0, 10.0), (7.5, 10.0)),
    ((5.0, 10.0), (7.5, 10.0)),
    ((5.0, 10.0), (7.5, 10.0)),
    ((5.0, 10.0), (7.5, 10.0)),
    ((5.0, 10.0), (7.5, 10.0)),
    ((5.0, 10.0), (7.5, 10.0)),
    ((5.0, 10.0), (7.5, 10.0)),
    ((5.0, 10.0), (7.5, 10.0)),
    ((5.0, 10.0), (7.5, 10.0)),
    ((5.0, 10.0), (7.5, 10.0)),
    ((5.0, 10.0), (7.5, 10.0)),
    ((5.0, 10.0), (7.5, 10.0)),
    ((5.0, 10.0), (7.5, 10.0)),
    ((5.0, 10.0), (7.5, 10.0)),
    ((7.5, 10.0), (10.0, 10.0)),
    ((7.5, 10.0), (10.0, 10.0)),
    ((7.5, 10.0), (10.0, 10.0)),
    ((7.5, 10.0), (10.0, 10.0)),
    ((7.5, 10.0), (10.0, 10.0)),
    ((7.5, 10.0), (10.0, 10.0)),
    ((7.5, 10.0), (10.0, 10.0)),
    ((7.5, 10.0), (10.0, 10.0)),
    ((7.5, 10.0), (10.0, 10.0)),
    ((7.5, 10.0), (10.0, 10.0)),
    ((7.5, 10.0), (10.0, 10.0)),
    ((7.5, 10.0), (10.0, 10.0)),
    ((10.0, 10.0), (10.0, 10.0)),
    ((7.5, 10.0), (10.0, 10.0)),
    ((10.0, 10.0), (10.0, 10.0)),
    ((10.0, 10.0), (10.0, 10.0)),
    ((7.5, 10.0), (10.0, 10.0)),
    ((7.5, 10.0), (10.0, 10.0)),
    ((10.0, 10.0), (10.0, 10.0)),
    ((7.5, 10.0), (10.0, 10.0)),
    ((7.5, 10.0), (10.0, 10.0)),
    ((10.0, 10.0), (10.0, 10.0)),
    ((10.0, 10.0), (10.0, 10.0)),
    ((2.5, 9.0), (2.5, 9.0)),
    ((2.5, 9.0), (2.5, 9.0)),
    ((2.5, 9.0), (5.0, 9.0)),
    ((2.5, 9.0), (5.0, 9.0)),
    ((2.5, 9.0), (2.5, 9.0)),
    ((2.5, 9.0), (5.0, 9.0)),
    ((2.5, 9.0), (2.5, 9.0)),
    ((2.5, 9.0), (5.0, 9.0)),
    ((2.5, 9.0), (2.5, 9.0)),
    ((2.5, 9.0), (2.5, 9.0)),
    ((2.5, 9.0), (5.0, 9.0)),
    ((2.5, 9.0), (5.0, 9.0)),
    ((5.0, 9.0), (7.5, 9.0)),
    ((5.0, 9.0), (7.5, 9.0)),
    ((5.0, 9.0), (7.5, 9.0)),
    ((5.0, 9.0), (7.5, 9.0)),
    ((5.0, 9.0), (7.5, 9.0)),
    ((5.0, 9.0), (7.5, 9.0)),
    ((5.0, 9.0), (7.5, 9.0)),
    ((5.0, 9.0), (7.5, 9.0)),
    ((5.0, 9.0), (7.5, 9.0)),
    ((5.0, 9.0), (7.5, 9.0)),
    ((5.0, 9.0), (7.5, 9.0)),
    ((5.0, 9.0), (7.5, 9.0)),
    ((5.0, 9.0), (7.5, 9.0)),
    ((5.0, 9.0), (7.5, 9.0)),
    ((5.0, 9.0), (7.5, 9.0)),
    ((5.0, 9.0), (7.5, 9.0)),
    ((5.0, 9.0), (7.5, 9.0)),
    ((7.5, 9.0), (10.0, 9.0)),
    ((7.5, 9.0), (10.0, 9.0)),
    ((7.5, 9.0), (10.0, 9.0)),
    ((7.5, 9.0), (10.0, 9.0)),
    ((7.5, 9.0), (10.0, 9.0)),
    ((7.5, 9.0), (10.0, 9.0)),
    ((7.5, 9.0), (10.0, 9.0)),
    ((7.5, 9.0), (10.0, 9.0)),
    ((7.5, 9.0), (10.0, 9.0)),
    ((7.5, 9.0), (10.0, 9.0)),
    ((7.5, 9.0), (10.0, 9.0)),
    ((7.5, 9.0), (10.0, 9.0)),
    ((7.5, 9.0), (10.0, 9.0)),
    ((7.5, 9.0), (10.0, 9.0)),
    ((10.0, 9.0), (10.0, 9.0)),
    ((10.0, 9.0), (10.0, 9.0)),
    ((10.0, 9.0), (10.0, 9.0)),
    ((10.0, 9.0), (10.0, 9.0)),
    ((7.5, 9.0), (10.0, 9.0)),
    ((7.5, 9.0), (10.0, 9.0)),
    ((7.5, 9.0), (10.0, 9.0)),
    ((7.5, 9.0), (10.0, 9.0)),
    ((10.0, 9.0), (10.0, 9.0)),
    ((7.5, 9.0), (10.0, 9.0)),
    ((10.0, 9.0), (10.0, 9.0)),
    ((10.0, 9.0), (10.0, 9.0)),
    ((2.5, 8.0), (2.5, 8.0)),
    ((2.5, 8.0), (5.0, 8.0)),
    ((2.5, 8.0), (5.0, 8.0)),
    ((2.5, 8.0), (5.0, 8.0)),
    ((2.5, 8.0), (2.5, 8.0)),
    ((2.5, 8.0), (5.0, 8.0)),
    ((5.0, 8.0), (7.5, 8.0)),
    ((5.0, 8.0), (7.5, 8.0)),
    ((5.0, 8.0), (7.5, 8.0)),
    ((5.0, 8.0), (7.5, 8.0)),
    ((5.0, 8.0), (7.5, 8.0)),
    ((5.0, 8.0), (7.5, 8.0)),
    ((5.0, 8.0), (7.5, 8.0)),
    ((7.5, 8.0), (10.0, 8.0)),
    ((7.5, 8.0), (10.0, 8.0)),
    ((7.5, 8.0), (10.0, 8.0)),
    ((7.5, 8.0), (10.0, 8.0)),
    ((7.5, 8.0), (10.0, 8.0)),
    ((7.5, 8.0), (10.0, 8.0)),
    ((10.0, 8.0), (10.0, 8.0)),
    ((7.5, 8.0), (10.0, 8.0)),
    ((10.0, 8.0), (10.0, 8.0)),
    ((7.5, 8.0), (10.0, 8.0)),
    ((7.5, 8.0), (10.0, 8.0)),
    ((10.0, 8.0), (10.0, 8.0)),
    ((10.0, 8.0), (10.0, 8.0)),
    ((10.0, 8.0), (10.0, 8.0)),
    ((7.5, 8.0), (10.0, 8.0)),
    ((2.5, 7.0), (5.0, 7.0)),
    ((2.5, 7.0), (5.0, 7.0)),
    ((2.5, 7.0), (2.5, 7.0)),
    ((2.5, 7.0), (5.0, 7.0)),
    ((2.5, 7.0), (2.5, 7.0)),
    ((2.5, 7.0), (5.0, 7.0)),
    ((2.5, 7.0), (2.5, 7.0)),
    ((5.0, 7.0), (7.5, 7.0)),
    ((5.0, 7.0), (7.5, 7.0)),
    ((5.0, 7.0), (7.5, 7.0)),
    ((5.0, 7.0), (7.5, 7.0)),
    ((5.0, 7.0), (7.5, 7.0)),
    ((5.0, 7.0), (7.5, 7.0)),
    ((5.0, 7.0), (7.5, 7.0)),
    ((5.0, 7.0), (7.5, 7.0)),
    ((7.5, 7.0), (10.0, 7.0)),
    ((7.5, 7.0), (10.0, 7.0)),
    ((7.5, 7.0), (10.0, 7.0)),
    ((7.5, 7.0), (10.0, 7.0)),
    ((7.5, 7.0), (10.0, 7.0)),
    ((7.5, 7.0), (10.0, 7.0)),
    ((10.0, 7.0), (10.0, 7.0)),
    ((7.5, 7.0), (10.0, 7.0)),
    ((10.0, 7.0), (10.0, 7.0)),
    ((10.0, 7.0), (10.0, 7.0)),
    ((10.0, 7.0), (10.0, 7.0)),
    ((10.0, 7.0), (10.0, 7.0)),
    ((10.0, 7.0), (10.0, 7.0)),
    ((2.5, 6.0), (5.0, 6.0)),
    ((2.5, 6.0), (5.0, 6.0)),
    ((2.5, 6.0), (2.5, 6.0)),
    ((2.5, 6.0), (2.5, 6.0)),
    ((2.5, 6.0), (2.5, 6.0)),
    ((5.0, 6.0), (7.5, 6.0)),
    ((5.0, 6.0), (7.5, 6.0)),
    ((5.0, 6.0), (7.5, 6.0)),
    ((5.0, 6.0), (7.5, 6.0)),
    ((5.0, 6.0), (7.5, 6.0)),
    ((5.0, 6.0), (7.5, 6.0)),
    ((5.0, 6.0), (7.5, 6.0)),
    ((7.5, 6.0), (10.0, 6.0)),
    ((7.5, 6.0), (10.0, 6.0)),
    ((7.5, 6.0), (10.0, 6.0)),
    ((7.5, 6.0), (10.0, 6.0)),
    ((10.0, 6.0), (10.0, 6.0)),
    ((10.0, 6.0), (10.0, 6.0)),
    ((10.0, 6.0), (10.0, 6.0)),
    ((10.0, 6.0), (10.0, 6.0)),
    ((10.0, 6.0), (10.0, 6.0)),
    ((10.0, 6.0), (10.0, 6.0)),
    ((7.5, 6.0), (10.0, 6.0)),
    ((10.0, 6.0), (10.0, 6.0)),
    ((7.5, 6.0), (10.0, 6.0)),
    ((7.5, 6.0), (10.0, 6.0)),
    ((7.5, 6.0), (10.0, 6.0)),
    ((7.5, 6.0), (10.0, 6.0)),
    ((2.5, 5.0), (5.0, 5.0)),
    ((2.5, 5.0), (2.5, 5.0)),
    ((2.5, 5.0), (2.5, 5.0)),
    ((2.5, 5.0), (5.0, 5.0)),
    ((2.5, 5.0), (2.5, 5.0)),
    ((2.5, 5.0), (2.5, 5.0)),
    ((2.5, 5.0), (2.5, 5.0)),
    ((2.5, 5.0), (5.0, 5.0)),
    ((5.0, 5.0), (7.5, 5.0)),
    ((5.0, 5.0), (7.5, 5.0)),
    ((5.0, 5.0), (7.5, 5.0)),
    ((5.0, 5.0), (7.5, 5.0)),
    ((5.0, 5.0), (7.5, 5.0)),
    ((5.0, 5.0), (7.5, 5.0)),
    ((5.0, 5.0), (7.5, 5.0)),
    ((7.5, 5.0), (10.0, 5.0)),
    ((7.5, 5.0), (10.0, 5.0)),
    ((7.5, 5.0), (10.0, 5.0)),
    ((7.5, 5.0), (10.0, 5.0)),
    ((7.5, 5.0), (10.0, 5.0)),
    ((7.5, 5.0), (10.0, 5.0)),
    ((7.5, 5.0), (10.0, 5.0)),
    ((7.5, 5.0), (10.0, 5.0)))

MUNSELL_HUE_TO_ANGLE = (
    (2.5, 1, 208.750),
    (2.5, 2, 153.750),
    (2.5, 3, 118.750),
    (2.5, 4, 63.750),
    (2.5, 5, 39.375),
    (2.5, 6, 16.875),
    (2.5, 7, 348.750),
    (2.5, 8, 300.000),
    (2.5, 9, 251.250),
    (2.5, 10, 236.250),
    (5.0, 1, 225.000),
    (5.0, 2, 160.000),
    (5.0, 3, 135.000),
    (5.0, 4, 70.000),
    (5.0, 5, 45.000),
    (5.0, 6, 22.500),
    (5.0, 7, 0.000),
    (5.0, 8, 315.000),
    (5.0, 9, 255.000),
    (5.0, 10, 240.000),
    (7.5, 1, 228.750),
    (7.5, 2, 176.250),
    (7.5, 3, 141.250),
    (7.5, 4, 86.250),
    (7.5, 5, 51.250),
    (7.5, 6, 28.125),
    (7.5, 7, 5.625),
    (7.5, 8, 326.250),
    (7.5, 9, 270.000),
    (7.5, 10, 243.750),
    (10.0, 1, 232.500),
    (10.0, 2, 192.500),
    (10.0, 3, 147.500),
    (10.0, 4, 102.500),
    (10.0, 5, 57.500),
    (10.0, 6, 33.750),
    (10.0, 7, 11.250),
    (10.0, 8, 337.500),
    (10.0, 9, 285.000),
    (10.0, 10, 247.500))

MUNSELL_HUE_TO_ASTM_HUE = (
    (2.5, 0, 72.5),
    (2.5, 1, 62.5),
    (2.5, 2, 52.5),
    (2.5, 3, 42.5),
    (2.5, 4, 32.5),
    (2.5, 5, 22.5),
    (2.5, 6, 12.5),
    (2.5, 7, 2.5),
    (2.5, 8, 92.5),
    (2.5, 9, 82.5),
    (2.5, 10, 72.5),
    (5.0, 0, 75.0),
    (5.0, 1, 65.0),
    (5.0, 2, 55.0),
    (5.0, 3, 45.0),
    (5.0, 4, 35.0),
    (5.0, 5, 25.0),
    (5.0, 6, 15.0),
    (5.0, 7, 5.0),
    (5.0, 8, 95.0),
    (5.0, 9, 85.0),
    (5.0, 10, 75.0),
    (7.5, 0, 77.5),
    (7.5, 1, 67.5),
    (7.5, 2, 57.5),
    (7.5, 3, 47.5),
    (7.5, 4, 37.5),
    (7.5, 5, 27.5),
    (7.5, 6, 17.5),
    (7.5, 7, 7.5),
    (7.5, 8, 97.5),
    (7.5, 9, 87.5),
    (7.5, 10, 77.5),
    (10.0, 0, 80.0),
    (10.0, 1, 70.0),
    (10.0, 2, 60.0),
    (10.0, 3, 50.0),
    (10.0, 4, 40.0),
    (10.0, 5, 30.0),
    (10.0, 6, 20.0),
    (10.0, 7, 10.0),
    (10.0, 8, 100.0),
    (10.0, 9, 90.0),
    (10.0, 10, 80.0))

MUNSELL_INTERPOLATION_METHODS = (
    'Radial',
    'Radial',
    'Radial',
    'Radial',
    'Radial',
    'Radial',
    'Radial',
    'Radial',
    'Radial',
    'Radial',
    'Linear',
    'Linear',
    None,
    None,
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Radial',
    'Linear',
    'Radial',
    'Radial',
    'Radial',
    'Linear',
    'Linear',
    'Linear',
    'Radial',
    'Radial',
    'Radial',
    'Radial',
    'Radial',
    'Radial',
    'Radial',
    'Radial',
    'Radial',
    'Radial',
    'Radial',
    'Radial',
    'Radial',
    'Radial',
    'Radial',
    'Radial',
    'Linear',
    'Radial',
    'Radial',
    'Radial',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    None,
    None,
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    None,
    None,
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Radial',
    'Radial',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Radial',
    'Radial',
    'Radial',
    'Radial',
    'Radial',
    'Radial',
    'Radial',
    'Radial',
    'Radial',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Radial',
    'Linear',
    'Linear',
    'Radial',
    None,
    'Radial',
    'Radial',
    'Radial',
    'Radial',
    'Radial',
    None,
    None,
    'Radial',
    'Radial',
    'Radial',
    'Radial',
    'Radial',
    'Radial',
    'Radial',
    'Radial',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    None,
    None,
    None,
    'Linear',
    'Linear',
    'Radial',
    'Radial',
    'Radial',
    None,
    None,
    'Radial',
    'Radial',
    'Radial',
    'Radial',
    'Radial',
    'Radial',
    'Radial',
    'Linear',
    'Linear',
    'Radial',
    'Linear',
    'Linear',
    'Linear',
    None,
    None,
    'Radial',
    'Radial',
    'Radial',
    'Radial',
    'Radial',
    'Radial',
    'Radial',
    None,
    None,
    'Radial',
    'Linear',
    'Linear',
    'Radial',
    'Radial',
    'Radial',
    'Radial',
    None,
    None,
    None,
    None,
    'Linear',
    'Linear',
    'Linear',
    'Radial',
    'Radial',
    'Radial',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Radial',
    'Linear',
    'Radial',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Radial',
    'Radial',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    None,
    None,
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    None,
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    None,
    None,
    'Linear',
    'Linear',
    'Linear',
    None,
    'Linear',
    'Linear',
    None,
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    None,
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    None,
    None,
    None,
    None,
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    None,
    'Linear',
    'Linear',
    'Linear',
    'Radial',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    None,
    None,
    None,
    'Radial',
    'Radial',
    'Radial',
    'Radial',
    'Radial',
    'Radial',
    'Radial',
    'Radial',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Radial',
    'Radial',
    'Radial',
    'Radial',
    'Radial',
    'Radial',
    'Radial',
    'Radial',
    'Radial',
    'Radial',
    'Radial',
    'Radial',
    'Radial',
    'Linear',
    'Radial',
    'Linear',
    'Radial',
    'Radial',
    'Radial',
    'Radial',
    'Radial',
    'Radial',
    'Radial',
    'Radial',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Radial',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Radial',
    'Linear',
    'Radial',
    'Radial',
    'Linear',
    'Radial',
    'Linear',
    'Radial',
    'Radial',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Radial',
    'Radial',
    None,
    None,
    'Radial',
    'Radial',
    'Radial',
    'Radial',
    'Linear',
    'Radial',
    'Radial',
    'Radial',
    'Linear',
    'Linear',
    'Linear',
    'Radial',
    'Radial',
    'Radial',
    'Linear',
    'Linear',
    'Linear',
    'Linear',
    'Radial',
    'Linear',
    'Radial',
    'Radial',
    None,
    None,
    'Linear',
    'Radial',
    'Linear',
    'Radial',
    'Radial',
    'Radial')

MUNSELL_XY_FROM_RENOTATION_OVOID = (
    (0.4333, 0.5602),
    None,
    None,
    None,
    None,
    None,
    (0.3799, 0.4470),
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    (0.3284, 0.3559),
    (0.3722, 0.4669),
    None,
    None,
    (0.2740, 0.8790),
    None,
    None,
    (0.3395, 0.5913),
    (0.3030, 0.8090),
    None,
    (0.3450, 0.5949),
    None,
    None,
    (0.3450, 0.5949),
    None,
    (0.2020, 0.8070),
    None,
    None,
    None,
    None,
    (0.1680, 0.8800),
    (0.3123, 0.4732),
    None,
    (0.3092, 0.5095),
    None,
    (0.3128, 0.4175),
    None,
    (0.1490, 0.6810),
    (0.1689, 0.6549),
    None,
    (0.2060, 0.6190),
    None,
    None,
    (0.1630, 0.6700),
    (0.1630, 0.6700),
    (0.2952, 0.3851),
    (0.0690, 0.7640),
    (0.2574, 0.4814),
    (0.1230, 0.5460),
    (0.1397, 0.5312),
    None,
    (0.2554, 0.4087),
    (0.2466, 0.4181),
    None,
    (0.2833, 0.3564),
    None,
    None,
    (0.1516, 0.4505),
    (0.1303, 0.4858),
    (0.1841, 0.4448),
    None,
    (0.1688, 0.4570),
    (0.1982, 0.4330),
    None,
    None,
    (0.1262, 0.4667),
    None,
    (0.1022, 0.4759),
    (0.1842, 0.4244),
    (0.2200, 0.3983),
    (0.0340, 0.5460),
    (0.2171, 0.4138),
    (0.1398, 0.4168),
    None,
    (0.2910, 0.3310),
    (0.0690, 0.4542),
    None,
    None,
    (0.1551, 0.4208),
    None,
    (0.0925, 0.4275),
    None,
    None,
    (0.0333, 0.4444),
    (0.2957, 0.3293),
    (0.2430, 0.3710),
    (0.2282, 0.3811),
    (0.1866, 0.4086),
    None,
    (0.2940, 0.3268),
    None,
    None,
    None,
    None,
    (0.0636, 0.3788),
    None,
    None,
    None,
    (0.26, 0.3289),
    None,
    None,
    (0.0781, 0.3211),
    None,
    (0.0670, 0.3200),
    None,
    None,
    None,
    (0.2500, 0.3141),
    None,
    None,
    None,
    (0.1220, 0.3510),
    None,
    None,
    (0.2234, 0.3150),
    None,
    None,
    None,
    None,
    (0.2768, 0.3287),
    None,
    (0.2094, 0.3165),
    None,
    None,
    None,
    None,
    None,
    None,
    (0.0680, 0.2830),
    None,
    (0.0920, 0.2900),
    (0.1961, 0.3110),
    None,
    None,
    (0.2035, 0.2956),
    None,
    None,
    (0.1671, 0.2832),
    (0.2035, 0.2956),
    (0.1841, 0.2892),
    (0.1937, 0.2978),
    None,
    None,
    (0.2686, 0.3130),
    (0.2120, 0.3025),
    (0.1180, 0.2730),
    (0.2501, 0.3118),
    None,
    None,
    None,
    None,
    None,
    None,
    (0.1027, 0.2057),
    None,
    None,
    None,
    None,
    None,
    (0.0650, 0.1700),
    None,
    (0.2909, 0.3125),
    (0.2280, 0.2960),
    None,
    None,
    (0.2559, 0.2874),
    None,
    (0.1245, 0.1827),
    None,
    None,
    None,
    None,
    (0.2616, 0.2857),
    None,
    None,
    (0.0980, 0.1460),
    None,
    None,
    None,
    None,
    None,
    (0.2688, 0.2956),
    (0.0960, 0.1260),
    (0.1203, 0.1505),
    None,
    (0.1666, 0.1964),
    None,
    None,
    None,
    (0.1280, 0.1620),
    None,
    (0.1280, 0.1620),
    None,
    (0.0840, 0.0940),
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    (0.1634, 0.1698),
    None,
    None,
    None,
    None,
    None,
    (0.1576, 0.1600),
    None,
    (0.2758, 0.2879),
    None,
    None,
    None,
    None,
    None,
    (0.2991, 0.3057),
    None,
    None,
    None,
    None,
    None,
    (0.1090, 0.0790),
    (0.2012, 0.1867),
    (0.1285, 0.0870),
    (0.0950, 0.0270),
    (0.1642, 0.0655),
    (0.1570, 0.0340),
    (0.1590, 0.0440),
    None,
    None,
    (0.2420, 0.2148),
    (0.1762, 0.0955),
    (0.1610, 0.0160),
    None,
    (0.2702, 0.2648),
    None,
    None,
    None,
    None,
    None,
    None,
    (0.1918, 0.0379),
    (0.2200, 0.1330),
    (0.1925, 0.0420),
    (0.2400, 0.1960),
    None,
    None,
    None,
    None,
    None,
    (0.2140, 0.1430),
    None,
    (0.2140, 0.1430),
    (0.1940, 0.1010),
    (0.2265, 0.1671),
    None,
    (0.1940, 0.1010),
    (0.2842, 0.2550),
    (0.2372, 0.1223),
    (0.2806, 0.2444),
    (0.2180, 0.0220),
    (0.2277, 0.0621),
    (0.2200, 0.0310),
    (0.2180, 0.0220),
    (0.2372, 0.0980),
    (0.2298, 0.0696),
    (0.2260, 0.0555),
    (0.2200, 0.0310),
    None,
    (0.2881, 0.2671),
    (0.2960, 0.2710),
    None,
    None,
    (0.2701, 0.1178),
    (0.2540, 0.0390),
    (0.2559, 0.0525),
    None,
    None,
    (0.3022, 0.2825),
    (0.3022, 0.2825),
    (0.2958, 0.2565),
    (0.3093, 0.2555),
    (0.3018, 0.1253),
    (0.3088, 0.2740),
    (0.2910, 0.0600),
    (0.3037, 0.1981),
    None,
    None,
    None,
    (0.3056, 0.2060),
    (0.3056, 0.2060),
    (0.3370, 0.1756),
    (0.3210, 0.2686),
    None,
    (0.3078, 0.0839),
    None,
    None,
    (0.3214, 0.2517),
    None,
    None,
    (0.3290, 0.2095),
    (0.3370, 0.0800),
    (0.3342, 0.1551),
    None,
    (0.4140, 0.1020),
    None,
    (0.3754, 0.1898),
    (0.3929, 0.1506),
    None,
    None,
    (0.3830, 0.0960),
    (0.3711, 0.1449),
    None,
    None,
    (0.4730, 0.1720),
    (0.4820, 0.1620),
    None,
    None,
    None,
    None,
    None,
    (0.3708, 0.2380),
    (0.4104, 0.2361),
    None,
    None,
    (0.4000, 0.2630),
    (0.3431, 0.2988),
    None,
    (0.3960, 0.2860),
    (0.4125, 0.2784),
    (0.3960, 0.2860),
    None,
    (0.3512, 0.3052),
    None,
    None,
    (0.5130, 0.2101),
    None,
    (0.4799, 0.2329),
    (0.5700, 0.2400),
    None,
    (0.5396, 0.2535),
    (0.5540, 0.2460),
    (0.5628, 0.2241),
    (0.5380, 0.2369),
    (0.4218, 0.2864),
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    (0.4140, 0.3020),
    (0.3760, 0.3100),
    None,
    (0.5369, 0.2810),
    None,
    (0.5898, 0.2622),
    (0.3614, 0.3033),
    None,
    (0.5734, 0.2083),
    None,
    (0.4435, 0.3119),
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    (0.5341, 0.3158),
    (0.3805, 0.3244),
    None,
    None,
    None,
    None,
    (0.4660, 0.2888),
    (0.6111, 0.2290),
    None,
    None,
    (0.6492, 0.3012),
    None,
    None,
    (0.4738, 0.3316),
    None,
    None,
    None,
    (0.4160, 0.350),
    (0.4160, 0.350),
    None,
    None,
    (0.5920, 0.3740),
    (0.5234, 0.3700),
    None,
    (0.6409, 0.3533),
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    (0.6020, 0.4050),
    None,
    None,
    None,
    None,
    None,
    (0.6840, 0.4150),
    (0.4674, 0.3738),
    None,
    None,
    None,
    (0.3437, 0.3397),
    None,
    None,
    None,
    None,
    (0.4399, 0.4164),
    (0.5179, 0.4670),
    None,
    None,
    None,
    None,
    (0.5450, 0.4580),
    None,
    None,
    None,
    None,
    (0.5450, 0.4580),
    (0.5320, 0.4780),
    None,
    (0.4517, 0.4421),
    (0.5160, 0.4920),
    (0.3761, 0.3800),
    (0.5049, 0.4843),
    None,
    None,
    None,
    None,
    None,
    None,
    (0.4830, 0.5092),
    None,
    None,
    (0.4905, 0.5038),
    None,
    None,
    None,
    None,
    (0.4745, 0.4810),
    (0.3378, 0.3504),
    None,
    None,
    (0.4331, 0.4688),
    (0.3811, 0.4123),
    None,
    None,
    None,
    None,
    None,
    None,
    (0.4652, 0.5128),
    None,
    None,
    (0.4728, 0.5215),
    None,
    None,
    (0.3761, 0.4155),
    (0.4271, 0.4920),
    None,
    None,
    None,
    (0.4341, 0.5020),
    None,
    None,
    None,
    None)

MUNSELL_SPECIFICATIONS_TO_XY = (
    ((2.5000000000000000, 8.0000000000000000, 11.9285463083509686, 4),
     (0.4149248329505340, 0.5123568112702328)),
    ((2.5000000000000000, 6.0000000000000000, 6.9236108262088836, 4),
     (0.3894593720512620, 0.4661649246438344)),
    ((3.0588107073010358, 6.0000000000000000, 14.5019666777045693, 4),
     (0.4297142783224320, 0.5665812104155267)),
    ((2.5000000000000000, 5.0000000000000000, 2.0000000000000000, 4),
     (0.3352000000000000, 0.3636000000000000)),
    ((5.6130070624423842, 8.0000000000000000, 18.5659089404439115, 4),
     (0.3990910794231500, 0.5888839244592698)),
    ((5.8456400710049072, 8.0000000000000000, 5.7823256145522954, 4),
     (0.3476544320849577, 0.4154512289066640)),
    ((5.7807941210595990, 3.0000000000000000, 3.3492086825591487, 4),
     (0.3397413408138117, 0.4168925891450931)),
    ((5.4836842996391173, 4.0000000000000000, 5.7614590625067148, 4),
     (0.3624285934444264, 0.4768182836607108)),
    ((5.8095803088134961, 6.0000000000000000, 6.6626137539588992, 4),
     (0.3569251029495760, 0.4554043675955618)),
    ((5.2092529556629028, 3.0000000000000000, 5.1414726438100136, 4),
     (0.3633176290702536, 0.4808425142327246)),
    ((7.7061058539115734, 3.0000000000000000, 11.3966488972748969, 4),
     (0.3131610795605752, 0.6936669653933070)),
    ((8.1176405645643435, 3.0000000000000000, 3.1653563832640272, 4),
     (0.3191717580108967, 0.3988687571291857)),
    ((7.8731203012311255, 3.0000000000000000, 13.2411079692977136, 4),
     (0.2945431122633574, 0.7584911897992045)),
    ((8.0498332221428903, 2.0000000000000000, 7.5019246790810632, 4),
     (0.3074690943666493, 0.6094306766573039)),
    ((8.3553075693910621, 3.0000000000000000, 11.9254413443363916, 4),
     (0.2925864520380243, 0.7013043294887223)),
    ((8.3427957605776086, 1.0000000000000000, 6.2988181459092560, 4),
     (0.2563315496261501, 0.7193319941337727)),
    ((7.5947244020062845, 2.0000000000000000, 4.6266131353312874, 4),
     (0.3240099379289350, 0.4735391066221303)),
    ((8.1951778660857908, 9.0000000000000000, 23.5711220101815080, 4),
     (0.3393737554129363, 0.6530759138839242)),
    ((7.7547636349124689, 8.0000000000000000, 21.0094490106106804, 4),
     (0.3524522658493676, 0.6357582251092715)),
    ((9.0102319629788621, 6.0000000000000000, 6.8033705689301751, 4),
     (0.3215881688765898, 0.4404084623233154)),
    ((9.0415668516516217, 6.0000000000000000, 17.0100372035664478, 4),
     (0.3077087669482304, 0.6419582684061792)),
    ((9.9156521698279132, 9.0000000000000000, 11.1310821598843201, 4),
     (0.3156394420758696, 0.4719064963381894)),
    ((10.0000000000000000, 9.0000000000000000, 27.3220461867991027, 4),
     (0.2710507921594040, 0.6981866004283883)),
    ((9.9613361115981434, 8.0000000000000000, 13.2000986334405592, 4),
     (0.3110050190589885, 0.5122850671666230)),
    ((9.8874065510631812, 8.0000000000000000, 2.0660963235598375, 4),
     (0.3124556963039024, 0.3470190461737042)),
    ((10.0000000000000000, 3.0000000000000000, 2.5700932200974145, 4),
     (0.3078023368648296, 0.3733350402476546)),
    ((10.0000000000000000, 3.0000000000000000, 13.5140666071695144, 4),
     (0.2343255740710980, 0.7247335078491779)),
    ((10.0000000000000000, 5.0000000000000000, 12.7538997749639886, 4),
     (0.2901551111476837, 0.5923266098579271)),
    ((10.0000000000000000, 6.0000000000000000, 15.2445982768494179, 4),
     (0.2905993077541776, 0.6049052757954609)),
    ((10.0000000000000000, 5.0000000000000000, 27.9290010198770950, 4),
     (0.1548874872515363, 0.9471895260068659)),
    ((9.7570396457430526, 6.0000000000000000, 3.4112871270786895, 4),
     (0.3133140053725343, 0.3734442304113026)),
    ((10.0000000000000000, 6.0000000000000000, 24.8636013065843109, 4),
     (0.2323865915072020, 0.7984084248251019)),
    ((9.8620758176293215, 4.0000000000000000, 7.6719680950003797, 4),
     (0.3036210954390051, 0.5020744140554851)),
    ((3.2140937198013564, 9.0000000000000000, 3.4367939376082868, 3),
     (0.3008524135601138, 0.3526678514951116)),
    ((3.1967035260607033, 9.0000000000000000, 24.7800313890532884, 3),
     (0.2002744068603639, 0.5778452716097220)),
    ((2.5000000000000000, 9.0000000000000000, 27.7365817049776346, 3),
     (0.1938439280651565, 0.6398389804597316)),
    ((2.7908763449337677, 8.0000000000000000, 20.8683045640276035, 3),
     (0.2242349039201748, 0.5571230146385515)),
    ((3.2214995668974771, 6.0000000000000000, 5.4677262571376586, 3),
     (0.2873858207778558, 0.3848413427300203)),
    ((2.6225120704322471, 6.0000000000000000, 19.3644722529733038, 3),
     (0.1963796698158202, 0.5896085735184569)),
    ((3.2873061024849806, 5.0000000000000000, 19.8557241925879140, 3),
     (0.1510722266321749, 0.6036632631469063)),
    ((5.7276124050033674, 3.0000000000000000, 10.7466425521665023, 3),
     (0.1792067699717417, 0.4665320350934196)),
    ((5.3479557011490932, 3.0000000000000000, 18.9004718151949049, 3),
     (0.0754352156182513, 0.5598764933852344)),
    ((5.7385751713204325, 4.0000000000000000, 4.2231608377596563, 3),
     (0.2734191193432446, 0.3703759727313385)),
    ((5.7208241035815108, 2.0000000000000000, 4.8780681593635187, 3),
     (0.2467519504501329, 0.3960429658769290)),
    ((5.3167800244843564, 1.0000000000000000, 8.0439576065413636, 3),
     (0.0538724204405257, 0.5623155400405534)),
    ((5.7623230008312385, 2.0000000000000000, 9.5074117162556888, 3),
     (0.1625788656566373, 0.4748117643951303)),
    ((5.5695553288489306, 2.0000000000000000, 17.5944919348104420, 3),
     (-0.0054388706155134, 0.5989078963083921)),
    ((5.9855795053875953, 2.0000000000000000, 14.8034345271893475, 3),
     (0.0508819972140898, 0.5508235640922985)),
    ((5.4616196034207549, 3.0000000000000000, 6.6471547360970025, 3),
     (0.2368393501455289, 0.4149076913619224)),
    ((8.4790509607452531, 3.0000000000000000, 19.9321706074452436, 3),
     (0.0559604297316293, 0.4808828600574318)),
    ((7.8382779261952082, 3.0000000000000000, 6.2385280252185922, 3),
     (0.2300693247638420, 0.3896583956403046)),
    ((8.2830613968174998, 3.0000000000000000, 10.3508251747691542, 3),
     (0.1715486279317307, 0.4234896181567097)),
    ((7.6031550323552723, 6.0000000000000000, 29.1395411651987040, 3),
     (0.0755258536348385, 0.5167802829864803)),
    ((8.3241150395279764, 7.0000000000000000, 23.5157789731952569, 3),
     (0.1410408872001719, 0.4624726907273586)),
    ((8.4442427312468595, 7.0000000000000000, 2.4130843113046656, 3),
     (0.2935043828079583, 0.3347759364340525)),
    ((8.3090617745210764, 6.0000000000000000, 17.5072521345144878, 3),
     (0.1663963086070524, 0.4409592268834456)),
    ((7.6986647972426248, 3.0000000000000000, 18.8288026602073764, 3),
     (0.0697647651863942, 0.4955781487363820)),
    ((8.1403711706809201, 3.0000000000000000, 14.6499332953540424, 3),
     (0.1158711695036809, 0.4588739141154994)),
    ((8.4849035532136945, 2.0000000000000000, 11.8795622626339483, 3),
     (0.1011840430707628, 0.4519866737136369)),
    ((8.4541090296230159, 2.0000000000000000, 4.6063171733042516, 3),
     (0.2398368021197325, 0.3722219663037951)),
    ((8.3052624291689856, 5.0000000000000000, 3.9045072719017924, 3),
     (0.2762948164597561, 0.3507630813736354)),
    ((8.1897300045792871, 5.0000000000000000, 28.1269927592368632, 3),
     (0.0570529273405628, 0.5054724651556731)),
    ((7.5402877810747500, 6.0000000000000000, 6.6353191939359162, 3),
     (0.2612474304996075, 0.3719851145582656)),
    ((7.9629991342362985, 5.0000000000000000, 20.2933548056268656, 3),
     (0.1171199892604328, 0.4747357359194437)),
    ((8.4329595590383715, 6.0000000000000000, 26.4699708737570667, 3),
     (0.0950374077805397, 0.4853470923031731)),
    ((10.0000000000000000, 9.0000000000000000, 6.0469956581432704, 3),
     (0.2699968780049759, 0.3515467272052522)),
    ((9.7713539460569141, 9.0000000000000000, 20.8297527154788895, 3),
     (0.1703271759874031, 0.4176338874276043)),
    ((9.3763807965222234, 9.0000000000000000, 13.3485223941066824, 3),
     (0.2248310611706177, 0.3904997531995366)),
    ((9.9127041795322288, 4.0000000000000000, 25.7782317703519226, 3),
     (0.0416825310907419, 0.4563810827964475)),
    ((10.0000000000000000, 5.0000000000000000, 13.7122476433708371, 3),
     (0.1697041588274939, 0.4075044010703485)),
    ((10.0000000000000000, 4.0000000000000000, 28.5879233609310326, 3),
     (0.0185905747930173, 0.4614698084023276)),
    ((9.2875351467329246, 4.0000000000000000, 6.9973895652846254, 3),
     (0.2277961811911799, 0.3782363477013876)),
    ((10.0000000000000000, 6.0000000000000000, 30.9324350684787923, 3),
     (0.0594729545206485, 0.4677973052054364)),
    ((10.0000000000000000, 5.0000000000000000, 7.9468547464613932, 3),
     (0.2302899123142785, 0.3726200114371990)),
    ((10.0000000000000000, 6.0000000000000000, 26.1724102978957731, 3),
     (0.0929707125487827, 0.4525172308936873)),
    ((2.5000000000000000, 7.0000000000000000, 12.6434105570570861, 2),
     (0.2047310102650148, 0.3654658906154655)),
    ((2.5000000000000000, 3.0000000000000000, 19.1675377625573944, 2),
     (0.0551094365707736, 0.3689588995456623)),
    ((3.2845817745734109, 4.0000000000000000, 10.3167618622771258, 2),
     (0.1661726301578022, 0.3484711045004487)),
    ((3.0814075494281132, 3.0000000000000000, 4.0316837245147514, 2),
     (0.2410240991989147, 0.3343695993283068)),
    ((2.5342634625499727, 9.0000000000000000, 14.8636631042532183, 2),
     (0.2017780568146162, 0.3722383864015372)),
    ((5.2759205646620941, 9.0000000000000000, 12.8791359497697275, 2),
     (0.2039735983690586, 0.3424643745315380)),
    ((5.5228564491289642, 9.0000000000000000, 17.4125865956868147, 2),
     (0.1669033134425928, 0.3431822968019505)),
    ((5.8859149397779467, 9.0000000000000000, 17.3880868140724374, 2),
     (0.1658435364668269, 0.3393430032707385)),
    ((5.9565603219671557, 4.0000000000000000, 4.3116902048143899, 2),
     (0.2412970987176909, 0.3181985042671494)),
    ((5.6279111948942635, 5.0000000000000000, 16.5668191444311503, 2),
     (0.1161662303839994, 0.3161181105644866)),
    ((5.8534547245334565, 5.0000000000000000, 18.8350698050853502, 2),
     (0.0953817576043788, 0.3095289993435369)),
    ((5.4455811466993644, 5.0000000000000000, 25.6907370240232069, 2),
     (0.0580839514033309, 0.3095707855049915)),
    ((5.1447203696302557, 5.0000000000000000, 18.9791729668054074, 2),
     (0.0970169204750135, 0.3213519214647800)),
    ((5.2907074463880175, 6.0000000000000000, 13.5985209980560526, 2),
     (0.1689464086892715, 0.3308774206782637)),
    ((5.4158444031977657, 6.0000000000000000, 15.1786174644616256, 2),
     (0.1547832053398711, 0.3292147430009046)),
    ((8.2041448522882447, 6.0000000000000000, 4.0201776913722949, 2),
     (0.2592762703697450, 0.3144147576406639)),
    ((9.3660699534030183, 3.0000000000000000, 15.4227661827945788, 2),
     (0.0696143575224152, 0.2215145245906554)),
    ((10.0000000000000000, 4.0000000000000000, 9.1923876167058154, 2),
     (0.1593065733661186, 0.2652494804914122)),
    ((10.0000000000000000, 3.0000000000000000, 15.9542478936070324, 2),
     (0.0653385655873080, 0.2062081720840880)),
    ((9.2605862715376066, 3.0000000000000000, 10.5951757917016192, 2),
     (0.1194483796728095, 0.2491723584774583)),
    ((9.5716758646706186, 3.0000000000000000, 17.3988475313979336, 2),
     (0.0561202206650064, 0.2098742745919364)),
    ((3.2387393821759787, 5.0000000000000000, 3.7435106940988625, 1),
     (0.2529831434272229, 0.2948457588752119)),
    ((2.5000000000000000, 4.0000000000000000, 7.3993436144209168, 1),
     (0.1830402067957548, 0.2579260387473276)),
    ((2.5000000000000000, 4.0000000000000000, 8.8608404173678377, 1),
     (0.1619064862820606, 0.2450828564523734)),
    ((2.5000000000000000, 8.0000000000000000, 10.8872656168291240, 1),
     (0.1982153399209648, 0.2800403945667933)),
    ((2.5000000000000000, 7.0000000000000000, 10.1023353741859108, 1),
     (0.1983919965642688, 0.2769729728229426)),
    ((2.6104349455855846, 8.0000000000000000, 4.2361715150659922, 1),
     (0.2643515482131810, 0.3055875202093172)),
    ((2.5000000000000000, 9.0000000000000000, 5.3613636980274295, 1),
     (0.2537056811707120, 0.3036923862002273)),
    ((3.1731014606584806, 8.0000000000000000, 15.1995362353083028, 1),
     (0.1580590485886202, 0.2514747579810662)),
    ((2.5000000000000000, 7.0000000000000000, 5.4202608625739925, 1),
     (0.2479162478998444, 0.2982609826359614)),
    ((2.5000000000000000, 8.0000000000000000, 9.6193649384034430, 1),
     (0.2103682871098059, 0.2854986672587055)),
    ((3.2525815090531771, 7.0000000000000000, 6.2224060204343745, 1),
     (0.2390630935153159, 0.2917705881260558)),
    ((2.5000000000000000, 8.0000000000000000, 12.2618083975850567, 1),
     (0.1856448040789573, 0.2739956813711087)),
    ((2.5000000000000000, 3.0000000000000000, 7.7685055469176172, 1),
     (0.1547460376360476, 0.2354728181440944)),
    ((2.5000000000000000, 3.0000000000000000, 6.9988409117240948, 1),
     (0.1668682556403455, 0.2433618806548280)),
    ((3.0205621196907169, 3.0000000000000000, 5.2030875391050824, 1),
     (0.1967384856281847, 0.2591147093725996)),
    ((5.2190911687613255, 3.0000000000000000, 13.5738875509672745, 1),
     (0.0906282824426043, 0.1526128858299024)),
    ((5.5962506280473505, 2.0000000000000000, 5.1651068503657331, 1),
     (0.1802998704232731, 0.2280057975669525)),
    ((5.0785748388973584, 3.0000000000000000, 6.8599427244043705, 1),
     (0.1703877047693254, 0.2261025920732490)),
    ((5.1756171558445825, 3.0000000000000000, 4.5608003821410303, 1),
     (0.2082057297112164, 0.2552779443369814)),
    ((5.4973530207828443, 5.0000000000000000, 2.0000000000000000, 1),
     (0.2795522354807150, 0.3025607207610571)),
    ((5.8417735135440010, 6.0000000000000000, 13.2893656678185508, 1),
     (0.1580509585265904, 0.2195781471641720)),
    ((5.5805491854636680, 7.0000000000000000, 16.1803201492634017, 1),
     (0.1443961574280030, 0.2138383106823240)),
    ((5.2877727269225270, 7.0000000000000000, 14.0989464615804039, 1),
     (0.1610894209189601, 0.2284159614826388)),
    ((8.3582212856142686, 5.0000000000000000, 17.2715635972971029, 1),
     (0.1177394904919951, 0.1596537139206562)),
    ((7.8772447963597703, 5.0000000000000000, 5.5989343468594752, 1),
     (0.2307667812589323, 0.2641771899347817)),
    ((8.3233369535874786, 5.0000000000000000, 7.0881523668119195, 1),
     (0.2134734863722117, 0.2482718153364515)),
    ((7.8454860962996813, 5.0000000000000000, 16.2337951792823887, 1),
     (0.1226987733157440, 0.1683464919309280)),
    ((8.0205644292879210, 5.0000000000000000, 18.3902607972839363, 1),
     (0.1091357015077960, 0.1533423459286211)),
    ((8.3825695023449427, 5.0000000000000000, 13.9751241108762905, 1),
     (0.1436832135477909, 0.1850808157279726)),
    ((7.8555937497823543, 3.0000000000000000, 5.3604358250617752, 1),
     (0.1986276216250285, 0.2335331893406649)),
    ((7.6555011537339137, 4.0000000000000000, 9.7695930479633919, 1),
     (0.1631153136408173, 0.2046758551433150)),
    ((7.6530191580084930, 6.0000000000000000, 11.7045897666252809, 1),
     (0.1767568752135769, 0.2222790981272987)),
    ((9.8432861463350942, 3.0000000000000000, 15.4733174004740430, 1),
     (0.0972770641476631, 0.1190280729126712)),
    ((10.0000000000000000, 3.0000000000000000, 5.4158461678022469, 1),
     (0.2024420074738948, 0.2258870613333070)),
    ((9.4744223583682956, 3.0000000000000000, 15.1781613955925074, 1),
     (0.0972252355767127, 0.1221290200731146)),
    ((9.1966481560049633, 8.0000000000000000, 5.0692233667592408, 1),
     (0.2595901106060160, 0.2847030494945458)),
    ((9.5966112843263396, 8.0000000000000000, 11.1801937971981040, 1),
     (0.1972527298467179, 0.2327377514066649)),
    ((10.0000000000000000, 6.0000000000000000, 7.7628023164068498, 1),
     (0.2213905756777281, 0.2489584989206977)),
    ((10.0000000000000000, 7.0000000000000000, 18.3743753864025088, 1),
     (0.1385665599883837, 0.1733793722951824)),
    ((2.8739501345809000, 4.0000000000000000, 4.4945211069126740, 10),
     (0.2436346344044748, 0.2527864036169822)),
    ((2.9797638317158932, 9.0000000000000000, 6.4267107939641992, 10),
     (0.2536062796096411, 0.2679911028334137)),
    ((2.5000000000000000, 9.0000000000000000, 2.4912528414503781, 10),
     (0.2922190319544085, 0.3020506629214542)),
    ((2.5000000000000000, 8.0000000000000000, 8.8233798640361893, 10),
     (0.2291778912916562, 0.2455895812236743)),
    ((2.5000000000000000, 5.0000000000000000, 18.8393399778644905, 10),
     (0.1311380591361334, 0.1355442901438808)),
    ((2.5000000000000000, 5.0000000000000000, 5.4177118115989469, 10),
     (0.2433418862137124, 0.2555545429854522)),
    ((2.5000000000000000, 9.0000000000000000, 8.0453567253469096, 10),
     (0.2386598245598982, 0.2566825029225717)),
    ((2.5000000000000000, 3.0000000000000000, 16.1958101598068147, 10),
     (0.1119230441210625, 0.1056293288813523)),
    ((5.6678871626197305, 2.0000000000000000, 18.2260108117431834, 10),
     (0.1271039527255801, 0.0648184871342615)),
    ((5.7596738401992056, 2.0000000000000000, 30.4287315274152483, 10),
     (0.1170922163828955, 0.0367419959060915)),
    ((5.7836346614632728, 2.0000000000000000, 21.4801942145111369, 10),
     (0.1252546828761673, 0.0545674582903900)),
    ((5.1181732488629281, 2.0000000000000000, 41.8684733585788322, 10),
     (0.0907957014092668, 0.0217960516920637)),
    ((5.7573497243896670, 2.0000000000000000, 13.6096042678049560, 10),
     (0.1430142065765375, 0.0919952369348554)),
    ((5.2793040612960453, 5.0000000000000000, 22.8761275280486629, 10),
     (0.1345875853434695, 0.1071322352345223)),
    ((5.7157098010598082, 5.0000000000000000, 30.3602134880221577, 10),
     (0.1188604948730221, 0.0698161646363042)),
    ((5.9479473045208477, 4.0000000000000000, 4.8966439066197935, 10),
     (0.2500490236976596, 0.2428877897256325)),
    ((5.0989932248172396, 5.0000000000000000, 26.2687504247525787, 10),
     (0.1170961328183621, 0.0876906684851776)),
    ((5.5322294976298503, 5.0000000000000000, 7.7564492627214818, 10),
     (0.2311862502859707, 0.2255138788367854)),
    ((5.9235845417681920, 5.0000000000000000, 19.5676050308493856, 10),
     (0.1579541805959956, 0.1254715070218464)),
    ((5.9501563870301712, 2.0000000000000000, 4.9536669461614116, 10),
     (0.2186507216671770, 0.1994067474023979)),
    ((5.6141581365353224, 2.0000000000000000, 20.6449539043668935, 10),
     (0.1223399021857134, 0.0568999724297949)),
    ((5.4359081407306382, 3.0000000000000000, 21.5850643322003926, 10),
     (0.1230157774331309, 0.0739703398954444)),
    ((5.5399085613433288, 3.0000000000000000, 44.9036990399531604, 10),
     (0.1037616438284486, 0.0279456823069682)),
    ((5.3792514320991325, 2.0000000000000000, 25.8890745588287317, 10),
     (0.1087354589429733, 0.0423623476092259)),
    ((5.6329098306822463, 6.0000000000000000, 21.3840425068616966, 10),
     (0.1545925291739519, 0.1315791146519223)),
    ((5.2033265149329200, 6.0000000000000000, 15.5144674274224315, 10),
     (0.1772117052552854, 0.1681504765856490)),
    ((5.9277936921340721, 5.0000000000000000, 3.7766395197414253, 10),
     (0.2713480548309330, 0.2704571168097510)),
    ((5.8173223961875111, 5.0000000000000000, 11.3180415809075203, 10),
     (0.2048472516348240, 0.1908000868612774)),
    ((7.8877860422500454, 1.0000000000000000, 12.5742407145616575, 10),
     (0.1803130361508687, 0.0780420183599527)),
    ((7.9499606335916067, 3.0000000000000000, 25.6213689020893334, 10),
     (0.1704449134180661, 0.0672547412267178)),
    ((8.3825924368107589, 2.0000000000000000, 40.5412719529260102, 10),
     (0.1717894907858984, 0.0241880920706110)),
    ((7.9637973633225698, 2.0000000000000000, 36.7073187099669482, 10),
     (0.1677020908907290, 0.0299390219350496)),
    ((8.3739244566104745, 2.0000000000000000, 8.6238460649901665, 10),
     (0.2072844480182091, 0.1438837097116739)),
    ((8.1519906864733880, 2.0000000000000000, 42.2291271964581441, 10),
     (0.1690414994139126, 0.0216563092053128)),
    ((8.4883840852320755, 8.0000000000000000, 9.7796280723158073, 10),
     (0.2488450360455433, 0.2262778851799687)),
    ((8.4383570688761633, 3.0000000000000000, 26.8734524920740441, 10),
     (0.1759955928328011, 0.0648395414079383)),
    ((8.3094349065304414, 2.0000000000000000, 48.4996639934449902, 10),
     (0.1695729540893724, 0.0122505040098325)),
    ((7.7115794149655015, 3.0000000000000000, 5.7298598433541956, 10),
     (0.2355951727785135, 0.2052200577467951)),
    ((7.6273740879401934, 2.0000000000000000, 26.7249730707769224, 10),
     (0.1665795361459382, 0.0478416893380129)),
    ((7.6939233372260842, 3.0000000000000000, 48.4078975056904852, 10),
     (0.1585200777350650, 0.0242658764917851)),
    ((10.0000000000000000, 6.0000000000000000, 10.9719538159106609, 10),
     (0.2491402309204467, 0.2089496110383951)),
    ((9.1130972747403813, 6.0000000000000000, 2.7564645951736484, 10),
     (0.2924932485533538, 0.2880400492114162)),
    ((10.0000000000000000, 9.0000000000000000, 6.0033883251860249, 10),
     (0.2779813642114769, 0.2659644225855468)),
    ((10.0000000000000000, 5.0000000000000000, 19.1707567215596981, 10),
     (0.2142974946878668, 0.1376681488510317)),
    ((9.3801100887551563, 6.0000000000000000, 18.8175077437544154, 10),
     (0.2111704138102771, 0.1491437428186353)),
    ((9.0017959465770332, 8.0000000000000000, 4.4538545632120776, 10),
     (0.2859321611242305, 0.2800965098757311)),
    ((10.0000000000000000, 8.0000000000000000, 3.6531597236888560, 10),
     (0.2931116736026046, 0.2880429565835092)),
    ((9.0461828964214455, 3.0000000000000000, 22.3009468068498471, 10),
     (0.1878970718888041, 0.0806325075235593)),
    ((9.4594207963837835, 3.0000000000000000, 10.5525569494149547, 10),
     (0.2199160703334100, 0.1509982577104353)),
    ((10.0000000000000000, 3.0000000000000000, 31.2476220198124004, 10),
     (0.1930514267881125, 0.0563442772435347)),
    ((10.0000000000000000, 3.0000000000000000, 29.2734347311524985, 10),
     (0.1942359391613085, 0.0617527414355611)),
    ((2.5000000000000000, 8.0000000000000000, 8.3750743751782615, 9),
     (0.2788747768744653, 0.2454618380609135)),
    ((2.5000000000000000, 7.0000000000000000, 9.5028468626493314, 9),
     (0.2746400359807273, 0.2331258016674807)),
    ((2.8619005171223564, 7.0000000000000000, 7.4661261346289010, 9),
     (0.2834963865952931, 0.2511139999686381)),
    ((3.0874221941355513, 8.0000000000000000, 2.4938578293607869, 9),
     (0.3032944607358996, 0.2995034682629164)),
    ((2.5000000000000000, 4.0000000000000000, 19.7747167807561688, 9),
     (0.2398055097946389, 0.1233503218668033)),
    ((2.5000000000000000, 3.0000000000000000, 45.2998448680714958, 9),
     (0.2183500775659643, 0.0294003102638570)),
    ((3.2205535070037539, 3.0000000000000000, 37.0593806627261628, 9),
     (0.2306933945582851, 0.0465932347925963)),
    ((2.5000000000000000, 4.0000000000000000, 39.1664185009443742, 9),
     (0.2188335814990556, 0.0535007444971669)),
    ((2.7654037016841957, 3.0000000000000000, 29.7265355691379369, 9),
     (0.2287662164867549, 0.0651724329920893)),
    ((2.5000000000000000, 4.0000000000000000, 12.0876546872501276, 9),
     (0.2556808632818747, 0.1723645035174366)),
    ((2.5000000000000000, 3.0000000000000000, 3.3828527595771778, 9),
     (0.2832114570627484, 0.2446297883631457)),
    ((2.8366121370807811, 4.0000000000000000, 2.0000000000000000, 9),
     (0.2970496478809859, 0.2808823856404053)),
    ((2.8888545547050946, 3.0000000000000000, 14.6183070378328566, 9),
     (0.2477649838426708, 0.1297032686876268)),
    ((5.1643993319905190, 6.0000000000000000, 9.1114653837439121, 9),
     (0.2895176496411621, 0.2337182478465021)),
    ((5.9541801299653683, 6.0000000000000000, 34.8449159168278655, 9),
     (0.2714161513182527, 0.0985720050812948)),
    ((5.5003569030033876, 7.0000000000000000, 13.1541281319682977, 9),
     (0.2872121998845203, 0.2147291790311368)),
    ((5.7772079140795913, 6.0000000000000000, 29.9439835353833885, 9),
     (0.2720370516301255, 0.1159240288631761)),
    ((5.5358100577424327, 7.0000000000000000, 8.8927166641344755, 9),
     (0.2942708344242646, 0.2451099871340891)),
    ((5.5900409663439943, 4.0000000000000000, 22.7566127868985504, 9),
     (0.2728742088784858, 0.1206681524718601)),
    ((5.2826202617433458, 3.0000000000000000, 18.7328236887543831, 9),
     (0.2686089817112237, 0.1139022423971973)),
    ((5.1728956401601813, 3.0000000000000000, 6.2292543458148515, 9),
     (0.2877332357942354, 0.2114241583957755)),
    ((5.2597218547319811, 3.0000000000000000, 35.8908721106814141, 9),
     (0.2577945586528000, 0.0515901545285382)),
    ((5.5536463415959245, 3.0000000000000000, 10.0766837095490551, 9),
     (0.2825792293975564, 0.1719757532037015)),
    ((5.7300039721591451, 2.0000000000000000, 15.9856983902699774, 9),
     (0.2730995343938890, 0.1063678860404959)),
    ((5.7823815169906521, 2.0000000000000000, 28.7746185183793024, 9),
     (0.2648379647447670, 0.0501957479096187)),
    ((5.0693797816654609, 7.0000000000000000, 2.2194841714206595, 9),
     (0.3055058860092272, 0.2991172470460963)),
    ((5.9037163337566136, 6.0000000000000000, 35.5055742919949680, 9),
     (0.2700285376143730, 0.0962670616600573)),
    ((5.1346796709796605, 6.0000000000000000, 27.7263986439234174, 9),
     (0.2627660248271106, 0.1222152172584064)),
    ((5.3832606878646239, 6.0000000000000000, 18.3022959341279225, 9),
     (0.2779886327084519, 0.1740366479136525)),
    ((5.8697920884647008, 5.0000000000000000, 32.5534321679666334, 9),
     (0.2711278988572353, 0.0979612733527640)),
    ((5.4624511435406120, 6.0000000000000000, 30.9488646344402127, 9),
     (0.2654630166563432, 0.1099792483234828)),
    ((5.3574452696396984, 6.0000000000000000, 5.2614344690064048, 9),
     (0.2990173572087526, 0.2662518452771558)),
    ((5.6263734530030343, 6.0000000000000000, 25.1708466664452359, 9),
     (0.2742275810312197, 0.1377261683537158)),
    ((8.2842008951649930, 2.0000000000000000, 17.2388998041601766, 9),
     (0.3015541927190563, 0.1059406922854441)),
    ((8.3181027841240187, 2.0000000000000000, 22.5961473835359179, 9),
     (0.2998682305173034, 0.0793394672401119)),
    ((7.8519368662427134, 7.0000000000000000, 20.9623744079114580, 9),
     (0.3127810446477129, 0.1816835017614740)),
    ((8.1460813360327027, 8.0000000000000000, 13.5339629184693369, 9),
     (0.3169359353603149, 0.2297919580402022)),
    ((8.0972086431627499, 7.0000000000000000, 17.3389915505245398, 9),
     (0.3163880687849892, 0.2028484172851260)),
    ((7.8302562919917973, 7.0000000000000000, 10.7068221638259242, 9),
     (0.3131765403497294, 0.2411294582908910)),
    ((7.8006589706884801, 6.0000000000000000, 6.2113756808778051, 9),
     (0.3116266436816035, 0.2642498962824071)),
    ((8.0448636471186354, 7.0000000000000000, 15.5571552615442279, 9),
     (0.3156670691172467, 0.2125748887713169)),
    ((8.4617748029090709, 5.0000000000000000, 36.0372969397773204, 9),
     (0.3193853258545449, 0.0982926965468475)),
    ((7.6123828822072843, 4.0000000000000000, 14.1686907807062248, 9),
     (0.3048713359950443, 0.1749148969907045)),
    ((8.1696339276959975, 4.0000000000000000, 27.2358461038644108, 9),
     (0.3101886458910356, 0.1119662532052473)),
    ((9.6020311360157748, 6.0000000000000000, 20.5806356758181010, 9),
     (0.3358702478125188, 0.1831568326135899)),
    ((9.6636860301788179, 6.0000000000000000, 29.0476584729829561, 9),
     (0.3420516528840192, 0.1451039062175931)),
    ((9.7529285473647107, 5.0000000000000000, 34.1149316052812921, 9),
     (0.3460058496414126, 0.1130635241267973)),
    ((10.0000000000000000, 6.0000000000000000, 4.2162157304370860, 9),
     (0.3185864853934834, 0.2854243280891126)),
    ((10.0000000000000000, 6.0000000000000000, 34.7285267558391624, 9),
     (0.3513642633779195, 0.1268144197324825)),
    ((10.0000000000000000, 5.0000000000000000, 14.7796272948823670, 9),
     (0.3368575900243706, 0.2021171430544264)),
    ((10.0000000000000000, 6.0000000000000000, 2.0000000000000000, 9),
     (0.3146000000000000, 0.3018000000000000)),
    ((9.4970509139487298, 6.0000000000000000, 10.8088547800987307, 9),
     (0.3261445983768467, 0.2380376784480618)),
    ((9.8266351634655322, 2.0000000000000000, 7.0671144318498502, 9),
     (0.3199412725610485, 0.1982548952372193)),
    ((9.3825023503012588, 5.0000000000000000, 19.9994768774463623, 9),
     (0.3328136867226051, 0.1703164585209611)),
    ((9.1155305918192742, 5.0000000000000000, 5.8834364886948176, 9),
     (0.3188381363847329, 0.2614152937200581)),
    ((10.0000000000000000, 5.0000000000000000, 24.7458702329524449, 9),
     (0.3456712832096572, 0.1519571163934759)),
    ((9.3783595885807927, 5.0000000000000000, 26.2957872574229228, 9),
     (0.3358063441615217, 0.1417054365528667)),
    ((10.0000000000000000, 2.0000000000000000, 21.5725763566023510, 9),
     (0.3230213711821699, 0.0882584893991581)),
    ((10.0000000000000000, 3.0000000000000000, 26.0398724912355775, 9),
     (0.3343139553719324, 0.0976046247929457)),
    ((2.5000000000000000, 3.0000000000000000, 4.7121381662539816, 8),
     (0.3435962977395826, 0.2553142252457729)),
    ((2.8874578666829285, 2.0000000000000000, 13.9948960521457479, 8),
     (0.3783554816672732, 0.1472522585909139)),
    ((3.4354195604394651, 2.0000000000000000, 6.7189891135327322, 8),
     (0.3598352213726018, 0.2214395880930813)),
    ((2.9925336062737173, 2.0000000000000000, 7.1980143398663090, 8),
     (0.3575475618962941, 0.2131380990474943)),
    ((2.5000000000000000, 1.0000000000000000, 14.1567267100244649, 8),
     (0.3368000000000000, 0.1010753124108557)),
    ((2.6104579288975813, 1.0000000000000000, 3.3458156268951917, 8),
     (0.3281972508449899, 0.2218263920769044)),
    ((5.1670653045538115, 8.0000000000000000, 2.1409481568506346, 8),
     (0.3190507908774075, 0.3116390008840625)),
    ((5.0544341143469511, 7.0000000000000000, 6.4421573326031334, 8),
     (0.3501536607617208, 0.2933004270294575)),
    ((5.8037356824506121, 9.0000000000000000, 10.4438417735233937, 8),
     (0.3753780093453096, 0.2859037624969863)),
    ((5.0448775397799679, 6.0000000000000000, 18.4244287014075532, 8),
     (0.4277251999841766, 0.2364041931422235)),
    ((5.4848324026214836, 7.0000000000000000, 5.4747774912956473, 8),
     (0.3449818965080228, 0.2983629686835587)),
    ((5.1623004272002886, 7.0000000000000000, 24.9990562485251253, 8),
     (0.4499951490629939, 0.2226727486889720)),
    ((5.8772563607434130, 7.0000000000000000, 15.4504441432596611, 8),
     (0.4135138022364607, 0.2615018632684154)),
    ((8.1974490801098732, 2.0000000000000000, 2.0000000000000000, 8),
     (0.3479365513139208, 0.2910133676082857)),
    ((7.9972372657542374, 2.0000000000000000, 11.6558293358065175, 8),
     (0.4536492593166012, 0.1964961320942676)),
    ((7.9731925609071839, 9.0000000000000000, 4.2728868868791814, 8),
     (0.3382290806106464, 0.3100993156381419)),
    ((7.5103557401084613, 8.0000000000000000, 14.3201413179509949, 8),
     (0.4160257314289051, 0.2791113211989992)),
    ((7.8364986461862207, 8.0000000000000000, 13.5966587179990253, 8),
     (0.4144950683870351, 0.2833993277746695)),
    ((7.7821869659085170, 9.0000000000000000, 13.9021055240679452, 8),
     (0.4117119567839591, 0.2858194582898892)),
    ((9.5317952667717609, 5.0000000000000000, 2.0000000000000000, 8),
     (0.3325257851841513, 0.3124819697521387)),
    ((10.0000000000000000, 5.0000000000000000, 11.0556249127789368, 8),
     (0.4462369676728198, 0.2877358440858011)),
    ((9.3122708373931626, 7.0000000000000000, 11.1852220991899731, 8),
     (0.4115569799140528, 0.2962671589127124)),
    ((10.0000000000000000, 7.0000000000000000, 13.8954559024466775, 8),
     (0.4445754678439774, 0.2933561330390057)),
    ((9.9256699400322717, 5.0000000000000000, 7.2040789887667955, 8),
     (0.3999342335645963, 0.3000028935899999)),
    ((9.4167408824024026, 5.0000000000000000, 8.7201163481804915, 8),
     (0.4142087527437497, 0.2926385477337663)),
    ((10.0000000000000000, 4.0000000000000000, 16.4696989109913723, 8),
     (0.5288485073674999, 0.2505105957717457)),
    ((10.0000000000000000, 6.0000000000000000, 6.5992372339473091, 8),
     (0.3796927537224994, 0.3063213729788948)),
    ((10.0000000000000000, 5.0000000000000000, 4.5502697844677806, 8),
     (0.3664709667304110, 0.3075968120496071)),
    ((9.9703325305196788, 6.0000000000000000, 10.8370227220876441, 8),
     (0.4235409167024141, 0.2965173827647608)),
    ((2.9627075871745849, 9.0000000000000000, 9.9991169316305388, 7),
     (0.4082112009458871, 0.3178515499760940)),
    ((3.1672052728994915, 9.0000000000000000, 7.3836247298929152, 7),
     (0.3812051955464745, 0.3201699383275524)),
    ((2.5000000000000000, 5.0000000000000000, 17.8815938530076153, 7),
     (0.5525791262360914, 0.2808499433585711)),
    ((2.7415018638966284, 6.0000000000000000, 18.0029087378013806, 7),
     (0.5290289042496541, 0.2948216346860084)),
    ((2.5000000000000000, 6.0000000000000000, 10.2326689962714923, 7),
     (0.4348850955537665, 0.3113811958067113)),
    ((2.8779022261852312, 6.0000000000000000, 3.5582034231201787, 7),
     (0.3519353725942859, 0.3171294726325335)),
    ((2.5000000000000000, 6.0000000000000000, 27.7799959269169712, 7),
     (0.5960099816711264, 0.2656600122192491)),
    ((5.4128217712844577, 3.0000000000000000, 7.2580400206056073, 7),
     (0.4917264876045092, 0.3162107392471261)),
    ((5.8375474760508403, 3.0000000000000000, 11.9982613806154710, 7),
     (0.5975448574102102, 0.2979480968890348)),
    ((5.9693975439749885, 4.0000000000000000, 14.3979064202833023, 7),
     (0.5881617924445709, 0.3123641899930510)),
    ((5.0040790005633813, 5.0000000000000000, 22.7366776144687748, 7),
     (0.6392054938847300, 0.2878307722444234)),
    ((5.1684384259452916, 4.0000000000000000, 4.8448605479076932, 7),
     (0.4083971950971300, 0.3230508265999140)),
    ((5.8632843152020939, 4.0000000000000000, 23.4897100232465128, 7),
     (0.7027892157150165, 0.2695458294720114)),
    ((5.7563333894119593, 9.0000000000000000, 7.3011356184221414, 7),
     (0.3907847562898140, 0.3289378590958607)),
    ((5.1083374030147883, 8.0000000000000000, 11.3597715314910968, 7),
     (0.4453918230630226, 0.3270125328628782)),
    ((8.3148984373785346, 9.0000000000000000, 4.2382336360058428, 7),
     (0.3599697419936892, 0.3311736677566133)),
    ((7.7299829867771086, 5.0000000000000000, 24.9236865714996476, 7),
     (0.6880520401649296, 0.3133621835227188)),
    ((8.2014603996082265, 4.0000000000000000, 11.5898408445204275, 7),
     (0.5584840865313621, 0.3395972173584274)),
    ((7.5956049192734421, 5.0000000000000000, 6.7982657472219277, 7),
     (0.4338251159286378, 0.3369357659148767)),
    ((8.3781863618289165, 9.0000000000000000, 8.0223578906755613, 7),
     (0.4105838750780343, 0.3420479634946895)),
    ((8.3001350007407968, 8.0000000000000000, 14.4335535476816563, 7),
     (0.5019186699595175, 0.3517244712842439)),
    ((8.2292707621139733, 9.0000000000000000, 6.3500223969273417, 7),
     (0.3879200067535180, 0.3380156966544355)),
    ((10.0000000000000000, 3.0000000000000000, 3.1152259635487924, 7),
     (0.4051415529429150, 0.3368646072213891)),
    ((9.7562679983086813, 3.0000000000000000, 14.8033847219145844, 7),
     (0.6814476281796187, 0.3179618099654896)),
    ((10.0000000000000000, 4.0000000000000000, 13.9016096097173900, 7),
     (0.6136634096115119, 0.3568983903902826)),
    ((10.0000000000000000, 8.0000000000000000, 19.3653583806798757, 7),
     (0.5632882254261189, 0.3740000000000000)),
    ((10.0000000000000000, 9.0000000000000000, 6.2184909658821841, 7),
     (0.3910588735223506, 0.3445663974459406)),
    ((10.0000000000000000, 8.0000000000000000, 13.8874930442766242, 7),
     (0.5055374095755961, 0.3687749860885532)),
    ((10.0000000000000000, 5.0000000000000000, 14.6890715994669296, 7),
     (0.5862646522729101, 0.3661588249401866)),
    ((10.0000000000000000, 5.0000000000000000, 24.2634423519120048, 7),
     (0.7045027023431640, 0.3534731152961760)),
    ((10.0000000000000000, 8.0000000000000000, 13.5181723549434167, 7),
     (0.5007362406142645, 0.3680363447098868)),
    ((2.7455640547144746, 2.0000000000000000, 5.5691106735491642, 6),
     (0.5147965927948814, 0.3594742707905298)),
    ((3.1452880891491906, 5.0000000000000000, 8.5958327172909996, 6),
     (0.4915655978299976, 0.3836139842529673)),
    ((2.5000000000000000, 4.0000000000000000, 4.9506791516086910, 6),
     (0.4364884940203847, 0.3603170842733587)),
    ((2.5000000000000000, 4.0000000000000000, 4.3832312494231553, 6),
     (0.4231250959239153, 0.3564868109336063)),
    ((2.5000000000000000, 2.0000000000000000, 3.3072822748362349, 6),
     (0.4339616288513916, 0.3458470682650791)),
    ((5.0455832680055721, 9.0000000000000000, 9.5919452486024408, 6),
     (0.4426976823901471, 0.3880493030502878)),
    ((5.5942845260414558, 9.0000000000000000, 10.1972012381662864, 6),
     (0.4500865164551526, 0.3950010499785958)),
    ((5.9888024672139428, 8.0000000000000000, 12.3059519561692294, 6),
     (0.4872574980467961, 0.4136999258156572)),
    ((5.4258509473962517, 5.0000000000000000, 8.0461568627031124, 6),
     (0.4837060124876794, 0.3993012908564904)),
    ((5.4058525432102122, 6.0000000000000000, 16.6357141095546055, 6),
     (0.5613340460279886, 0.4289299103499902)),
    ((5.3693642401195847, 5.0000000000000000, 7.3405738273399619, 6),
     (0.4695873659349679, 0.3934549781157231)),
    ((5.7020458215905094, 5.0000000000000000, 10.3256520517245409, 6),
     (0.5189311698950891, 0.4137392425047715)),
    ((5.4110963269588286, 6.0000000000000000, 5.2920348430950259, 6),
     (0.4094625687136605, 0.3702255025507858)),
    ((8.2429685366357628, 9.0000000000000000, 4.9002058653288101, 6),
     (0.3800667308386849, 0.3693561101855342)),
    ((8.2387545704858169, 9.0000000000000000, 5.9413301103786500, 6),
     (0.3940851904797918, 0.3790809702245060)),
    ((8.3956842438974792, 5.0000000000000000, 9.4615159687151351, 6),
     (0.5006508183439433, 0.4314784424608576)),
    ((10.0000000000000000, 5.0000000000000000, 12.7049634856464984, 6),
     (0.5274094231965362, 0.4628198539425860)),
    ((10.0000000000000000, 5.0000000000000000, 15.6753707607593995, 6),
     (0.5498899099449361, 0.4755391684234173)),
    ((10.0000000000000000, 6.0000000000000000, 3.5065733883684942, 6),
     (0.3769716076848171, 0.3696933421148326)),
    ((10.0000000000000000, 6.0000000000000000, 14.0639228795685085, 6),
     (0.5203515758376268, 0.4625141416465545)),
    ((10.0000000000000000, 7.0000000000000000, 3.1284434139449528, 6),
     (0.3632014271835780, 0.3603518752347706)),
    ((10.0000000000000000, 7.0000000000000000, 11.6324059143146474, 6),
     (0.4857175289017656, 0.4453349428787812)),
    ((9.0502631824660114, 7.0000000000000000, 17.0836769427597908, 6),
     (0.5287506410501914, 0.4599292192392070)),
    ((10.0000000000000000, 6.0000000000000000, 4.7369669473269207, 6),
     (0.4000655236518452, 0.3863911153573490)),
    ((9.4094025438018623, 7.0000000000000000, 6.2876602116865898, 6),
     (0.4147883501478832, 0.3954848563702238)),
    ((9.6333946040069609, 8.0000000000000000, 4.6230440017025254, 6),
     (0.3793131647314573, 0.3728258686141236)),
    ((9.0207701922757479, 7.0000000000000000, 13.4222450145776442, 6),
     (0.5060230048016112, 0.4473800806806889)),
    ((9.2631760968615406, 7.0000000000000000, 15.2332951824776668, 6),
     (0.5179404854021230, 0.4564421027270388)),
    ((3.3327820263877230, 7.0000000000000000, 16.1134199776775375, 5),
     (0.4994384335339968, 0.4913166627882885)),
    ((2.5000000000000000, 5.0000000000000000, 6.5436496028361315, 5),
     (0.4462906564432510, 0.4355063353928991)),
    ((2.5000000000000000, 6.0000000000000000, 15.5721297408543045, 5),
     (0.5138820422172288, 0.4876949957096056)),
    ((2.5000000000000000, 3.0000000000000000, 2.0000000000000000, 5),
     (0.3703000000000000, 0.3700000000000000)),
    ((2.8285591842433737, 9.0000000000000000, 21.4732588172908727, 5),
     (0.5043337880565358, 0.4951865962256489)),
    ((2.5000000000000000, 8.0000000000000000, 12.0201086586348378, 5),
     (0.4679648910008057, 0.4590236682506042)),
    ((2.5000000000000000, 9.0000000000000000, 14.4279044141537192, 5),
     (0.4757813786919992, 0.4673534742778455)),
    ((2.5000000000000000, 5.0000000000000000, 8.3802438034108171, 5),
     (0.4726826818375190, 0.4554229382371160)),
    ((3.3630794166715381, 5.0000000000000000, 2.7755096642090313, 5),
     (0.3688926051226334, 0.3743534718948429)),
    ((5.9271524261020545, 9.0000000000000000, 20.6031319524719265, 5),
     (0.4802472251615271, 0.5159774928137729)),
    ((5.3390799626536243, 8.0000000000000000, 16.6115749394242549, 5),
     (0.4789877671483087, 0.5049439528400183)),
    ((5.3473567647815976, 8.0000000000000000, 15.4121651982320493, 5),
     (0.4745806920664243, 0.5005081883465945)),
    ((5.3689506096346218, 7.0000000000000000, 7.0381659199243058, 5),
     (0.4134115471619250, 0.4348136096758073)),
    ((5.0633162392116553, 7.0000000000000000, 16.0133193348210305, 5),
     (0.4871641094183440, 0.5051543240966131)),
    ((5.9295528545359080, 7.0000000000000000, 7.5728134470480599, 5),
     (0.4185365312414376, 0.4442439765575030)),
    ((5.7279465595089096, 7.0000000000000000, 10.6681726339340361, 5),
     (0.4531835711184842, 0.4790487226808438)),
    ((5.6417821396686794, 6.0000000000000000, 9.5490168857451856, 5),
     (0.4561067615040472, 0.4783274892995489)),
    ((5.3443596420587474, 3.0000000000000000, 5.4304895609724859, 5),
     (0.4516333592896905, 0.4611095801939120)),
    ((7.7499092978023167, 4.0000000000000000, 4.2689337511750507, 5),
     (0.4017588344995081, 0.4334105720840665)),
    ((8.1454092289099975, 5.0000000000000000, 7.5456335290643839, 5),
     (0.4357892455698010, 0.4810623452292749)),
    ((7.9072536701593048, 6.0000000000000000, 10.7709862292896226, 5),
     (0.4538016350466874, 0.5021167554370949)),
    ((7.5925084922613122, 5.0000000000000000, 4.9335683444997134, 5),
     (0.4009033326671016, 0.4323309706149007)),
    ((7.6748726904108207, 5.0000000000000000, 3.5502452884794837, 5),
     (0.3759059629211176, 0.4014473524868083)),
    ((7.9919799870620540, 7.0000000000000000, 3.2837012487472252, 5),
     (0.3567830330164742, 0.3797850235174489)),
    ((9.3455991852868827, 7.0000000000000000, 17.4885217578818200, 5),
     (0.4649278192859861, 0.5374052696200110)),
    ((9.6595952185113880, 8.0000000000000000, 3.3572177484844636, 5),
     (0.3514360929632240, 0.3774175257667460)))

MUNSELL_COLOURS_TO_XYY = (
    (0.41515095, 0.51288165, 0.57024410),
    (0.38804358, 0.46299149, 0.31592072),
    (0.33491518, 0.36277402, 0.22128409),
    (0.39936353, 0.58547238, 0.64852094),
    (0.34767896, 0.41529220, 0.58706989),
    (0.33966055, 0.41527226, 0.07167165),
    (0.36265912, 0.47966922, 0.11068168),
    (0.35748002, 0.45915987, 0.27273590),
    (0.36348032, 0.48213512, 0.06293782),
    (0.30330033, 0.73038471, 0.05538644),
    (0.33159302, 0.43388935, 0.89380734),
    (0.31838794, 0.40167814, 0.05382145),
    (0.27202005, 0.83522048, 0.04995375),
    (0.31425413, 0.58372544, 0.04377268),
    (0.27634942, 0.75063178, 0.05211431),
    (0.25883700, 0.71096717, 0.01266934),
    (0.31405111, 0.53120144, 0.02111891),
    (0.33914454, 0.65636470, 0.71217401),
    (0.35328989, 0.63157007, 0.65497851),
    (0.32167873, 0.43862617, 0.30809910),
    (0.31168045, 0.62700640, 0.34717087),
    (0.31496017, 0.47530248, 0.67920304),
    (0.26882355, 0.70549119, 0.69614462),
    (0.31107787, 0.51188895, 0.58306925),
    (0.31254722, 0.34686238, 0.63343340),
    (0.30880402, 0.37157402, 0.08263161),
    (0.23582365, 0.72197618, 0.06667783),
    (0.29476305, 0.57521949, 0.23583791),
    (0.28891056, 0.61005165, 0.28191444),
    (0.17590584, 0.91365000, 0.23196178),
    (0.31292041, 0.37520740, 0.25538037),
    (0.22307972, 0.81536440, 0.26986020),
    (0.30648167, 0.48754769, 0.15098549),
    (0.30382174, 0.34089453, 0.84210967),
    (0.28517207, 0.38369148, 0.89445395),
    (0.20621151, 0.56369357, 0.77955867),
    (0.24658480, 0.49294784, 0.87271533),
    (0.22538285, 0.55646110, 0.60532773),
    (0.28500017, 0.38833563, 0.24045742),
    (0.19598037, 0.59002914, 0.29181101),
    (0.16437784, 0.59069112, 0.23370301),
    (0.17940333, 0.46639290, 0.06448045),
    (0.07553293, 0.55981543, 0.06406275),
    (0.27330162, 0.37048932, 0.11621278),
    (0.23251367, 0.40832841, 0.02585745),
    (0.05704598, 0.55990299, 0.01221862),
    (0.09405428, 0.51916421, 0.02268015),
    (0.06306305, 0.54336526, 0.03610370),
    (0.23250342, 0.41833342, 0.05599130),
    (0.22630523, 0.39163204, 0.05597116),
    (0.15858055, 0.42916814, 0.05259972),
    (0.07933408, 0.51474312, 0.30905098),
    (0.14028772, 0.46282023, 0.41589047),
    (0.29271668, 0.33531051, 0.37326792),
    (0.17253811, 0.43786778, 0.33686994),
    (0.09180367, 0.46823752, 0.05151176),
    (0.10903846, 0.44893518, 0.03595462),
    (0.24286930, 0.37094376, 0.04060119),
    (0.27771166, 0.34994832, 0.23574564),
    (0.05867972, 0.50502648, 0.19891229),
    (0.25930387, 0.37349411, 0.26874577),
    (0.12284826, 0.47211684, 0.21388094),
    (0.08906820, 0.48703791, 0.27058998),
    (0.27018357, 0.35138182, 0.76804186),
    (0.22062535, 0.38110738, 0.85084234),
    (0.26193025, 0.35814050, 0.86839733),
    (0.04310530, 0.45634623, 0.12074655),
    (0.16522669, 0.40881359, 0.18014875),
    (0.02517244, 0.46138968, 0.13173010),
    (0.23349872, 0.37536989, 0.14476492),
    (0.05119965, 0.46839242, 0.26212526),
    (0.23159950, 0.37207726, 0.20351563),
    (0.08301372, 0.45335265, 0.25304755),
    (0.20183026, 0.36561544, 0.39526058),
    (0.06340759, 0.37121187, 0.07975536),
    (0.16044634, 0.34707426, 0.10145605),
    (0.24416648, 0.33434737, 0.07774819),
    (0.28155768, 0.33248001, 0.89992977),
    (0.28105936, 0.33270880, 0.88937678),
    (0.25255297, 0.34594245, 0.87623351),
    (0.20616318, 0.34192146, 0.77176579),
    (0.21898553, 0.33335124, 0.85174026),
    (0.19119679, 0.33526743, 0.80792502),
    (0.29624596, 0.31950269, 0.96665647),
    (0.24328961, 0.31868567, 0.12931978),
    (0.10471116, 0.30938022, 0.15549815),
    (0.08624520, 0.30268915, 0.15900713),
    (0.10497041, 0.32451898, 0.22191645),
    (0.16894641, 0.33087742, 0.29312371),
    (0.16144965, 0.33133829, 0.34018592),
    (0.25864013, 0.31415379, 0.28205753),
    (0.07732853, 0.22846579, 0.08121964),
    (0.15795868, 0.26417318, 0.11377678),
    (0.06907834, 0.20994435, 0.07225730),
    (0.12862477, 0.25616557, 0.08539517),
    (0.05881481, 0.21256736, 0.07052095),
    (0.25058288, 0.29329096, 0.17796585),
    (0.18830894, 0.26192867, 0.13740285),
    (0.16840760, 0.25029878, 0.13934697),
    (0.19516480, 0.27716957, 0.51306785),
    (0.19935306, 0.27783329, 0.44060477),
    (0.26308512, 0.30462120, 0.52610451),
    (0.25324160, 0.30291555, 0.67153139),
    (0.15890128, 0.25325980, 0.59956247),
    (0.24841933, 0.29869620, 0.43833832),
    (0.20821330, 0.28356991, 0.52733609),
    (0.23939654, 0.29206110, 0.43144538),
    (0.18279859, 0.27122662, 0.52199238),
    (0.16449512, 0.24371038, 0.08686299),
    (0.16724393, 0.24366794, 0.06480227),
    (0.19881487, 0.26071106, 0.06927689),
    (0.09076654, 0.15277497, 0.06421355),
    (0.18253778, 0.23018215, 0.03460635),
    (0.16926303, 0.22496873, 0.06237928),
    (0.20398493, 0.25134710, 0.05473403),
    (0.28140041, 0.30378091, 0.23081828),
    (0.15231331, 0.21384066, 0.25883348),
    (0.14386953, 0.21327677, 0.41482428),
    (0.15935060, 0.22670722, 0.40114326),
    (0.10949743, 0.15034868, 0.15892888),
    (0.22674934, 0.26033997, 0.17110185),
    (0.20569472, 0.24048470, 0.15700695),
    (0.11359218, 0.15851929, 0.15851498),
    (0.13446868, 0.17456223, 0.15665285),
    (0.20295637, 0.23758918, 0.07464645),
    (0.16020908, 0.20160833, 0.11096053),
    (0.17946292, 0.22546056, 0.33406930),
    (0.19584886, 0.21874231, 0.05264774),
    (0.25950493, 0.28494406, 0.60260113),
    (0.22170777, 0.24928491, 0.29763974),
    (0.13564759, 0.16991066, 0.38138893),
    (0.23373145, 0.24171207, 0.08831548),
    (0.25339824, 0.26720506, 0.67917402),
    (0.29210338, 0.30192924, 0.75127547),
    (0.22958296, 0.24621680, 0.59738522),
    (0.12585350, 0.12764109, 0.16297312),
    (0.24227309, 0.25436998, 0.18624748),
    (0.23758242, 0.25457444, 0.66865194),
    (0.10476265, 0.09497701, 0.05235122),
    (0.12612865, 0.06066443, 0.02676646),
    (0.11705747, 0.03587748, 0.02951591),
    (0.12329050, 0.04415430, 0.02037758),
    (0.09139852, 0.01529466, 0.02045231),
    (0.13833192, 0.07953813, 0.02236117),
    (0.13361693, 0.10504399, 0.18414205),
    (0.12104740, 0.06862453, 0.15728175),
    (0.25249867, 0.24628189, 0.13536950),
    (0.11706407, 0.08706468, 0.18814811),
    (0.22549284, 0.21806210, 0.16192792),
    (0.15344950, 0.11674072, 0.15905692),
    (0.22358720, 0.20668864, 0.04253357),
    (0.12515256, 0.06568452, 0.04436879),
    (0.12125722, 0.06874820, 0.05533026),
    (0.10373316, 0.02774140, 0.06333516),
    (0.10925991, 0.04419045, 0.03405371),
    (0.15402461, 0.13042053, 0.28570417),
    (0.17573216, 0.16578146, 0.27364637),
    (0.27401103, 0.27401935, 0.23451177),
    (0.20759130, 0.19464274, 0.21940166),
    (0.17049737, 0.06465369, 0.05868583),
    (0.17064728, 0.02889150, 0.04372401),
    (0.16720380, 0.03196773, 0.03579761),
    (0.21031018, 0.15034168, 0.03888934),
    (0.16827351, 0.02413193, 0.03757647),
    (0.29178046, 0.29061931, 0.90323404),
    (0.24910224, 0.22648966, 0.59336016),
    (0.17601554, 0.05876060, 0.05160293),
    (0.16834537, 0.01686511, 0.04374851),
    (0.23182863, 0.19825806, 0.05291206),
    (0.16638758, 0.05075245, 0.03650792),
    (0.16028497, 0.01948654, 0.05046003),
    (0.24957235, 0.21006823, 0.31587613),
    (0.29306654, 0.28917618, 0.32466527),
    (0.28495343, 0.27687408, 0.81760638),
    (0.21441304, 0.13814375, 0.19716723),
    (0.20941829, 0.14321541, 0.24327119),
    (0.28541299, 0.27913907, 0.54006024),
    (0.29230469, 0.28656219, 0.52465762),
    (0.18804124, 0.08137467, 0.06580398),
    (0.22025958, 0.15180899, 0.06551257),
    (0.19309397, 0.06115047, 0.07873642),
    (0.19437258, 0.06326427, 0.06829742),
    (0.27887167, 0.24543217, 0.57450962),
    (0.27487624, 0.23376357, 0.46322748),
    (0.28356864, 0.25190050, 0.45980664),
    (0.30333596, 0.30005216, 0.66401066),
    (0.23835467, 0.11558036, 0.09827669),
    (0.23067198, 0.05028062, 0.07671426),
    (0.21902307, 0.05208443, 0.11065271),
    (0.22907253, 0.06719948, 0.06903321),
    (0.25361450, 0.16387485, 0.09900850),
    (0.28535713, 0.25114971, 0.08429109),
    (0.29701504, 0.28076672, 0.11652327),
    (0.24894294, 0.13513311, 0.07507850),
    (0.28976435, 0.23551078, 0.32030680),
    (0.28699217, 0.21227390, 0.38376156),
    (0.29423180, 0.24483482, 0.41568603),
    (0.27112866, 0.10892559, 0.09137276),
    (0.26932562, 0.11871922, 0.07456975),
    (0.28774446, 0.21149857, 0.06409553),
    (0.25815891, 0.05632389, 0.07763328),
    (0.28438514, 0.18361032, 0.08751006),
    (0.27466364, 0.11623324, 0.04459164),
    (0.26635689, 0.06032880, 0.04436654),
    (0.30526917, 0.29787617, 0.38438766),
    (0.26275899, 0.12295408, 0.30482710),
    (0.27733084, 0.16764806, 0.24584118),
    (0.27121622, 0.09967670, 0.21385417),
    (0.26547923, 0.10802713, 0.26515926),
    (0.29841781, 0.26325636, 0.25902873),
    (0.27412192, 0.13541072, 0.26778091),
    (0.30429530, 0.11611832, 0.04387000),
    (0.30157505, 0.08506396, 0.03768091),
    (0.31391169, 0.18564420, 0.48667459),
    (0.31670790, 0.22835511, 0.52829657),
    (0.31664956, 0.20454265, 0.45562827),
    (0.31300137, 0.23982828, 0.40210613),
    (0.31187872, 0.26667157, 0.33190218),
    (0.31537904, 0.21052765, 0.39335492),
    (0.31803143, 0.09273886, 0.17122630),
    (0.30594132, 0.18152717, 0.14244072),
    (0.31195968, 0.12089229, 0.15102095),
    (0.33618672, 0.17589268, 0.24249386),
    (0.34207627, 0.13875616, 0.24138597),
    (0.34605075, 0.11899797, 0.23580785),
    (0.31923003, 0.28291153, 0.25504488),
    (0.35136426, 0.12256902, 0.26410270),
    (0.33639641, 0.20777481, 0.23332748),
    (0.31464507, 0.30107880, 0.27040807),
    (0.32622786, 0.23679153, 0.28338647),
    (0.31964789, 0.19702337, 0.02988488),
    (0.33202416, 0.16293316, 0.16828902),
    (0.31883410, 0.26119414, 0.19149517),
    (0.34497302, 0.14740581, 0.17674791),
    (0.33396066, 0.13204228, 0.15759269),
    (0.32447663, 0.09207588, 0.03498261),
    (0.32823298, 0.08288658, 0.04740281),
    (0.34263192, 0.24928260, 0.04966462),
    (0.37863885, 0.14805570, 0.03133476),
    (0.36067287, 0.22508694, 0.03664306),
    (0.35583972, 0.20890369, 0.02874030),
    (0.34728299, 0.11402692, 0.01746108),
    (0.32940771, 0.22789278, 0.01489395),
    (0.31972567, 0.31122932, 0.53600948),
    (0.35012172, 0.29333067, 0.42147094),
    (0.37589661, 0.28507170, 0.66934047),
    (0.42549932, 0.23904177, 0.33329037),
    (0.34641765, 0.29725050, 0.38411768),
    (0.45441652, 0.21797623, 0.36276856),
    (0.41521602, 0.25989123, 0.39086156),
    (0.34780042, 0.29284040, 0.03605620),
    (0.45445510, 0.19822245, 0.03201793),
    (0.33858745, 0.30985450, 0.70004006),
    (0.41381262, 0.28393710, 0.60579167),
    (0.39278492, 0.29146870, 0.81034741),
    (0.33239612, 0.31251827, 0.19604738),
    (0.43846181, 0.29096381, 0.23141236),
    (0.40958022, 0.29719222, 0.48882871),
    (0.44399899, 0.29369509, 0.43379687),
    (0.40554919, 0.29723013, 0.16687769),
    (0.42007003, 0.28930815, 0.16729330),
    (0.52108329, 0.25574146, 0.13999526),
    (0.37638010, 0.30728007, 0.34070289),
    (0.36495307, 0.30801481, 0.20910915),
    (0.42566912, 0.29564012, 0.28217939),
    (0.38537971, 0.31745807, 0.82116554),
    (0.37201534, 0.31965197, 0.79705828),
    (0.55136347, 0.28138892, 0.19712193),
    (0.53899416, 0.29048788, 0.25823634),
    (0.43854811, 0.31033170, 0.27612362),
    (0.35589069, 0.31655370, 0.24649473),
    (0.60150190, 0.26287828, 0.27670596),
    (0.49631592, 0.30111191, 0.04570504),
    (0.60338354, 0.27468340, 0.04600213),
    (0.57619776, 0.31554717, 0.14073356),
    (0.65681487, 0.27970869, 0.16409107),
    (0.40414547, 0.32310724, 0.13347887),
    (0.68743116, 0.27762719, 0.14148314),
    (0.39097754, 0.32893313, 0.75691217),
    (0.44274163, 0.32668916, 0.63163346),
    (0.35143720, 0.32860694, 0.80679695),
    (0.55200335, 0.34090583, 0.13240521),
    (0.43719237, 0.33673056, 0.18274766),
    (0.36439573, 0.33015311, 0.87403366),
    (0.34753957, 0.32343836, 0.93909237),
    (0.38880059, 0.33783693, 0.70408740),
    (0.40006019, 0.33663147, 0.07790261),
    (0.67248369, 0.32330365, 0.07220649),
    (0.64354918, 0.33973639, 0.08803122),
    (0.39181364, 0.34469480, 0.72252254),
    (0.49958346, 0.36703778, 0.64322822),
    (0.60231065, 0.36168845, 0.16068191),
    (0.50023387, 0.36789369, 0.58191450),
    (0.54791150, 0.34892913, 0.02263783),
    (0.48794187, 0.38293202, 0.20665660),
    (0.42836733, 0.35891726, 0.14365536),
    (0.41518890, 0.35495306, 0.14408043),
    (0.46542405, 0.34082576, 0.02079253),
    (0.44637245, 0.38945422, 0.69298338),
    (0.45404134, 0.39647154, 0.69233629),
    (0.48546253, 0.41305920, 0.59936297),
    (0.47978428, 0.39825981, 0.20940783),
    (0.46785973, 0.39297126, 0.20053901),
    (0.51331905, 0.41265964, 0.21747039),
    (0.41770486, 0.37303747, 0.24430738),
    (0.37481816, 0.36536787, 0.78442772),
    (0.35992953, 0.34813048, 0.94470358),
    (0.39620152, 0.37951049, 0.68657974),
    (0.51299224, 0.43528921, 0.15241166),
    (0.52529122, 0.46205863, 0.20512581),
    (0.54693353, 0.47444863, 0.20705936),
    (0.38249101, 0.37271251, 0.24466996),
    (0.52465528, 0.46381752, 0.27037022),
    (0.36332827, 0.36042625, 0.41747900),
    (0.49103786, 0.44741782, 0.37625337),
    (0.53260672, 0.46085797, 0.38175879),
    (0.39947636, 0.38607101, 0.30024818),
    (0.41566751, 0.39590836, 0.41203391),
    (0.38379951, 0.37533829, 0.49499836),
    (0.50184813, 0.44587964, 0.46556628),
    (0.51381974, 0.45512612, 0.47222976),
    (0.49758967, 0.49049612, 0.45242214),
    (0.43935016, 0.43103452, 0.23259639),
    (0.37285524, 0.37178029, 0.05690988),
    (0.50597795, 0.49533702, 0.71175266),
    (0.46625888, 0.45805354, 0.60604474),
    (0.47877996, 0.46893263, 0.71437706),
    (0.48009865, 0.45955664, 0.16263768),
    (0.37334852, 0.37797455, 0.16515524),
    (0.47856178, 0.50475320, 0.58732427),
    (0.47119689, 0.49876821, 0.65483021),
    (0.40983957, 0.43186708, 0.47430500),
    (0.42018308, 0.44575131, 0.40534100),
    (0.45841191, 0.48304710, 0.36340530),
    (0.45040867, 0.47371048, 0.34766863),
    (0.45990386, 0.46807933, 0.05857873),
    (0.39793427, 0.42894800, 0.13930127),
    (0.43789903, 0.48337683, 0.18110313),
    (0.45785880, 0.50623924, 0.25901491),
    (0.39985008, 0.43116457, 0.20152195),
    (0.37226961, 0.39738120, 0.23381716),
    (0.35899980, 0.38250951, 0.36788985),
    (0.46379107, 0.53608140, 0.45228296),
    (0.35180708, 0.37798088, 0.55904288))

MUNSELL_GREYS_TO_XYY = (
    (0.31006, 0.31616, 0.00000000),
    (0.31006, 0.31616, 0.00473582),
    (0.31006, 0.31616, 0.00961944),
    (0.31006, 0.31616, 0.01545756),
    (0.31006, 0.31616, 0.02293343),
    (0.31006, 0.31616, 0.03261914),
    (0.31006, 0.31616, 0.04498800),
    (0.31006, 0.31616, 0.06042690),
    (0.31006, 0.31616, 0.07924864),
    (0.31006, 0.31616, 0.10170428),
    (0.31006, 0.31616, 0.12799549),
    (0.31006, 0.31616, 0.15828689),
    (0.31006, 0.31616, 0.19271844),
    (0.31006, 0.31616, 0.23141772),
    (0.31006, 0.31616, 0.27451233),
    (0.31006, 0.31616, 0.32214224),
    (0.31006, 0.31616, 0.37447210),
    (0.31006, 0.31616, 0.43170362),
    (0.31006, 0.31616, 0.49408790),
    (0.31006, 0.31616, 0.56193781),
    (0.31006, 0.31616, 0.63564030),
    (0.31006, 0.31616, 0.71566876),
    (0.31006, 0.31616, 0.80259539),
    (0.31006, 0.31616, 0.89710353),
    (0.31006, 0.31616, 1.00000000))

XYY_TO_MUNSELL_SPECIFICATIONS = (
    ((0.4151509500000000, 0.5128816500000000, 0.5702441000000000),
     (2.4974254984450397, 7.9653798278107182, 11.928549858473941, 4)),
    ((0.3880435800000000, 0.4629914900000000, 0.3159207200000000),
     (2.5006439954892556, 6.1977947932238182, 6.923610697067909, 4)),
    ((0.3349151800000000, 0.3627740200000000, 0.2212840900000000),
     (2.4903091620270921, 5.3119569245226996, 1.998638034101546, 4)),
    ((0.3993635300000000, 0.5854723800000000, 0.6485209400000000),
     (5.6117848436675910, 8.4027565252844596, 18.567138539062916, 4)),
    ((0.3476789600000000, 0.4152922000000000, 0.5870698900000000),
     (5.8509402259358456, 8.0626386670645687, 5.784142504579139, 4)),
    ((0.3396605500000000, 0.4152722600000000, 0.0716716500000000),
     (5.7816682069236824, 3.1748039447941760, 3.349385637554362, 4)),
    ((0.3626591200000000, 0.4796692200000000, 0.1106816800000000),
     (5.4883266507471262, 3.8994120308445170, 5.762709554941629, 4)),
    ((0.3574800200000000, 0.4591598700000000, 0.2727359000000000),
     (5.8091607027319458, 5.8169751521608619, 6.666016191540733, 4)),
    ((0.3634803200000000, 0.4821351200000000, 0.0629378200000000),
     (5.2106889846935189, 2.9770363655668297, 5.141836619699955, 4)),
    ((0.3183879400000000, 0.4016781400000000, 0.0538214500000000),
     (8.1126705827645740, 2.7489429215052237, 3.164465784963284, 4)),
    ((0.3142541300000000, 0.5837254400000000, 0.0437726800000000),
     (8.0454960027236311, 2.4630649861246052, 7.500061207356352, 4)),
    ((0.3140511100000000, 0.5312014400000000, 0.0211189100000000),
     (7.5930163631603786, 1.5750743888437724, 4.626174843109444, 4)),
    ((0.3216787300000000, 0.4386261700000000, 0.3080991000000000),
     (9.0152103567385531, 6.1312711912840276, 6.805293664943519, 4)),
    ((0.3116804500000000, 0.6270064000000000, 0.3471708700000000),
     (9.0429810192754712, 6.4540531798090557, 17.010229922329884, 4)),
    ((0.3149601700000000, 0.4753024800000000, 0.6792030400000000),
     (9.9147922785103972, 8.5643879483802650, 11.130516599293573, 4)),
    ((0.3110778700000000, 0.5118889500000000, 0.5830692500000000),
     (9.9600028417318498, 8.0396827436851854, 13.199185802941963, 4)),
    ((0.3125472200000000, 0.3468623800000000, 0.6334334000000000),
     (9.8852069946025658, 8.3213426295629134, 2.070847452019443, 4)),
    ((0.3088040200000000, 0.3715740200000000, 0.0826316100000000),
     (9.9880916197181513, 3.4007871110002572, 2.568453642887343, 4)),
    ((0.2358236500000000, 0.7219761800000000, 0.0666778300000000),
     (9.9984874163608151, 3.0639156530856830, 13.513388270694880, 4)),
    ((0.2947630500000000, 0.5752194900000001, 0.2358379100000000),
     (9.9976375658465031, 5.4614654126991260, 12.753028500364775, 4)),
    ((0.2889105600000000, 0.6100516499999999, 0.2819144400000000),
     (9.9978537677920656, 5.9008140943398812, 15.244070244743728, 4)),
    ((0.3129204100000000, 0.3752074000000000, 0.2553803700000000),
     (9.7548278524788188, 5.6536473977778643, 3.410784496514246, 4)),
    ((0.3064816700000000, 0.4875476900000000, 0.1509854900000000),
     (9.8616202679251828, 4.4878641370082102, 7.675705535447796, 4)),
    ((0.3038217400000000, 0.3408945300000000, 0.8421096700000000),
     (3.2303908627800304, 9.3451636022528515, 3.440344036721676, 3)),
    ((0.2253828500000000, 0.5564611000000000, 0.6053277300000000),
     (2.7893896011049968, 8.1660999142444801, 20.865361708231859, 3)),
    ((0.2850001700000000, 0.3883356300000000, 0.2404574200000000),
     (3.2217895159709276, 5.5077419322120686, 5.463072825387044, 3)),
    ((0.1959803700000000, 0.5900291400000000, 0.2918110100000000),
     (2.6212805877659342, 5.9893806530207669, 19.362394859916822, 3)),
    ((0.1643778400000000, 0.5906911200000000, 0.2337030100000000),
     (3.2866319103339858, 5.4398925329719443, 19.854410847201201, 3)),
    ((0.1794033300000000, 0.4663929000000000, 0.0644804500000000),
     (5.7282498562431972, 3.0132952817775913, 10.742779986622125, 3)),
    ((0.0755329300000000, 0.5598154300000000, 0.0640627500000000),
     (5.3447124424617432, 3.0035376238327340, 18.896804785539025, 3)),
    ((0.2733016200000000, 0.3704893200000000, 0.1162127800000000),
     (5.7450040846850658, 3.9875599357896836, 4.223830433964055, 3)),
    ((0.2325136700000000, 0.4083284100000000, 0.0258574500000000),
     (5.7167484363405752, 1.8040377069834592, 4.877534962939156, 3)),
    ((0.0570459800000000, 0.5599029900000000, 0.0122186200000000),
     (5.3134014009438513, 1.0305079836154281, 8.042375156649795, 3)),
    ((0.0940542800000000, 0.5191642100000000, 0.0226801500000000),
     (5.7582243025990465, 1.6541936507188186, 9.505587064089532, 3)),
    ((0.0630630500000000, 0.5433652600000000, 0.0361037000000000),
     (5.9824401855202680, 2.2109766905487875, 14.800767502406950, 3)),
    ((0.2325034200000000, 0.4183334200000000, 0.0559913000000000),
     (5.4561315302616542, 2.8055682870311611, 6.645891042225424, 3)),
    ((0.2263052300000000, 0.3916320400000000, 0.0559711600000000),
     (7.8355832971785766, 2.8050499614961404, 6.238116104100503, 3)),
    ((0.1585805500000000, 0.4291681400000000, 0.0525997200000000),
     (8.2773608986748037, 2.7163438063692538, 10.349070921928152, 3)),
    ((0.1402877200000000, 0.4628202300000000, 0.4158904700000000),
     (8.3193983733801780, 6.9718015635272170, 23.511305166143138, 3)),
    ((0.2927166800000000, 0.3353105100000000, 0.3732679200000000),
     (8.4446073967845603, 6.6574922739334328, 2.410001695449483, 3)),
    ((0.1725381100000000, 0.4378677800000000, 0.3368699400000000),
     (8.3062536616909277, 6.3711907135696917, 17.505739300252280, 3)),
    ((0.0918036700000000, 0.4682375200000000, 0.0515117600000000),
     (8.1366760619167078, 2.6868572712647723, 14.647241441839769, 3)),
    ((0.1090384600000000, 0.4489351800000000, 0.0359546200000000),
     (8.4804731414469181, 2.2057044721731964, 11.877741549341193, 3)),
    ((0.2428693000000000, 0.3709437600000000, 0.0406011900000000),
     (8.4528136469348514, 2.3630505444499383, 4.606241088392112, 3)),
    ((0.2777116600000000, 0.3499483200000000, 0.2357456400000000),
     (8.3053924744903362, 5.4605354972582942, 3.902088380849585, 3)),
    ((0.0586797200000000, 0.5050264800000001, 0.1989122900000000),
     (8.1854332748360292, 5.0699333724644822, 28.120377018384665, 3)),
    ((0.2593038700000000, 0.3734941100000000, 0.2687457700000000),
     (7.5395384410319366, 5.7799955833061949, 6.641297576530984, 3)),
    ((0.1228482600000000, 0.4721168400000000, 0.2138809400000000),
     (7.9597279338124061, 5.2335977016179758, 20.290421956393593, 3)),
    ((0.0890682000000000, 0.4870379100000000, 0.2705899800000000),
     (8.4291971680074340, 5.7971282967167381, 26.464916551909961, 3)),
    ((0.2701835700000000, 0.3513818200000000, 0.7680418600000000),
     (0.0029116335013413, 9.0051614807808651, 6.052147995993515, 2)),
    ((0.0431053000000000, 0.4563462300000000, 0.1207465500000000),
     (9.9090940657999482, 4.0578050043303753, 25.772345770419661, 3)),
    ((0.1652266900000000, 0.4088135900000000, 0.1801487500000000),
     (9.9976284265517368, 4.8536959511881248, 13.711432917514352, 3)),
    ((0.2334987200000000, 0.3753698900000000, 0.1447649200000000),
     (9.2805000536689342, 4.4042068716055978, 6.996679713939470, 3)),
    ((0.2315995000000000, 0.3720772600000000, 0.2035156300000000),
     (9.9968489688298856, 5.1210462040554292, 7.946601187910490, 3)),
    ((0.0830137200000000, 0.4533526500000000, 0.2530475500000000),
     (9.9995652694576354, 5.6311864544796251, 26.172410781131642, 3)),
    ((0.2018302600000000, 0.3656154400000000, 0.3952605800000000),
     (2.4927553349648868, 6.8222787267061182, 12.637144808932388, 2)),
    ((0.0634075900000000, 0.3712118700000000, 0.0797553600000000),
     (2.4983577898662457, 3.3435595770060069, 19.166881554015059, 2)),
    ((0.1604463400000000, 0.3470742600000000, 0.1014560500000000),
     (3.2889502832179485, 3.7457477699374353, 10.316221955381685, 2)),
    ((0.2441664800000000, 0.3343473700000000, 0.0777481900000000),
     (3.0834483697271065, 3.3027889692997099, 4.031478930213133, 2)),
    ((0.2810593600000000, 0.3327088000000000, 0.8893767800000000),
     (2.5569047038313286, 9.5505578383904854, 8.259294961927313, 2)),
    ((0.2432896100000000, 0.3186856700000000, 0.1293197800000000),
     (5.9613749867002497, 4.1861233500163211, 4.311062434737238, 2)),
    ((0.1047111600000000, 0.3093802200000000, 0.1554981500000000),
     (5.6283551180725855, 4.5472024424749877, 16.561283650587430, 2)),
    ((0.0862452000000000, 0.3026891500000000, 0.1590071300000000),
     (5.8537976763848487, 4.5925997102794662, 18.829597868809777, 2)),
    ((0.1049704100000000, 0.3245189800000000, 0.2219164500000000),
     (5.1450898946865120, 5.3185755017128393, 18.972215318595008, 2)),
    ((0.1689464100000000, 0.3308774200000000, 0.2931237100000000),
     (5.2908544585019168, 6.0000000000000000, 13.596495394774379, 2)),
    ((0.1614496500000000, 0.3313382900000000, 0.3401859200000000),
     (5.4160474347058862, 6.3980311319848546, 15.175835231610054, 2)),
    ((0.2586401300000000, 0.3141537900000000, 0.2820575300000000),
     (8.2025559167724538, 5.9021079413020505, 4.026097765262806, 2)),
    ((0.1579586800000000, 0.2641731800000000, 0.1137767800000000),
     (0.0125362353214431, 3.9490817702416789, 9.188897738692464, 1)),
    ((0.1286247700000000, 0.2561655700000000, 0.0853951700000000),
     (9.2612367834642395, 3.4545177235566826, 10.589381636411930, 2)),
    ((0.2505828800000000, 0.2932909600000000, 0.1779658500000000),
     (3.2410793031343310, 4.8276508362373640, 3.743365882969607, 1)),
    ((0.1883089400000000, 0.2619286700000000, 0.1374028500000000),
     (2.5158920949660768, 4.3022043533741527, 7.397111178524947, 1)),
    ((0.1684076000000000, 0.2502987800000000, 0.1393469700000000),
     (2.5131427837361731, 4.3294708614782440, 8.858704782948033, 1)),
    ((0.1951648000000000, 0.2771695700000000, 0.5130678500000000),
     (2.5013789364411032, 7.6200943072565934, 10.879242856664603, 1)),
    ((0.1993530600000000, 0.2778332900000000, 0.4406047700000000),
     (2.5013473966576427, 7.1449996623226202, 10.096343120964025, 1)),
    ((0.2630851200000000, 0.3046212000000000, 0.5261045100000000),
     (2.6086704821311635, 7.7009394844971908, 4.240922869043813, 1)),
    ((0.2532416000000000, 0.3029155500000000, 0.6715313900000000),
     (2.4970799555123779, 8.5244556402579850, 5.361811270572150, 1)),
    ((0.2484193300000000, 0.2986962000000000, 0.4383383200000000),
     (2.5045629814751091, 7.1293721373614156, 5.419670697065572, 1)),
    ((0.2082133000000000, 0.2835699100000000, 0.5273360900000000),
     (2.5131771577412465, 7.7085098474609053, 9.614914133684164, 1)),
    ((0.2393965400000000, 0.2920611000000000, 0.4314453800000000),
     (3.2651896190197771, 7.0815325065875827, 6.220404225264666, 1)),
    ((0.1827985900000000, 0.2712266200000000, 0.5219923800000000),
     (2.5035978550267846, 7.6755794342432786, 12.259534701163053, 1)),
    ((0.1644951200000000, 0.2437103800000000, 0.0868629900000000),
     (2.5134247452410996, 3.4825807614595732, 7.766187653700966, 1)),
    ((0.1672439300000000, 0.2436679400000000, 0.0648022700000000),
     (2.5011108303709673, 3.0207831644481167, 6.995204172520695, 1)),
    ((0.1988148700000000, 0.2607110600000000, 0.0692768900000000),
     (3.0402893156428767, 3.1223174742573971, 5.200639560347585, 1)),
    ((0.1825377800000000, 0.2301821500000000, 0.0346063500000000),
     (5.5976576256856170, 2.1572823771224261, 5.162764461216634, 1)),
    ((0.1692630300000000, 0.2249687300000000, 0.0623792800000000),
     (5.0963427942773887, 2.9637551635524759, 6.858793778789698, 1)),
    ((0.2039849300000000, 0.2513471000000000, 0.0547340300000000),
     (5.1969692716096816, 2.7729516729961743, 4.559422294586083, 1)),
    ((0.2814004100000000, 0.3037809100000000, 0.2308182800000000),
     (5.4965612691575760, 5.4105513508464149, 2.000813675440119, 1)),
    ((0.1523133100000000, 0.2138406600000000, 0.2588334800000000),
     (5.8564961151865536, 5.6866675850563144, 13.288114571635580, 1)),
    ((0.1593506000000000, 0.2267072200000000, 0.4011432600000000),
     (5.2738491300453916, 6.8653966593629132, 14.101257686332218, 1)),
    ((0.2267493400000000, 0.2603399700000000, 0.1711018500000000),
     (7.8734843119339804, 4.7444381618873681, 5.598894447763097, 1)),
    ((0.2056947200000000, 0.2404847000000000, 0.1570069500000000),
     (8.3151048023527618, 4.5668004089848395, 7.088011481510175, 1)),
    ((0.1135921800000000, 0.1585192900000000, 0.1585149800000000),
     (7.8472605398133766, 4.5862706767212797, 16.224610174484769, 1)),
    ((0.1344686800000000, 0.1745622300000000, 0.1566528500000000),
     (8.3830368679291833, 4.5622116144353626, 13.972942153711159, 1)),
    ((0.2029563700000000, 0.2375891800000000, 0.0746464500000000),
     (7.8564355331771640, 3.2383503193149088, 5.358491628406157, 1)),
    ((0.1602090800000000, 0.2016083300000000, 0.1109605300000000),
     (7.6347078681738623, 3.9039238069235207, 9.768736791650276, 1)),
    ((0.1794629200000000, 0.2254605600000000, 0.3340693000000000),
     (7.6477212875072365, 6.3483962769765867, 11.704570140652267, 1)),
    ((0.1958488600000000, 0.2187423100000000, 0.0526477400000000),
     (9.9898352731700779, 2.7176352217817406, 5.416015499804008, 1)),
    ((0.2595049300000000, 0.2849440600000000, 0.6026011300000000),
     (9.1958881934475922, 8.1507829370037523, 5.072514995394412, 1)),
    ((0.2217077700000000, 0.2492849100000000, 0.2976397400000000),
     (9.9993390910882241, 6.0406947699165405, 7.762161515915440, 1)),
    ((0.2337314500000000, 0.2417120700000000, 0.0883154800000000),
     (2.8725008274335195, 3.5100389169220145, 4.494437475944106, 10)),
    ((0.2921033800000000, 0.3019292400000000, 0.7512754699999999),
     (2.5573566809324166, 8.9248766535028565, 2.493240458692857, 10)),
    ((0.1258535000000000, 0.1276410900000000, 0.1629731200000000),
     (2.5007360838933224, 4.6431603215843316, 18.831399277234545, 10)),
    ((0.2422730900000000, 0.2543699800000000, 0.1862474800000000),
     (2.4933583981166230, 4.9254430182491289, 5.417488749765833, 10)),
    ((0.2524986700000000, 0.2462818900000000, 0.1353695000000000),
     (5.9478312821937074, 4.2734224989493885, 4.896630871834125, 10)),
    ((0.2254928400000000, 0.2180621000000000, 0.1619279200000000),
     (5.5322841596577277, 4.6299109349866532, 7.752252929326820, 10)),
    ((0.1534495000000000, 0.1167407200000000, 0.1590569200000000),
     (5.9220754173118184, 4.5932393336579818, 19.565654953046173, 10)),
    ((0.2235872000000000, 0.2066886400000000, 0.0425335700000000),
     (5.9517759003047477, 2.4246350396441594, 4.953911358694802, 10)),
    ((0.1757321600000000, 0.1657814600000000, 0.2736463700000000),
     (5.2008216960854803, 5.8253671682452843, 15.513073935981227, 10)),
    ((0.2740110300000000, 0.2740193500000000, 0.2345117700000000),
     (5.8906638458196703, 5.4480790029813448, 3.774035430896981, 10)),
    ((0.2075913000000000, 0.1946427400000000, 0.2194016600000000),
     (5.8176239302630783, 5.2921858536043249, 11.318160969256796, 10)),
    ((0.2103101800000000, 0.1503416800000000, 0.0388893400000000),
     (8.3733696479385600, 2.3066883689858764, 8.631037795394968, 10)),
    ((0.2318286300000000, 0.1982580600000000, 0.0529120600000000),
     (7.7121590219520009, 2.7247287040518557, 5.730016018596034, 10)),
    ((0.2495723500000000, 0.2100682300000000, 0.3158761300000000),
     (9.9948213055847646, 6.1974183872008091, 10.969746297783991, 10)),
    ((0.2930665400000000, 0.2891761800000000, 0.3246652700000000),
     (9.1126758970844079, 6.2709966101935857, 2.750829451410893, 10)),
    ((0.2144130400000000, 0.1381437500000000, 0.1971672300000000),
     (9.9987133328585820, 5.0503674185535150, 19.168834467026883, 10)),
    ((0.2094182900000000, 0.1432154100000000, 0.2432711900000000),
     (9.3829287452992816, 5.5356649367693560, 18.821027597932278, 10)),
    ((0.2854129900000000, 0.2791390700000000, 0.5400602400000000),
     (9.0020804073489380, 7.7860618086454938, 4.453888691443739, 10)),
    ((0.2923046900000000, 0.2865621900000000, 0.5246576200000000),
     (9.9912356133547320, 7.6920309739694908, 3.652324068875179, 10)),
    ((0.1880412400000000, 0.0813746700000000, 0.0658039800000000),
     (9.0425654223037455, 3.0439259368754286, 22.292781603620266, 10)),
    ((0.2202595800000000, 0.1518089900000000, 0.0655125700000000),
     (9.4618219859335095, 3.0372188057325871, 10.554387963579108, 10)),
    ((0.2788716700000000, 0.2454321700000000, 0.5745096200000001),
     (2.5015728390355108, 7.9902138330733292, 8.383060885731012, 9)),
    ((0.2748762400000000, 0.2337635700000000, 0.4632274800000000),
     (2.4996651744668252, 7.2983010268318242, 9.502846729929050, 9)),
    ((0.2835686400000000, 0.2519005000000000, 0.4598066400000000),
     (2.8618699795852098, 7.2754260032575946, 7.466115678753070, 9)),
    ((0.3033359600000000, 0.3000521600000000, 0.6640106600000000),
     (3.0821290942108526, 8.4850005681408049, 2.493500289871143, 9)),
    ((0.2383546700000000, 0.1155803600000000, 0.0982766900000000),
     (2.5037499210886072, 3.6906677571198614, 19.779266412450269, 9)),
    ((0.2536145000000000, 0.1638748500000000, 0.0990085000000000),
     (2.4992857563119486, 3.7034489184917470, 12.087654980113633, 9)),
    ((0.2853571300000000, 0.2511497100000000, 0.0842910900000000),
     (2.4890318095296315, 3.4331943344980584, 3.381492920102487, 9)),
    ((0.2970150400000000, 0.2807667200000000, 0.1165232700000000),
     (2.8365261300922739, 3.9924266007584350, 2.000804943904234, 9)),
    ((0.2489429400000000, 0.1351331100000000, 0.0750785000000000),
     (2.8801166239961695, 3.2474344484707478, 14.610390101910370, 9)),
    ((0.2897643500000000, 0.2355107800000000, 0.3203068000000000),
     (5.1644159196302653, 6.2346627202329179, 9.113326052349766, 9)),
    ((0.2869921700000000, 0.2122739000000000, 0.3837615600000000),
     (5.5001367381271749, 6.7368412342989172, 13.153893654633693, 9)),
    ((0.2942318000000000, 0.2448348200000000, 0.4156860300000000),
     (5.5355820929681876, 6.9703425464446100, 8.892567135407102, 9)),
    ((0.2711286600000000, 0.1089255900000000, 0.0913727600000000),
     (5.5896397325517455, 3.5668608875999919, 22.756075736806582, 9)),
    ((0.2693256200000000, 0.1187192200000000, 0.0745697500000000),
     (5.2816804395644468, 3.2367339016487255, 18.731686204369552, 9)),
    ((0.2877444600000000, 0.2114985700000000, 0.0640955300000000),
     (5.1732976532712982, 3.0043049868755096, 6.229409296844157, 9)),
    ((0.2843851400000000, 0.1836103200000000, 0.0875100600000000),
     (5.5527847263028818, 3.4948508627605213, 10.076128044498652, 9)),
    ((0.2746636400000000, 0.1162332400000000, 0.0445916400000000),
     (5.7295438817735267, 2.4880342757163407, 15.986078477134842, 9)),
    ((0.3052691700000000, 0.2978761700000000, 0.3843876600000000),
     (5.0697134888906703, 6.7415332853801191, 2.219542488732086, 9)),
    ((0.2773308400000000, 0.1676480600000000, 0.2458411800000000),
     (5.3830683593123041, 5.5609978860251674, 18.302040299981690, 9)),
    ((0.2984178100000000, 0.2632563600000000, 0.2590287300000000),
     (5.3573489710687028, 5.6885266519601112, 5.255330957728729, 9)),
    ((0.3042953000000000, 0.1161183200000000, 0.0438700000000000),
     (8.2848827562071747, 2.4660497719128411, 17.239324057538755, 9)),
    ((0.3167079000000000, 0.2283551100000000, 0.5282965700000000),
     (8.1437019218335216, 7.7144058793057653, 13.532589823660204, 9)),
    ((0.3166495600000000, 0.2045426500000000, 0.4556282700000000),
     (8.0950673003541418, 7.2473398239288285, 17.337282390950395, 9)),
    ((0.3130013700000000, 0.2398282800000000, 0.4021061300000000),
     (7.8286696950237911, 6.8724169794156120, 10.705903656469590, 9)),
    ((0.3118787200000000, 0.2666715700000000, 0.3319021800000000),
     (7.7997180017353251, 6.3306783023663034, 6.211065875095611, 9)),
    ((0.3153790400000000, 0.2105276500000000, 0.3933549200000000),
     (8.0441828325585440, 6.8082262964881988, 15.556673432533850, 9)),
    ((0.3059413200000000, 0.1815271700000000, 0.1424407200000000),
     (7.6125352376790900, 4.3723674448096084, 14.168811791437310, 9)),
    ((0.3119596800000000, 0.1208922900000000, 0.1510209500000000),
     (8.1692313405838135, 4.4883347140084675, 27.235344441294184, 9)),
    ((0.3361867200000000, 0.1758926800000000, 0.2424938600000000),
     (9.6020077678142357, 5.5279706708857370, 20.580420190036015, 9)),
    ((0.3420762700000000, 0.1387561600000000, 0.2413859700000000),
     (9.6633397378583119, 5.5169784439059200, 29.047422373609325, 9)),
    ((0.3192300300000000, 0.2829115300000000, 0.2550448800000000),
     (9.9976373285878672, 5.6504248984821048, 4.215952139435246, 9)),
    ((0.3363964100000000, 0.2077748100000000, 0.2333274800000000),
     (9.9973290603118059, 5.4360854689605906, 14.778819186976173, 9)),
    ((0.3146450700000000, 0.3010788000000000, 0.2704080700000000),
     (9.9981812895483202, 5.7954414845493361, 1.989415064944841, 9)),
    ((0.3262278600000000, 0.2367915300000000, 0.2833864700000000),
     (9.4957863462098757, 5.9141054746428159, 10.808393081380504, 9)),
    ((0.3196478900000000, 0.1970233700000000, 0.0298848800000000),
     (9.8255286116794061, 1.9759993085792666, 7.066915765354878, 9)),
    ((0.3320241600000000, 0.1629331600000000, 0.1682890200000000),
     (9.3833430362731995, 4.7097386903129328, 20.001567691188572, 9)),
    ((0.3188341000000000, 0.2611941400000000, 0.1914951700000000),
     (9.1151365491745828, 4.9860254232389654, 5.883351646224200, 9)),
    ((0.3449730200000000, 0.1474058100000000, 0.1767479100000000),
     (9.9993661799172529, 4.8130329637008931, 24.745869339266416, 9)),
    ((0.3339606600000000, 0.1320422800000000, 0.1575926900000000),
     (9.3788217006559904, 4.5743767927679624, 26.296103195549775, 9)),
    ((0.3426319200000000, 0.2492826000000000, 0.0496646200000000),
     (2.5000378111202792, 2.6357603664122133, 4.711601146262771, 8)),
    ((0.3786388500000000, 0.1480557000000000, 0.0313347600000000),
     (2.8899890484960267, 2.0337680336115653, 13.996029826620704, 8)),
    ((0.3606728700000000, 0.2250869400000000, 0.0366430600000000),
     (3.4356425026035353, 2.2299191499236484, 6.721902205502408, 8)),
    ((0.3558397200000000, 0.2089036900000000, 0.0287403000000000),
     (2.9923928706346814, 1.9289333829363795, 7.195747540270221, 8)),
    ((0.3472829900000000, 0.1140269200000000, 0.0174610800000000),
     (2.5005843682203910, 1.3726888338846817, 14.170766991414973, 8)),
    ((0.3294077100000000, 0.2278927800000000, 0.0148939500000000),
     (2.6107190169889272, 1.2137703916290237, 3.348897655453758, 8)),
    ((0.3197256700000000, 0.3112293200000000, 0.5360094800000000),
     (5.1656963312103521, 7.7615027982277569, 2.141042339555734, 8)),
    ((0.3501217200000000, 0.2933306700000000, 0.4214709400000000),
     (5.0476033618933336, 7.0114568961277142, 6.451158237816637, 8)),
    ((0.3758966100000000, 0.2850717000000000, 0.6693404700000000),
     (5.8078232482982983, 8.5129934287980920, 10.441995222524335, 8)),
    ((0.4254993200000000, 0.2390417700000000, 0.3332903700000000),
     (5.0441061851453028, 6.3420359675892799, 18.425239270239359, 8)),
    ((0.3464176500000000, 0.2972505000000000, 0.3841176800000000),
     (5.4855084167238033, 6.7395106003336087, 5.474628963370974, 8)),
    ((0.4152160200000000, 0.2598912300000000, 0.3908615600000000),
     (5.8766492669350967, 6.7897767720171638, 15.440071996140499, 8)),
    ((0.3478004200000000, 0.2928404000000000, 0.0360562000000000),
     (8.1980634796784635, 2.2092985443242368, 1.999968831556617, 8)),
    ((0.4544551000000000, 0.1982224500000000, 0.0320179300000000),
     (7.9964595957086892, 2.0603132172126144, 11.656390753035632, 8)),
    ((0.3385874500000000, 0.3098545000000000, 0.7000400600000000),
     (7.9849282958267054, 8.6712830581905536, 4.270518902402659, 8)),
    ((0.3323961200000000, 0.3125182700000000, 0.1960473800000000),
     (9.5354740556904183, 5.0377553290689132, 1.999288540673044, 8)),
    ((0.4384618100000000, 0.2909638100000000, 0.2314123600000000),
     (9.9958803631122173, 5.4166120275490517, 11.058404052106437, 8)),
    ((0.4095802200000000, 0.2971922200000000, 0.4888287100000000),
     (9.3095533384450668, 7.4662030728919255, 11.186885768515551, 8)),
    ((0.4439989900000000, 0.2936950900000000, 0.4337968700000000),
     (9.9949745495327491, 7.0979059067934074, 13.900038696652488, 8)),
    ((0.4055491900000000, 0.2972301300000000, 0.1668776900000000),
     (9.9247647943179373, 4.6921921147267351, 7.197057639852876, 8)),
    ((0.4200700300000000, 0.2893081500000000, 0.1672933000000000),
     (9.4149775768693900, 4.6973688173207524, 8.721128174425862, 8)),
    ((0.5210832900000000, 0.2557414600000000, 0.1399952600000000),
     (0.0019220181363622, 4.3385094379603402, 16.467729969190529, 7)),
    ((0.3763801000000000, 0.3072800700000000, 0.3407028900000000),
     (0.0136223234250288, 6.4022012583755865, 6.594463952286430, 7)),
    ((0.3649530700000000, 0.3080148100000000, 0.2091091500000000),
     (0.0108592562153120, 5.1822080765947414, 4.548054950595445, 7)),
    ((0.4256691200000000, 0.2956401200000000, 0.2821793900000000),
     (9.9664422617946791, 5.9032094955646706, 10.839641908347603, 8)),
    ((0.5513634700000000, 0.2813889200000000, 0.1971219300000000),
     (2.4976580614381660, 5.0498581073710413, 17.884020184861189, 7)),
    ((0.5389941600000000, 0.2904878800000000, 0.2582363400000000),
     (2.7485546629604585, 5.6809766685439209, 17.995724599746377, 7)),
    ((0.4385481100000000, 0.3103317000000000, 0.2761236200000000),
     (2.4943078709206112, 5.8481154233099364, 10.235754640176427, 7)),
    ((0.3558906900000000, 0.3165537000000000, 0.2464947300000000),
     (2.8983979131373161, 5.5674143643020697, 3.554523683911656, 7)),
    ((0.4963159200000000, 0.3011119100000000, 0.0457050400000000),
     (5.4103686174523524, 2.5214548835517885, 7.258854844117816, 7)),
    ((0.6033835400000001, 0.2746834000000000, 0.0460021300000000),
     (5.8342027901144569, 2.5302731702838765, 12.000615206905717, 7)),
    ((0.5761977600000000, 0.3155471700000000, 0.1407335600000000),
     (5.9788280212108607, 4.3487706615362089, 14.392102528163205, 7)),
    ((0.4041454700000000, 0.3231072400000000, 0.1334788700000000),
     (5.1783763049447264, 4.2464126669886673, 4.842813896150843, 7)),
    ((0.3514372000000000, 0.3286069400000000, 0.8067969500000000),
     (8.3238088961026691, 9.1859534871888915, 4.236444337774098, 7)),
    ((0.5520033500000000, 0.3409058300000000, 0.1324052100000000),
     (8.1957437659127912, 4.2309654368964633, 11.592666113548890, 7)),
    ((0.4371923700000000, 0.3367305600000000, 0.1827476600000000),
     (7.6095498405871478, 4.8844510705007274, 6.793632746717810, 7)),
    ((0.3888005900000000, 0.3378369300000000, 0.7040874000000000),
     (8.2547548704792710, 8.6917863577621315, 6.342564639006561, 7)),
    ((0.4000601900000000, 0.3366314700000000, 0.0779026100000000),
     (9.9982747461203623, 3.3059508982469912, 3.115406955944987, 7)),
    ((0.6724836900000000, 0.3233036500000000, 0.0722064900000000),
     (9.7627074021020093, 3.1863606829924529, 14.800047578330392, 7)),
    ((0.6435491800000001, 0.3397363900000000, 0.0880312200000000),
     (9.9982818667900677, 3.5046891565568421, 13.900125103004391, 7)),
    ((0.3918136400000000, 0.3446948000000000, 0.7225225400000000),
     (0.0121464533192501, 8.7841366100324851, 6.215547341236711, 6)),
    ((0.6023106499999999, 0.3616884500000000, 0.1606819100000000),
     (9.9987262089904458, 4.6140459204878024, 14.686472574413189, 7)),
    ((0.5479115000000000, 0.3489291300000000, 0.0226378300000000),
     (2.7484465676051997, 1.6521000438004581, 5.568003028926484, 6)),
    ((0.4879418700000000, 0.3829320200000000, 0.2066566000000000),
     (3.1414206652130154, 5.1555158376170800, 8.596656843523602, 6)),
    ((0.4283673300000000, 0.3589172600000000, 0.1436553600000000),
     (2.4978474308952627, 4.3890476205890190, 4.951047519073632, 6)),
    ((0.4151889000000000, 0.3549530600000000, 0.1440804300000000),
     (2.4977653679963083, 4.3948638180430448, 4.383585180612334, 6)),
    ((0.4654240500000000, 0.3408257600000000, 0.0207925300000000),
     (2.4915569441282575, 1.5580251224422779, 3.308961030538007, 6)),
    ((0.4463724500000000, 0.3894542200000000, 0.6929833800000000),
     (5.0392323757189894, 8.6353345266290926, 9.593295926760944, 6)),
    ((0.4540413400000000, 0.3964715400000000, 0.6923362900000000),
     (5.5898251806057857, 8.6320252640015713, 10.198248046129890, 6)),
    ((0.4854625300000000, 0.4130592000000000, 0.5993629700000001),
     (5.9863485388822646, 8.1325317705551239, 12.306429102395787, 6)),
    ((0.4797842800000000, 0.3982598100000000, 0.2094078300000000),
     (5.4230242822957280, 5.1854456161550386, 8.046653489436829, 6)),
    ((0.4678597300000000, 0.3929712600000000, 0.2005390100000000),
     (5.3628014385081624, 5.0880776998143249, 7.341529902605595, 6)),
    ((0.5133190500000000, 0.4126596400000000, 0.2174703900000000),
     (5.6984660234683009, 5.2717939258175175, 10.326189160215149, 6)),
    ((0.4177048600000000, 0.3730374700000000, 0.2443073800000000),
     (5.4077385973960945, 5.5458983510218864, 5.292471393326970, 6)),
    ((0.3748181600000000, 0.3653678700000000, 0.7844277200000001),
     (8.2404764347274213, 9.0824007127074626, 4.900546438317654, 6)),
    ((0.3962015200000000, 0.3795104900000000, 0.6865797400000000),
     (8.2335502892855814, 8.6024898895588020, 5.942135876130584, 6)),
    ((0.5129922400000000, 0.4352892100000000, 0.1524116600000000),
     (8.3941734779753023, 4.5067363905693325, 9.461660902200202, 6)),
    ((0.5252912199999999, 0.4620586300000000, 0.2051258100000000),
     (9.9976788857335226, 5.1387570613223055, 12.705101930365435, 6)),
    ((0.3824910100000000, 0.3727125100000000, 0.2446699600000000),
     (9.9984253107088783, 5.5494729401082514, 3.506732159833540, 6)),
    ((0.5246552799999999, 0.4638175200000000, 0.2703702200000000),
     (9.9982936130158038, 5.7950904465362001, 14.064036258689498, 6)),
    ((0.3633282700000000, 0.3604262500000000, 0.4174790000000000),
     (9.9982298873025890, 6.9831232007774773, 3.128628040783353, 6)),
    ((0.4910378600000000, 0.4474178200000000, 0.3762533700000000),
     (9.9996397801166204, 6.6802047042100119, 11.629600879949830, 6)),
    ((0.5326067200000000, 0.4608579700000000, 0.3817587900000000),
     (9.0504958615431423, 6.7218006441915881, 17.087646825588862, 6)),
    ((0.3994763600000000, 0.3860710100000000, 0.3002481800000000),
     (9.9937932662019193, 6.0634615534921332, 4.737775043523384, 6)),
    ((0.4156675100000000, 0.3959083600000000, 0.4120339100000000),
     (9.4034002686824678, 6.9442036094507777, 6.288715586566071, 6)),
    ((0.3837995100000000, 0.3753382900000000, 0.4949983600000000),
     (9.6285300256909068, 7.5058275345570138, 4.623713090688181, 6)),
    ((0.5018481299999999, 0.4458796400000000, 0.4655662800000000),
     (9.0181705389692919, 7.3138793763343148, 13.422606463167975, 6)),
    ((0.5138197400000000, 0.4551261200000000, 0.4722297600000000),
     (9.2621126809280128, 7.3579949627042600, 15.233390464742534, 6)),
    ((0.4975896700000000, 0.4904961200000000, 0.4524221400000000),
     (3.3345190350913629, 7.2256790432797890, 16.113161767033556, 5)),
    ((0.4393501600000000, 0.4310345200000000, 0.2325963900000000),
     (2.4976979465295202, 5.4286630628146790, 6.543946978037446, 5)),
    ((0.3728552400000000, 0.3717802900000000, 0.0569098800000000),
     (2.5014987253563152, 2.8290724121722768, 2.002239254370286, 5)),
    ((0.4662588800000000, 0.4580535400000000, 0.6060447400000000),
     (2.5003392833265892, 8.1701201056514439, 12.020108503258743, 5)),
    ((0.4787799600000000, 0.4689326300000000, 0.7143770600000000),
     (2.4988626911278722, 8.7435404322303185, 14.428234223724066, 5)),
    ((0.4800986500000000, 0.4595566400000000, 0.1626376800000000),
     (2.4965520790199704, 4.6389139549495999, 8.380657446927768, 5)),
    ((0.3733485200000000, 0.3779745500000000, 0.1651552400000000),
     (3.3620190337138762, 4.6706516252868591, 2.775534906359987, 5)),
    ((0.4785617800000000, 0.5047532000000000, 0.5873242700000000),
     (5.3384876152333405, 8.0640948096578331, 16.611694509643459, 5)),
    ((0.4711968900000000, 0.4987682100000000, 0.6548302100000000),
     (5.3460599386324459, 8.4364175866715456, 15.412331133566334, 5)),
    ((0.4098395700000000, 0.4318670800000000, 0.4743050000000000),
     (5.3668129384921270, 7.3716537562112343, 7.038251111170072, 5)),
    ((0.4201830800000000, 0.4457513100000000, 0.4053410000000000),
     (5.9280307732079107, 6.8959269327086279, 7.572887338365732, 5)),
    ((0.4584119100000000, 0.4830471000000000, 0.3634053000000000),
     (5.7279389031851657, 6.5816608432402566, 10.668259627347156, 5)),
    ((0.4504086700000000, 0.4737104800000000, 0.3476686300000000),
     (5.6432925986164273, 6.4580191382703749, 9.548898590442330, 5)),
    ((0.4599038600000000, 0.4680793300000000, 0.0585787300000000),
     (5.3436234094102453, 2.8710977062882246, 5.430515721489820, 5)),
    ((0.3979342700000000, 0.4289480000000000, 0.1393012700000000),
     (7.7529577552993034, 4.3288327150737675, 4.269028839349022, 5)),
    ((0.4378990300000000, 0.4833768300000000, 0.1811031300000000),
     (8.1477743423848548, 4.8650216831032598, 7.545599812926251, 5)),
    ((0.4578588000000000, 0.5062392400000000, 0.2590149100000000),
     (7.9076312988106556, 5.6883950942156059, 10.778720186943521, 5)),
    ((0.3998500800000000, 0.4311645700000000, 0.2015219500000000),
     (7.5927891641099432, 5.0989975862006771, 4.937003269893032, 5)),
    ((0.3722696100000000, 0.3973812000000000, 0.2338171600000000),
     (7.6750849187559878, 5.4410490370532214, 3.553157956788445, 5)),
    ((0.3589998000000000, 0.3825095100000000, 0.3678898500000000),
     (7.9921807359007735, 6.6162954611683462, 3.285836544709042, 5)),
    ((0.3518070800000000, 0.3779808800000000, 0.5590428800000000),
     (9.7114794283318631, 7.8995777724463387, 3.357708715547692, 5)))

XYY_TO_MUNSELL_GREYS_SPECIFICATIONS = (
    ((0.31006, 0.31616, 0.00000000), 0.00000000000000000),
    ((0.31006, 0.31616, 0.00473582), 0.41666654855514407),
    ((0.31006, 0.31616, 0.00961944), 0.83333338141854485),
    ((0.31006, 0.31616, 0.01545756), 1.24999972063995890),
    ((0.31006, 0.31616, 0.02293343), 1.66666660762781160),
    ((0.31006, 0.31616, 0.03261914), 2.08333344968136870),
    ((0.31006, 0.31616, 0.04498800), 2.50000005875963180),
    ((0.31006, 0.31616, 0.06042690), 2.91666658331704460),
    ((0.31006, 0.31616, 0.07924864), 3.33333324629683950),
    ((0.31006, 0.31616, 0.10170428), 3.74999999845355920),
    ((0.31006, 0.31616, 0.12799549), 4.16666667734919830),
    ((0.31006, 0.31616, 0.15828689), 4.58333326357445880),
    ((0.31006, 0.31616, 0.19271844), 5.00000000000000000),
    ((0.31006, 0.31616, 0.23141772), 5.41666666793887240),
    ((0.31006, 0.31616, 0.27451233), 5.83333327891779780),
    ((0.31006, 0.31616, 0.32214224), 6.24999999244354140),
    ((0.31006, 0.31616, 0.37447210), 6.66666665220609200),
    ((0.31006, 0.31616, 0.43170362), 7.08333334036935370),
    ((0.31006, 0.31616, 0.49408790), 7.49999999749801920),
    ((0.31006, 0.31616, 0.56193781), 7.91666664651751530),
    ((0.31006, 0.31616, 0.63564030), 8.33333332730242480),
    ((0.31006, 0.31616, 0.71566876), 8.74999999354493770),
    ((0.31006, 0.31616, 0.80259539), 9.16666662209726190),
    ((0.31006, 0.31616, 0.89710353), 9.58333329997106450),
    ((0.31006, 0.31616, 1.00000000), 10.0000000000000000))

NON_CONVERGING_XYY = (
    (0.33159302, 0.43388935, 0.89380734),
    (0.25883700, 0.71096717, 0.01266934),
    (0.28517207, 0.38369148, 0.89445395),
    (0.28155768, 0.33248001, 0.89992977),
    (0.29624596, 0.31950269, 0.96665647),
    (0.13833192, 0.07953813, 0.02236117),
    (0.22907253, 0.06719948, 0.06903321))


class TestMunsellValuePriest1920(unittest.TestCase):
    """
    Defines :func:`colour.notation.munsell.munsell_value_Priest1920` definition
    unit tests methods.
    """

    def test_munsell_value_Priest1920(self):
        """
        Tests :func:`colour.notation.munsell.munsell_value_Priest1920`
        definition.
        """

        self.assertAlmostEqual(
            munsell_value_Priest1920(10.08),
            3.17490157328,
            places=7)

        self.assertAlmostEqual(
            munsell_value_Priest1920(56.76),
            7.53392328073,
            places=7)

        self.assertAlmostEqual(
            munsell_value_Priest1920(98.32),
            9.91564420499,
            places=7)

    def test_n_dimensional_munsell_value_Priest1920(self):
        """
        Tests :func:`colour.notation.munsell.munsell_value_Priest1920`
        definition n-dimensional arrays support.
        """

        Y = 10.08
        V = 3.17490157328
        np.testing.assert_almost_equal(
            munsell_value_Priest1920(Y),
            V,
            decimal=7)

        V = np.tile(V, 6)
        Y = np.tile(Y, 6)
        np.testing.assert_almost_equal(
            munsell_value_Priest1920(Y),
            V,
            decimal=7)

        V = np.reshape(V, (2, 3))
        Y = np.reshape(Y, (2, 3))
        np.testing.assert_almost_equal(
            munsell_value_Priest1920(Y),
            V,
            decimal=7)

        V = np.reshape(V, (2, 3, 1))
        Y = np.reshape(Y, (2, 3, 1))
        np.testing.assert_almost_equal(
            munsell_value_Priest1920(Y),
            V,
            decimal=7)

    @ignore_numpy_errors
    def test_nan_munsell_value_Priest1920(self):
        """
        Tests :func:`colour.notation.munsell.munsell_value_Priest1920`
        definition nan support.
        """

        munsell_value_Priest1920(
            np.array([-1.0, 0.0, 1.0, -np.inf, np.inf, np.nan]))


class TestMunsellValueMunsell1933(unittest.TestCase):
    """
    Defines :func:`colour.notation.munsell.munsell_value_Munsell1933`
    definition unit tests methods.
    """

    def test_munsell_value_Munsell1933(self):
        """
        Tests :func:`colour.notation.munsell.munsell_value_Munsell1933`
        definition.
        """

        self.assertAlmostEqual(
            munsell_value_Munsell1933(10.08),
            3.79183555086,
            places=7)

        self.assertAlmostEqual(
            munsell_value_Munsell1933(56.76),
            8.27013181776,
            places=7)

        self.assertAlmostEqual(
            munsell_value_Munsell1933(98.32),
            9.95457710587,
            places=7)

    def test_n_dimensional_munsell_value_Munsell1933(self):
        """
        Tests :func:`colour.notation.munsell.munsell_value_Munsell1933`
        definition n-dimensional arrays support.
        """

        Y = 10.08
        V = 3.79183555086
        np.testing.assert_almost_equal(
            munsell_value_Munsell1933(Y),
            V,
            decimal=7)

        V = np.tile(V, 6)
        Y = np.tile(Y, 6)
        np.testing.assert_almost_equal(
            munsell_value_Munsell1933(Y),
            V,
            decimal=7)

        V = np.reshape(V, (2, 3))
        Y = np.reshape(Y, (2, 3))
        np.testing.assert_almost_equal(
            munsell_value_Munsell1933(Y),
            V,
            decimal=7)

        V = np.reshape(V, (2, 3, 1))
        Y = np.reshape(Y, (2, 3, 1))
        np.testing.assert_almost_equal(
            munsell_value_Munsell1933(Y),
            V,
            decimal=7)

    @ignore_numpy_errors
    def test_nan_munsell_value_Munsell1933(self):
        """
        Tests :func:`colour.notation.munsell.munsell_value_Munsell1933`
        definition nan support.
        """

        munsell_value_Munsell1933(
            np.array([-1.0, 0.0, 1.0, -np.inf, np.inf, np.nan]))


class TestMunsellValueMoon1943(unittest.TestCase):
    """
    Defines :func:`colour.notation.munsell.munsell_value_Moon1943` definition
    unit tests methods.
    """

    def test_munsell_value_Moon1943(self):
        """
        Tests :func:`colour.notation.munsell.munsell_value_Moon1943`
        definition.
        """

        self.assertAlmostEqual(
            munsell_value_Moon1943(10.08),
            3.74629715382,
            places=7)

        self.assertAlmostEqual(
            munsell_value_Moon1943(56.76),
            7.8225814259,
            places=7)

        self.assertAlmostEqual(
            munsell_value_Moon1943(98.32),
            9.88538236116,
            places=7)

    def test_n_dimensional_munsell_value_Moon1943(self):
        """
        Tests :func:`colour.notation.munsell.munsell_value_Moon1943`
        definition n-dimensional arrays support.
        """

        Y = 10.08
        V = 3.74629715382
        np.testing.assert_almost_equal(
            munsell_value_Moon1943(Y),
            V,
            decimal=7)

        V = np.tile(V, 6)
        Y = np.tile(Y, 6)
        np.testing.assert_almost_equal(
            munsell_value_Moon1943(Y),
            V,
            decimal=7)

        V = np.reshape(V, (2, 3))
        Y = np.reshape(Y, (2, 3))
        np.testing.assert_almost_equal(
            munsell_value_Moon1943(Y),
            V,
            decimal=7)

        V = np.reshape(V, (2, 3, 1))
        Y = np.reshape(Y, (2, 3, 1))
        np.testing.assert_almost_equal(
            munsell_value_Moon1943(Y),
            V,
            decimal=7)

    @ignore_numpy_errors
    def test_nan_munsell_value_Moon1943(self):
        """
        Tests :func:`colour.notation.munsell.munsell_value_Moon1943`
        definition nan support.
        """

        munsell_value_Moon1943(
            np.array([-1.0, 0.0, 1.0, -np.inf, np.inf, np.nan]))


class TestMunsellValueSaunderson1944(unittest.TestCase):
    """
    Defines :func:`colour.notation.munsell.munsell_value_Saunderson1944`
    definition unit tests methods.
    """

    def test_munsell_value_Saunderson1944(self):
        """
        Tests :func:`colour.notation.munsell.munsell_value_Saunderson1944`
        definition.
        """

        self.assertAlmostEqual(
            munsell_value_Saunderson1944(10.08),
            3.68650805994,
            places=7)

        self.assertAlmostEqual(
            munsell_value_Saunderson1944(56.76),
            7.89881184275,
            places=7)

        self.assertAlmostEqual(
            munsell_value_Saunderson1944(98.32),
            9.85197100995,
            places=7)

    def test_n_dimensional_munsell_value_Saunderson1944(self):
        """
        Tests :func:`colour.notation.munsell.munsell_value_Saunderson1944`
        definition n-dimensional arrays support.
        """

        Y = 10.08
        V = 3.68650805994
        np.testing.assert_almost_equal(
            munsell_value_Saunderson1944(Y),
            V,
            decimal=7)

        V = np.tile(V, 6)
        Y = np.tile(Y, 6)
        np.testing.assert_almost_equal(
            munsell_value_Saunderson1944(Y),
            V,
            decimal=7)

        V = np.reshape(V, (2, 3))
        Y = np.reshape(Y, (2, 3))
        np.testing.assert_almost_equal(
            munsell_value_Saunderson1944(Y),
            V,
            decimal=7)

        V = np.reshape(V, (2, 3, 1))
        Y = np.reshape(Y, (2, 3, 1))
        np.testing.assert_almost_equal(
            munsell_value_Saunderson1944(Y),
            V,
            decimal=7)

    @ignore_numpy_errors
    def test_nan_munsell_value_Saunderson1944(self):
        """
        Tests :func:`colour.notation.munsell.munsell_value_Saunderson1944`
        definition nan support.
        """

        munsell_value_Saunderson1944(
            np.array([-1.0, 0.0, 1.0, -np.inf, np.inf, np.nan]))


class TestMunsellValueLadd1955(unittest.TestCase):
    """
    Defines :func:`colour.notation.munsell.munsell_value_Ladd1955` definition
    unit tests methods.
    """

    def test_munsell_value_Ladd1955(self):
        """
        Tests :func:`colour.notation.munsell.munsell_value_Ladd1955`
        definition.
        """

        self.assertAlmostEqual(
            munsell_value_Ladd1955(10.08),
            3.69528622419,
            places=7)

        self.assertAlmostEqual(
            munsell_value_Ladd1955(56.76),
            7.84875137062,
            places=7)

        self.assertAlmostEqual(
            munsell_value_Ladd1955(98.32),
            9.75492813681,
            places=7)

    def test_n_dimensional_munsell_value_Ladd1955(self):
        """
        Tests :func:`colour.notation.munsell.munsell_value_Ladd1955`
        definition n-dimensional arrays support.
        """

        Y = 10.08
        V = 3.69528622419
        np.testing.assert_almost_equal(
            munsell_value_Ladd1955(Y),
            V,
            decimal=7)

        V = np.tile(V, 6)
        Y = np.tile(Y, 6)
        np.testing.assert_almost_equal(
            munsell_value_Ladd1955(Y),
            V,
            decimal=7)

        V = np.reshape(V, (2, 3))
        Y = np.reshape(Y, (2, 3))
        np.testing.assert_almost_equal(
            munsell_value_Ladd1955(Y),
            V,
            decimal=7)

        V = np.reshape(V, (2, 3, 1))
        Y = np.reshape(Y, (2, 3, 1))
        np.testing.assert_almost_equal(
            munsell_value_Ladd1955(Y),
            V,
            decimal=7)

    @ignore_numpy_errors
    def test_nan_munsell_value_Ladd1955(self):
        """
        Tests :func:`colour.notation.munsell.munsell_value_Ladd1955`
        definition nan support.
        """

        munsell_value_Ladd1955(
            np.array([-1.0, 0.0, 1.0, -np.inf, np.inf, np.nan]))


class TestMunsellValueMcCamy1992(unittest.TestCase):
    """
    Defines :func:`colour.notation.munsell.munsell_value_McCamy1987` definition
    unit tests methods.
    """

    def test_munsell_value_McCamy1987(self):
        """
        Tests :func:`colour.notation.munsell.munsell_value_McCamy1987`
        definition.
        """

        self.assertAlmostEqual(
            munsell_value_McCamy1987(10.08),
            3.7347235258546316,
            places=7)

        self.assertAlmostEqual(
            munsell_value_McCamy1987(56.76),
            7.951693123402992,
            places=7)

        self.assertAlmostEqual(
            munsell_value_McCamy1987(98.32),
            9.935265378231314,
            places=7)

    def test_n_dimensional_munsell_value_McCamy1987(self):
        """
        Tests :func:`colour.notation.munsell.munsell_value_McCamy1987`
        definition n-dimensional arrays support.
        """

        Y = 10.08
        V = 3.7347235258546316
        np.testing.assert_almost_equal(
            munsell_value_McCamy1987(Y),
            V,
            decimal=7)

        V = np.tile(V, 6)
        Y = np.tile(Y, 6)
        np.testing.assert_almost_equal(
            munsell_value_McCamy1987(Y),
            V,
            decimal=7)

        V = np.reshape(V, (2, 3))
        Y = np.reshape(Y, (2, 3))
        np.testing.assert_almost_equal(
            munsell_value_McCamy1987(Y),
            V,
            decimal=7)

        V = np.reshape(V, (2, 3, 1))
        Y = np.reshape(Y, (2, 3, 1))
        np.testing.assert_almost_equal(
            munsell_value_McCamy1987(Y),
            V,
            decimal=7)

    @ignore_numpy_errors
    def test_nan_munsell_value_McCamy1987(self):
        """
        Tests :func:`colour.notation.munsell.munsell_value_McCamy1987`
        definition nan support.
        """

        munsell_value_McCamy1987(
            np.array([-1.0, 0.0, 1.0, -np.inf, np.inf, np.nan]))


class TestMunsellValueASTMD153508(unittest.TestCase):
    """
    Defines :func:`colour.notation.munsell.munsell_value_ASTMD153508`
    definition unit tests methods.
    """

    def test_munsell_value_ASTMD153508(self):
        """
        Tests :func:`colour.notation.munsell.munsell_value_ASTMD153508`
        definition.
        """

        self.assertAlmostEqual(
            munsell_value_ASTMD153508(10.08),
            3.7344764769311354,
            places=7)

        self.assertAlmostEqual(
            munsell_value_ASTMD153508(56.76),
            7.94992421224809,
            places=7)

        self.assertAlmostEqual(
            munsell_value_ASTMD153508(98.32),
            9.934372853115544,
            places=7)

    def test_n_dimensional_munsell_value_ASTMD153508(self):
        """
        Tests :func:`colour.notation.munsell.munsell_value_ASTMD153508`
        definition n-dimensional arrays support.
        """

        Y = 10.08
        V = 3.7344764769311354
        np.testing.assert_almost_equal(
            munsell_value_ASTMD153508(Y),
            V,
            decimal=7)

        V = np.tile(V, 6)
        Y = np.tile(Y, 6)
        np.testing.assert_almost_equal(
            munsell_value_ASTMD153508(Y),
            V,
            decimal=7)

        V = np.reshape(V, (2, 3))
        Y = np.reshape(Y, (2, 3))
        np.testing.assert_almost_equal(
            munsell_value_ASTMD153508(Y),
            V,
            decimal=7)

        V = np.reshape(V, (2, 3, 1))
        Y = np.reshape(Y, (2, 3, 1))
        np.testing.assert_almost_equal(
            munsell_value_ASTMD153508(Y),
            V,
            decimal=7)

    @ignore_numpy_errors
    def test_nan_munsell_value_ASTMD153508(self):
        """
        Tests :func:`colour.notation.munsell.munsell_value_ASTMD153508`
        definition nan support.
        """

        munsell_value_ASTMD153508(
            np.array([-1.0, 0.0, 1.0, -np.inf, np.inf, np.nan]))


class TestMunsellSpecification_to_xyY(unittest.TestCase):
    """
    Defines :func:`colour.notation.munsell.munsell_specification_to_xyY`
    definition unit tests methods.
    """

    def test_munsell_specification_to_xyY(self):
        """
        Tests :func:`colour.notation.munsell.munsell_specification_to_xyY`
        definition.
        """

        for i, specification in enumerate(MUNSELL_SPECIFICATIONS):
            np.testing.assert_almost_equal(
                munsell_specification_to_xyY(specification),
                np.array(MUNSELL_COLOURS_TO_XYY[i]),
                decimal=7)

        for i, specification in enumerate(MUNSELL_GREYS_SPECIFICATIONS):
            np.testing.assert_almost_equal(
                munsell_specification_to_xyY(specification),
                np.array(MUNSELL_GREYS_TO_XYY[i]),
                decimal=7)


class TestMunsellColour_to_xyY(unittest.TestCase):
    """
    Defines :func:`colour.notation.munsell.munsell_colour_to_xyY` definition
    unit tests methods.
    """

    def test_munsell_colour_to_xyY(self):
        """
        Tests :func:`colour.notation.munsell.munsell_colour_to_xyY` definition.
        """

        # TODO: This test is covered by the previous class,
        # do we need a dedicated one?

        pass


class TestxyY_to_munsell_specification(unittest.TestCase):
    """
    Defines :func:`colour.notation.munsell.xyY_to_munsell_specification`
    definition unit tests methods.
    """

    def test_xyY_to_munsell_specification(self):
        """
        Tests :func:`colour.notation.munsell.xyY_to_munsell_specification`
        definition.
        """

        for xyY, specification in XYY_TO_MUNSELL_SPECIFICATIONS:
            np.testing.assert_almost_equal(
                xyY_to_munsell_specification(xyY),
                specification,
                decimal=7)

        for xyY in NON_CONVERGING_XYY:
            self.assertRaises((RuntimeError, ValueError),
                              xyY_to_munsell_specification, xyY)

        for xyY, specification in XYY_TO_MUNSELL_GREYS_SPECIFICATIONS:
            np.testing.assert_almost_equal(
                xyY_to_munsell_specification(xyY),
                specification,
                decimal=7)


class TestxyY_to_munsell_colour(unittest.TestCase):
    """
    Defines :func:`colour.notation.munsell.xyY_to_munsell_colour` definition
    unit tests methods.
    """

    def test_xyY_to_munsell_colour(self):
        """
        Tests :func:`colour.notation.munsell.xyY_to_munsell_colour` definition.
        """

        # TODO: This test is covered by the previous class,
        # do we need a dedicated one?

        pass


class TestParseMunsellColour(unittest.TestCase):
    """
    Defines :func:`colour.notation.munsell.parse_munsell_colour` definition
    unit tests methods.
    """

    def test_parse_munsell_colour(self):
        """
        Tests :func:`colour.notation.munsell.is_grey_munsell_colour`
        definition.
        """

        self.assertEqual(parse_munsell_colour('N5.2'), 5.2)

        self.assertTupleEqual(parse_munsell_colour('0YR 2.0/4.0'),
                              (0.0, 2.0, 4.0, 6))

        self.assertTupleEqual(parse_munsell_colour('4.2YR 8.1/5.3'),
                              (4.2, 8.1, 5.3, 6))


class TestIsGreyMunsellColour(unittest.TestCase):
    """
    Defines :func:`colour.notation.munsell.is_grey_munsell_colour` definition
    unit tests methods.
    """

    def test_is_grey_munsell_colour(self):
        """
        Tests :func:`colour.notation.munsell.is_grey_munsell_colour`
        definition.
        """

        self.assertTrue(is_grey_munsell_colour(5.2))

        self.assertFalse(is_grey_munsell_colour((0.0, 2.0, 4.0, 6)))

        self.assertFalse(is_grey_munsell_colour((4.2, 8.1, 5.3, 6)))


class TestNormalizeMunsellSpecification(unittest.TestCase):
    """
    Defines :func:`colour.notation.munsell.normalize_munsell_specification`
    definition unit tests methods.
    """

    def test_normalize_munsell_specification(self):
        """
        Tests :func:`colour.notation.munsell.normalize_munsell_specification`
        definition.
        """

        self.assertTupleEqual(
            normalize_munsell_specification((0.0, 2.0, 4.0, 6)),
            (10.0, 2.0, 4.0, 7))

        self.assertTupleEqual(
            normalize_munsell_specification((0.0, 2.0, 4.0, 8)),
            (10.0, 2.0, 4.0, 9))

        self.assertTupleEqual(
            normalize_munsell_specification((0, 2.0, 4.0, 10)),
            (10.0, 2.0, 4.0, 1))

        self.assertEqual(
            normalize_munsell_specification((0, 2.0, 0, 10)),
            2)


class TestMunsellColourToMunsellSpecification(unittest.TestCase):
    """
    Defines :func:`colour.notation.munsell.\
munsell_colour_to_munsell_specification` definition unit tests methods.
    """

    def test_munsell_colour_to_munsell_specification(self):
        """
        Tests :func:`colour.notation.munsell.\
munsell_colour_to_munsell_specification` definition.
        """

        self.assertTupleEqual(
            munsell_colour_to_munsell_specification('0.0YR 2.0/4.0'),
            (10., 2.0, 4.0, 7))

        self.assertTupleEqual(
            munsell_colour_to_munsell_specification('0.0RP 2.0/4.0'),
            (10.0, 2.0, 4.0, 9))

        self.assertTupleEqual(
            munsell_colour_to_munsell_specification('10.0B 2.0/4.0'),
            (10.0, 2.0, 4.0, 1))

        self.assertEqual(munsell_colour_to_munsell_specification('N5.2'),
                         5.2)
        self.assertEqual(
            munsell_colour_to_munsell_specification('0.0YR 2.0/0.0'),
            2)


class TestMunsellSpecificationToMunsellColour(unittest.TestCase):
    """
    Defines :func:`colour.notation.munsell.\
munsell_specification_to_munsell_colour` definition unit tests methods.
    """

    def test_munsell_specification_to_munsell_colour(self):
        """
        Tests :func:`colour.notation.munsell.\
munsell_specification_to_munsell_colour` definition.
        """

        self.assertEqual(
            munsell_specification_to_munsell_colour((10, 2.0, 4.0, 7)),
            '10.0R 2.0/4.0')

        self.assertEqual(
            munsell_specification_to_munsell_colour((10.0, 2.0, 4.0, 9)),
            '10.0P 2.0/4.0')

        self.assertEqual(
            munsell_specification_to_munsell_colour((10.0, 2.0, 4.0, 1)),
            '10.0B 2.0/4.0')

        self.assertEqual(munsell_specification_to_munsell_colour(5.2),
                         'N5.2')


class Test_xyY_fromRenotation(unittest.TestCase):
    """
    Defines :func:`colour.notation.munsell.xyY_from_renotation` definition
    unit tests methods.
    """

    def test_xyY_from_renotation(self):
        """
        Tests :func:`colour.notation.munsell.xyY_from_renotation`
        definition.
        """

        self.assertTupleEqual(xyY_from_renotation((2.5, 0.2, 2.0, 4)),
                              (0.713, 1.414, 0.237))

        self.assertTupleEqual(xyY_from_renotation((5.0, 0.2, 2.0, 4)),
                              (0.449, 1.145, 0.237))

        self.assertTupleEqual(xyY_from_renotation((7.5, 0.2, 2.0, 4)),
                              (0.262, 0.837, 0.237))


class TestIsSpecificationInRenotation(unittest.TestCase):
    """
    Defines :func:`colour.notation.munsell.is_specification_in_renotation`
    definition unit tests methods.
    """

    def test_is_specification_in_renotation(self):
        """
        Tests :func:`colour.notation.munsell.is_specification_in_renotation`
        definition.
        """

        self.assertTrue(is_specification_in_renotation((2.5, 0.2, 2.0, 4)))

        self.assertTrue(is_specification_in_renotation((5.0, 0.2, 2.0, 4)))

        self.assertFalse(is_specification_in_renotation((25.0, 0.2, 2.0, 4)))


class TestBoundingHuesFromRenotation(unittest.TestCase):
    """
    Defines :func:`colour.notation.munsell.bounding_hues_from_renotation`
    definition unit tests methods.
    """

    def test_bounding_hues_from_renotation(self):
        """
        Tests :func:`colour.notation.munsell.bounding_hues_from_renotation`
        definition.
        """

        for i, specification in enumerate(MUNSELL_SPECIFICATIONS):
            hue, _value, _chroma, code = specification
            self.assertTupleEqual(bounding_hues_from_renotation(hue, code),
                                  MUNSELL_BOUNDING_HUES[i])


class TestHueToHueAngle(unittest.TestCase):
    """
    Defines :func:`colour.notation.munsell.hue_to_hue_angle` definition units
    tests methods.
    """

    def test_hue_to_hue_angle(self):
        """
        Tests :func:`colour.notation.munsell.hue_to_hue_angle` definition.
        """

        for hue, code, angle in MUNSELL_HUE_TO_ANGLE:
            self.assertEqual(hue_to_hue_angle(hue, code), angle)


class TestHueAngleToHue(unittest.TestCase):
    """
    Defines :func:`colour.notation.munsell.hue_angle_to_hue` definition units
    tests methods.
    """

    def test_hue_angle_to_hue(self):
        """
        Tests :func:`colour.notation.munsell.hue_angle_to_hue` definition.
        """

        for hue, code, angle in MUNSELL_HUE_TO_ANGLE:
            self.assertEqual(hue_angle_to_hue(angle), (hue, code))


class TestHueTo_ASTM_hue(unittest.TestCase):
    """
    Defines :func:`colour.notation.munsell.hue_to_ASTM_hue` definition units
    tests methods.
    """

    def test_hue_to_ASTM_hue(self):
        """
        Tests :func:`colour.notation.munsell.hue_to_ASTM_hue` definition.
        """

        for hue, code, angle in MUNSELL_HUE_TO_ASTM_HUE:
            self.assertEqual(hue_to_ASTM_hue(hue, code), angle)


class TestInterpolationMethodFromRenotationOvoid(unittest.TestCase):
    """
    Defines :func:`colour.notation.munsell.\
interpolation_method_from_renotation_ovoid` definition unit tests methods.
    """

    def test_interpolation_method_from_renotation_ovoid(self):
        """
        Tests :func:`colour.notation.munsell.\
interpolation_method_from_renotation_ovoid` definition.
        """

        for i, specification in enumerate(MUNSELL_EVEN_SPECIFICATIONS):
            self.assertEqual(
                interpolation_method_from_renotation_ovoid(specification),
                MUNSELL_INTERPOLATION_METHODS[i])


class Test_xy_fromRenotationOvoid(unittest.TestCase):
    """
    Defines :func:`colour.notation.munsell.xy_from_renotation_ovoid` definition
    unit tests methods.
    """

    def test_xy_from_renotation_ovoid(self):
        """
        Tests :func:`colour.notation.munsell.xy_from_renotation_ovoid`
        definition.
        """

        for i, specification in enumerate(MUNSELL_EVEN_SPECIFICATIONS):
            if is_specification_in_renotation(specification):
                self.assertEqual(xy_from_renotation_ovoid(specification),
                                 MUNSELL_XY_FROM_RENOTATION_OVOID[i])


class TestLCHabToMunsellSpecification(unittest.TestCase):
    """
    Defines :func:`colour.notation.munsell.LCHab_to_munsell_specification`
    definition unit tests methods.
    """

    def test_LCHab_to_munsell_specification(self):
        """
        Tests :func:`colour.notation.munsell.LCHab_to_munsell_specification`
        definition.
        """

        self.assertTupleEqual(
            LCHab_to_munsell_specification(
                np.array([100.00000000, 21.57210357, 272.22819350])),
            (5.6189426388888819, 10.0, 4.3144207139999997, 10))

        self.assertTupleEqual(
            LCHab_to_munsell_specification(
                np.array([100.00000000, 426.67945353, 72.39590835])),
            (0.10997454166666643, 10.0, 85.335890706000001, 5))

        self.assertTupleEqual(
            LCHab_to_munsell_specification(
                np.array([100.00000000, 74.05216981, 276.45318193])),
            (6.7925505361111194, 10.0, 14.810433961999999, 10))


class TestMaximumChromaFromRenotation(unittest.TestCase):
    """
    Defines :func:`colour.notation.munsell.maximum_chroma_from_renotation`
    definition unit tests methods.
    """

    def test_maximum_chroma_from_renotation(self):
        """
        Tests :func:`colour.notation.munsell.maximum_chroma_from_renotation`
        definition.
        """

        self.assertEqual(maximum_chroma_from_renotation(2.5, 5, 5),
                         14.0)

        self.assertEqual(maximum_chroma_from_renotation(8.675, 1.225, 10),
                         48.0)

        self.assertEqual(maximum_chroma_from_renotation(6.875, 3.425, 1),
                         16.0)


class TestMunsellSpecification_to_xy(unittest.TestCase):
    """
    Defines :func:`colour.notation.munsell.munsell_specification_to_xy`
    definition unit tests methods.
    """

    def test_munsell_specification_to_xy(self):
        """
        Tests :func:`colour.notation.munsell.munsell_specification_to_xy`
        definition.
        """

        for specification, xy in MUNSELL_SPECIFICATIONS_TO_XY:
            np.testing.assert_almost_equal(
                munsell_specification_to_xy(specification),
                xy,
                decimal=7)

        for specification in MUNSELL_GREYS_SPECIFICATIONS:
            self.assertEqual(munsell_specification_to_xy(specification),
                             (0.31006, 0.31616))


if __name__ == '__main__':
    unittest.main()
